Package: ggfortify
Type: Package
Title: Data Visualization Tools for Statistical Analysis Results
Version: 0.4.22
Date: 2026-08-20
Authors@R: c(
  person("Masaaki", "Horikoshi", role = c("aut"),
         email = "sinhrks@gmail.com"),
  person("Yuan", "Tang", role = c("aut", "cre"),
         email = "terrytangyuan@gmail.com",
         comment = c(ORCID = "0000-0001-5243-233X")),
  person("Austin", "Dickey", role = c("ctb")),
  person("Matthias", "Grenié", role = c("ctb")),
  person("Ryan", "Thompson", role = c("ctb")),
  person("Luciano", "Selzer", role = c("ctb")),
  person("Dario", "Strbenac", role = c("ctb")),
  person("Kirill", "Voronin", role = c("ctb")),
  person("Damir", "Pulatov", role = c("ctb")),
  person("Emik", "Lin", role = c("ctb"))
  )
Maintainer: Yuan Tang <terrytangyuan@gmail.com>
URL: https://github.com/sinhrks/ggfortify
BugReports: https://github.com/sinhrks/ggfortify/issues
Encoding: UTF-8
Description: Unified plotting tools for statistics commonly used, such as GLM,
    time series, PCA families, clustering and survival analysis. The package offers
    a single plotting interface for these analysis results and plots in a unified
    style using 'ggplot2'.
License: MIT + file LICENSE
Depends: methods, ggplot2 (>= 2.0.0)
Imports: dplyr (>= 0.3), tidyr, gridExtra, grid, scales, stringr,
        tibble
Suggests: testthat, cluster, changepoint, fGarch, forecast, ggrepel,
        glmnet, grDevices, KFAS, knitr, lintr, mapdata, markdown, MASS,
        MSwM, nlme, raster, ROCR, sp, stats, strucchange, survival,
        timeSeries, tseries, utils, vars, xts, zoo, lfda
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-08-21 06:21:55 UTC; emiklin
Author: Masaaki Horikoshi [aut],
  Yuan Tang [aut, cre] (ORCID: <https://orcid.org/0000-0001-5243-233X>),
  Austin Dickey [ctb],
  Matthias Grenié [ctb],
  Ryan Thompson [ctb],
  Luciano Selzer [ctb],
  Dario Strbenac [ctb],
  Kirill Voronin [ctb],
  Damir Pulatov [ctb],
  Emik Lin [ctb]
Repository: CRAN
Date/Publication: 2026-08-24 08:30:20 UTC
Built: R 4.6.0; ; 2026-08-24 10:36:38 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(autoplot,Arima)
S3method(autoplot,HoltWinters)
S3method(autoplot,KFS)
S3method(autoplot,Line)
S3method(autoplot,Lines)
S3method(autoplot,MSM.lm)
S3method(autoplot,Polygon)
S3method(autoplot,Polygons)
S3method(autoplot,RasterBrick)
S3method(autoplot,RasterCommon)
S3method(autoplot,RasterLayer)
S3method(autoplot,RasterStack)
S3method(autoplot,SpatialCommon)
S3method(autoplot,SpatialLines)
S3method(autoplot,SpatialLinesDataFrame)
S3method(autoplot,SpatialPoints)
S3method(autoplot,SpatialPointsDataFrame)
S3method(autoplot,SpatialPolygons)
S3method(autoplot,SpatialPolygonsDataFrame)
S3method(autoplot,aareg)
S3method(autoplot,acf)
S3method(autoplot,ar)
S3method(autoplot,basis)
S3method(autoplot,bats)
S3method(autoplot,breakpoints)
S3method(autoplot,clara)
S3method(autoplot,cpt)
S3method(autoplot,cv.glmnet)
S3method(autoplot,decomposed.ts)
S3method(autoplot,density)
S3method(autoplot,dist)
S3method(autoplot,dlmFiltered)
S3method(autoplot,ets)
S3method(autoplot,fGARCH)
S3method(autoplot,factanal)
S3method(autoplot,fanny)
S3method(autoplot,forecast)
S3method(autoplot,fracdiff)
S3method(autoplot,ggmultiplot)
S3method(autoplot,ggplot)
S3method(autoplot,glm)
S3method(autoplot,glmnet)
S3method(autoplot,irts)
S3method(autoplot,kmeans)
S3method(autoplot,lfda)
S3method(autoplot,list)
S3method(autoplot,lm)
S3method(autoplot,map)
S3method(autoplot,matrix)
S3method(autoplot,nnetar)
S3method(autoplot,partition)
S3method(autoplot,pca_common)
S3method(autoplot,performance)
S3method(autoplot,prcomp)
S3method(autoplot,princomp)
S3method(autoplot,silhouette)
S3method(autoplot,spec)
S3method(autoplot,stepfun)
S3method(autoplot,stl)
S3method(autoplot,survfit)
S3method(autoplot,timeSeries)
S3method(autoplot,ts)
S3method(autoplot,tsmodel)
S3method(autoplot,varprd)
S3method(autoplot,xts)
S3method(autoplot,zooreg)
S3method(fitted,ar)
S3method(fortify,Arima)
S3method(fortify,HoltWinters)
S3method(fortify,KFS)
S3method(fortify,MSM.lm)
S3method(fortify,RasterBrick)
S3method(fortify,RasterCommon)
S3method(fortify,RasterLayer)
S3method(fortify,RasterStack)
S3method(fortify,SpatialCommon)
S3method(fortify,SpatialLines)
S3method(fortify,SpatialPoints)
S3method(fortify,SpatialPointsDataFrame)
S3method(fortify,aareg)
S3method(fortify,acf)
S3method(fortify,ar)
S3method(fortify,basis)
S3method(fortify,bats)
S3method(fortify,breakpoints)
S3method(fortify,breakpointsfull)
S3method(fortify,clara)
S3method(fortify,cpt)
S3method(fortify,cv.glmnet)
S3method(fortify,decomposed.ts)
S3method(fortify,density)
S3method(fortify,dist)
S3method(fortify,dlmFiltered)
S3method(fortify,ets)
S3method(fortify,fGARCH)
S3method(fortify,factanal)
S3method(fortify,fanny)
S3method(fortify,forecast)
S3method(fortify,fracdiff)
S3method(fortify,glmnet)
S3method(fortify,irts)
S3method(fortify,kmeans)
S3method(fortify,lfda)
S3method(fortify,list)
S3method(fortify,matrix)
S3method(fortify,nnetar)
S3method(fortify,pam)
S3method(fortify,partition)
S3method(fortify,performance)
S3method(fortify,prcomp)
S3method(fortify,princomp)
S3method(fortify,silhouette)
S3method(fortify,spec)
S3method(fortify,stepfun)
S3method(fortify,stl)
S3method(fortify,survfit)
S3method(fortify,table)
S3method(fortify,timeSeries)
S3method(fortify,ts)
S3method(fortify,tsmodel)
S3method(fortify,varprd)
S3method(grid.draw,ggmultiplot)
S3method(residuals,ar)
export(fortify_map)
export(ggbiplot)
export(ggcpgram)
export(ggdistribution)
export(ggfreqplot)
export(ggtsdiag)
export(rbind_ts)
export(unscale)
exportClasses(ggmultiplot)
import(dplyr)
import(ggplot2)
import(methods)
importFrom(grid,grid.draw)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(scales,percent)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,predict)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,weights)
importFrom(stringr,str_wrap)
importFrom(tibble,as_tibble)
importFrom(tidyr,gather)
importFrom(utils,installed.packages)

#' Convert \code{sp} instances to \code{data.frame}.
#'
#' @param model \code{sp} instances
#' @inheritParams fortify_base
#' @param rename logical flag indicating whether to rename coordinates to long and lat
#' @return data.frame
#' @export
fortify.SpatialCommon <- function(model, data = NULL,
                                  rename = TRUE, ...) {
  if (is(model, 'SpatialPoints') ||
      is(model, 'SpatialPointsDataFrame')) {
    df <- as.data.frame(model)

  } else if (is(model, 'SpatialLines')) {
    dfs <- lapply(model@lines, fortify)
    suppressWarnings(
      # bind_row against factors with different levels
      df <- do.call(dplyr::bind_rows, dfs)
    )
    df[['piece']] <- as.factor(df[['piece']])
    df[['group']] <- as.factor(df[['group']])
  } else {
    # Line, Lines, SpatialLinesDataFrame are defined in ggplot2
    stop(paste0('Unsupported class for fortify.SpatialCommon: ', class(model)))
  }

  if (rename) {
    if (!is(model, 'SpatialLines')) {
      # SpatialLines are renamed in lapply
      coords <- colnames(sp::coordinates(model))
      names(coords) <- c('long', 'lat')
      df <- dplyr::rename(df, coords)
      }
  }
  post_fortify(df, klass = model)
}

#' @export
fortify.SpatialPoints <- fortify.SpatialCommon

#' @export
fortify.SpatialPointsDataFrame <- fortify.SpatialCommon

#' @export
fortify.SpatialLines <- fortify.SpatialCommon

# followings are defined in ggplot2
# fortify.Line, fortify.Lines, SpatialLinesDataFrame, fortify.Polygon
# fortify.Polygons, fortify.SpatialPolygons, fortify.SpatialPolygonsDataFrame

#' Autoplot \code{maps::map}
#'
#' @param object \code{maps::map} instance
#' @param p \code{ggplot2::ggplot} instance
#' @param group key for grouping geoms
#' @param colour line colour
#' @param size point size
#' @param linetype line type
#' @param alpha alpha
#' @param fill fill colour
#' @param shape point shape
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @export
autoplot.SpatialCommon <- function(object, p = NULL, group = NULL,
                                   colour = 'black', size = NULL, linetype = NULL,
                                   alpha = NULL, fill = NULL, shape = NULL,
                                   xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                                   main = NULL, xlab = '', ylab = '', asp = NULL,
                                   ...) {

  plot.data <- ggplot2::fortify(object)

  if (is_derived_from(object, 'SpatialPoints') ||
      is_derived_from(object, 'SpatialPointsDataFrame')) {
    geomfunc <- ggplot2::geom_point
  } else if (is_derived_from(object, 'Polygon') ||
             is_derived_from(object, 'Polygons') ||
             is_derived_from(object, 'SpatialPolygons') ||
             is_derived_from(object, 'SpatialPolygonsDataFrame')) {
    # Polygon must be checked before Line
    geomfunc <- ggplot2::geom_polygon
  } else if (is_derived_from(object, 'Line') ||
             is_derived_from(object, 'Lines') ||
             is_derived_from(object, 'SpatialLines') ||
             is_derived_from(object, 'SpatialLinesDataFrame')) {
    geomfunc <- ggplot2::geom_line
    if (is_derived_from(object, 'Lines') ||
        is_derived_from(object, 'SpatialLines') ||
        is_derived_from(object, 'SpatialLinesDataFrame')) {
      # when line contains multiple segments
      group <- 'group'
    }
  } else {
    stop(paste0('Unsupported class for autoplot.SpatialCommon: ', class(object)))
  }
  if (is_derived_from(object, 'Polygons') ||
      is_derived_from(object, 'SpatialPolygons') ||
      is_derived_from(object, 'SpatialPolygonsDataFrame') ||
      is_derived_from(object, 'Lines') ||
      is_derived_from(object, 'SpatialLines') ||
      is_derived_from(object, 'SpatialLinesDataFrame')) {
      # when object contains multiple segments
    group <- 'group'
  }
  if (is_derived_from(object, 'SpatialPolygonsDataFrame') ||
      is_derived_from(object, 'SpatialLinesDataFrame')) {
    df2 <- as.data.frame(object)
    df2[['id']] <- rownames(df2)
    plot.data <- dplyr::inner_join(plot.data, df2, by = 'id')
  }

  # create ggplot instance if not passed
  if (is.null(p)) {
    mapping <- ggplot2::aes_string(x = 'long', y = 'lat')
    p <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    # ToDo: allow data to be none. ggplot instance should be
    # passed to factory to check its data
    p <- p + geom_factory(geomfunc, data = plot.data, group = group,
                          colour = colour, size = size, linetype = linetype,
                          alpha = alpha, fill = fill, shape = shape)
  } else {
    p <- p + geom_factory(geomfunc, data = plot.data, x = 'long', y = 'lat',
                          group = group, colour = colour, size = size,
                          linetype = linetype,
                          alpha = alpha, fill = fill, shape = shape)
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' @export
autoplot.SpatialPointsDataFrame <- autoplot.SpatialCommon

#' @export
autoplot.SpatialPoints <- autoplot.SpatialCommon

#' @export
autoplot.Line <- autoplot.SpatialCommon

#' @export
autoplot.Lines <- autoplot.SpatialCommon

#' @export
autoplot.SpatialLines <- autoplot.SpatialCommon

#' @export
autoplot.SpatialLinesDataFrame <- autoplot.SpatialCommon

#' @export
autoplot.Polygon <- autoplot.SpatialCommon

#' @export
autoplot.Polygons <- autoplot.SpatialCommon

#' @export
autoplot.SpatialPolygons <- autoplot.SpatialCommon

#' @export
autoplot.SpatialPolygonsDataFrame <- autoplot.SpatialCommon

## ----global_options, include=FALSE--------------------------------------------
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/ts-', warning=FALSE)
hasDep <- require("zoo", quietly = TRUE) && require("forecast", quietly = TRUE) && require("timeSeries", quietly = TRUE) && require("KFAS", quietly = TRUE) && require("changepoint", quietly = TRUE) && require("forecast", quietly = TRUE)

## ----message = FALSE, eval=hasDep---------------------------------------------
library(ggfortify)
autoplot(AirPassengers)

## ----eval=hasDep--------------------------------------------------------------
autoplot(AirPassengers, ts.colour = 'red', ts.linetype = 'dashed')

## ----message = FALSE, eval=hasDep---------------------------------------------
library(vars)
data(Canada)
autoplot(Canada)

## ----eval=hasDep--------------------------------------------------------------
autoplot(Canada, facets = FALSE)

## ----message = FALSE, eval = FALSE--------------------------------------------
#  library(xts)
#  autoplot(as.xts(AirPassengers), ts.colour = 'green')
#  
#  library(timeSeries)
#  autoplot(as.timeSeries(AirPassengers), ts.colour = ('dodgerblue3'))

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(AirPassengers, ts.geom = 'bar', fill = 'blue')

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(AirPassengers, ts.geom = 'ribbon', fill = 'green')

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(AirPassengers, ts.geom = 'point', shape = 3)

## ----message = FALSE, eval=hasDep---------------------------------------------
mts <- ts(data.frame(a = c(1, 2, 3, 4, 4, 3), b = c(3, 2, 3, 2, 2, 1)), start = 2010)
autoplot(mts, ts.geom = 'bar', facets = FALSE)

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(mts, ts.geom = 'bar', facets = FALSE, stacked = TRUE)

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(mts, ts.geom = 'ribbon', facets = FALSE)

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(mts, ts.geom = 'ribbon', facets = FALSE, stacked = TRUE)

## ----message = FALSE, eval=hasDep---------------------------------------------
library(forecast)
d.arima <- auto.arima(AirPassengers)
d.forecast <- forecast(d.arima, level = c(95), h = 50)
autoplot(d.forecast)

## ----eval=hasDep--------------------------------------------------------------
autoplot(d.forecast, ts.colour = 'firebrick1', predict.colour = 'red',
         predict.linetype = 'dashed', conf.int = FALSE)

## ----eval=hasDep--------------------------------------------------------------
library(vars)
d.vselect <- VARselect(Canada, lag.max = 5, type = 'const')$selection[1]
d.var <- VAR(Canada, p = d.vselect, type = 'const')

## ----eval=hasDep--------------------------------------------------------------
autoplot(predict(d.var, n.ahead = 50), ts.colour = 'dodgerblue4',
         predict.colour = 'blue', predict.linetype = 'dashed')

## ----message = FALSE, eval=hasDep---------------------------------------------
library(changepoint)
autoplot(cpt.meanvar(AirPassengers))

## ----eval=hasDep--------------------------------------------------------------
autoplot(cpt.meanvar(AirPassengers), cpt.colour = 'blue', cpt.linetype = 'solid')

## ----message = FALSE, eval=hasDep---------------------------------------------
library(strucchange)
autoplot(breakpoints(Nile ~ 1), ts.colour = 'blue', ts.linetype = 'dashed',
         cpt.colour = 'dodgerblue3', cpt.linetype = 'solid')

## ----message=FALSE, warning=FALSE, eval=hasDep--------------------------------
library(KFAS)
model <- SSModel(
  Nile ~ SSMtrend(degree=1, Q=matrix(NA)), H=matrix(NA)
)
 
fit <- fitSSM(model=model, inits=c(log(var(Nile)),log(var(Nile))), method="BFGS")
smoothed <- KFS(fit$model)
autoplot(smoothed)

## ----eval=hasDep--------------------------------------------------------------
filtered <- KFS(fit$model, filtering="mean", smoothing='none')
autoplot(filtered)

## ----eval=hasDep--------------------------------------------------------------
trend <- signal(smoothed, states="trend")
class(trend)

## ----eval=hasDep--------------------------------------------------------------
p <- autoplot(filtered)
autoplot(trend, ts.colour = 'blue', p = p)

## ----message = FALSE, eval=hasDep---------------------------------------------
autoplot(stl(AirPassengers, s.window = 'periodic'), ts.colour = 'blue')

## ----fig.width = 4, fig.height = 2, eval=hasDep-------------------------------
autoplot(acf(AirPassengers, plot = FALSE))

## ----fig.width = 4, fig.height = 2, eval=hasDep-------------------------------
autoplot(acf(AirPassengers, plot = FALSE), conf.int.fill = '#0000FF', conf.int.value = 0.8, conf.int.type = 'ma')

## ----fig.width = 4, fig.height = 2, eval=hasDep-------------------------------
autoplot(spec.ar(AirPassengers, plot = FALSE))

## ----message = FALSE, fig.width=3, fig.height=3, eval=hasDep------------------
ggcpgram(arima.sim(list(ar = c(0.7, -0.5)), n = 50))

## ----message = FALSE, fig.width = 5, fig.height = 5, eval=hasDep--------------
library(forecast)
ggtsdiag(auto.arima(AirPassengers))

## ----message = FALSE, fig.width = 4, fig.height = 4, eval=hasDep--------------
ggfreqplot(AirPassengers)
ggfreqplot(AirPassengers, freq = 4)

#' Backtransform \code{scale}-ed object
#'
#' @param data Scaled data
#' @param center Centered vector
#' @param scale Scale vector
#' @return data.frame
#' @examples
#' df <- iris[-5]
#' ggfortify::unscale(base::scale(df))
#' @export
unscale <- function(data, center = NULL, scale = NULL) {
  if (is.null(scale)) {
    scale <- attr(data, 'scaled:scale')
  }
  if (is.null(center)) {
    center <- attr(data, 'scaled:center')
  }
  if (!is.null(scale) && !is.logical(scale)) {
    data <- base::scale(data, center = FALSE, scale = 1 / scale)
  }
  if (!is.null(center) && !is.logical(center)) {
    data <- base::scale(data, center = -center, scale = FALSE)
  }
  as.data.frame(data)
}

#' Wrapper for cbind
#'
#' @param df1 1st data
#' @param df2 2nd data
#' @return list
#' @examples
#' ggfortify:::cbind_wraps(iris[1:2], iris[3:5])
cbind_wraps <- function(df1, df2) {
  if (is.null(df1)) {
    return(df2)
  } else if (!is.data.frame(df1)) {
    df1 <- ggplot2::fortify(df1)
  }
  if (is.null(df2)) {
    return(df1)
  } else if (!is.data.frame(df2)) {
    df2 <- ggplot2::fortify(df2)
  }
  # prioritize df1 columns
  dots <- names(df2)[! colnames(df2) %in% colnames(df1)]
  if (length(dots) != length(colnames(df2))) {
    df2 <- dplyr::select(df2, all_of(dots))
  }
  return(cbind(df1, df2))
}


#' Check object is target class, or object is \code{data.frame} fortified from target.
#'
#' @param object instance to be checked. For data.frame, check whether it is fortified from target class
#' @param target class name
#' @return logical
#' @examples
#' ggfortify:::is_derived_from(prcomp(iris[-5]), 'prcomp')
is_derived_from <- function(object, target) {
  if (is(object, target)) {
    return(TRUE)
  } else {
    # 'base_class' attr is only added by autoplot.aareg for now
    atr <- attr(object, 'base_class')
    # this null check should be separeted to avoid logical(0)
    if (is.null(atr)) {
      return(FALSE)
    }
    return(atr == target)
  }
  return(FALSE)
}


#' Show deprecate warning
#'
#' @param old.kw Keyword being deprecated
#' @param new.kw Keyword being replaced
#' @return NULL
#' @examples
#' ggfortify:::deprecate.warning('old', 'new')
deprecate.warning <- function(old.kw, new.kw) {
  message <- paste0("Argument '", old.kw, "' is being deprecated. Use '", new.kw, "' instead.")
  warning(message, call. = FALSE)
}

#' Flatten dataframe contains matrix
#'
#' tains list or matrix as column
#'
#' @param df \code{data.frame} to be flatten
flatten <- function(df) {
  ismatrix <- vapply(df, is.matrix, logical(1))
  if (any(ismatrix)) {
    return(data.frame(c(df[!ismatrix], do.call(data.frame, df[ismatrix])),
                      stringsAsFactors = FALSE))
  } else {
    return(df)
  }
}

#' Post process for fortify.
#'
#' @param data data.frame
#' @param klass instance to be added as base_class attr, should be original model before fortified
#' @return data.frame
post_fortify <- function(data, klass = NULL) {
  if (is(data, 'tbl_df')) {
    data <- as.data.frame(data)
  }
  if (!is.data.frame(data)) {
    stop('data must be a data.frame')
  }
  if (!is.null(klass)) {
    attr(data, 'base_class') <- class(klass)
  }
  as.data.frame(data)
}

context('test util')

test_that('unscale', {
  df <- iris[c(1, 2, 3, 4)]
  expect_equal(ggfortify::unscale(scale(df)), df)
  expect_equal(ggfortify::unscale(scale(df, center = FALSE)), df)
  expect_equal(ggfortify::unscale(scale(df, scale = FALSE)), df)
  expect_equal(ggfortify::unscale(scale(df, center = FALSE, scale = FALSE)), df)

  scaled <- scale(df)
  center <- attr(scaled, 'scaled:center')
  scale <- attr(scaled, 'scaled:scale')
  expect_equal(ggfortify::unscale(scaled, center = center, scale = scale), df)

  scaled <- scale(df, center = FALSE)
  scale <- attr(scaled, 'scaled:scale')
  expect_equal(ggfortify::unscale(scaled, center = FALSE, scale = scale), df)

  scaled <- scale(df, scale = FALSE)
  center <- attr(scaled, 'scaled:center')
  expect_equal(ggfortify::unscale(scaled, center = center, scale = FALSE), df)

  scaled <- scale(df, center = FALSE, scale = FALSE)
  expect_equal(ggfortify::unscale(scaled, center = FALSE, scale = FALSE), df)
})

test_that('post_fortify', {
  df <- data.frame(x = c(1, 2, 3), y = c(4, 5, 6))
  res <- ggfortify:::post_fortify(df)
  expect_equal(res, df)

  tbl <- dplyr::tibble(x = c(1, 2, 3), y = c(4, 5, 6))
  expect_true(is(tbl, 'tbl_df'))
  res <- ggfortify:::post_fortify(tbl)
  expect_equal(res, df)
  expect_true(!is(res, 'tbl_df'))

  res <- ggfortify:::post_fortify(tbl, klass = as.ts(tbl))
  expect_equal(res$x, df$x)
  expect_equal(res$y, df$y)
  expect_true('mts' %in% attr(res, 'base_class'))
})

test_that('remove_conflicting_columns works correctly', {
  # Test basic functionality
  df1 <- data.frame(a = 1:3, b = 4:6, c = 7:9)
  df2 <- data.frame(b = 10:12, d = 13:15)
  df3 <- data.frame(c = 16:18, e = 19:21)

  result <- remove_conflicting_columns(df1, df2, df3)

  # Should only keep column 'a' since 'b' and 'c' conflict
  expect_equal(names(result), "a")
  expect_equal(result$a, df1$a)
  expect_equal(nrow(result), nrow(df1))
  expect_equal(rownames(result), rownames(df1))
})

test_that('remove_conflicting_columns handles NULL data', {
  df1 <- data.frame(a = 1:3, b = 4:6)

  # NULL data should return NULL
  expect_null(remove_conflicting_columns(NULL, df1))

  # NULL in ... should be ignored
  result <- remove_conflicting_columns(df1, NULL, data.frame(c = 1:3))
  expect_equal(names(result), c("a", "b"))
  expect_equal(result, df1)
})

test_that('remove_conflicting_columns handles no conflicts', {
  df1 <- data.frame(a = 1:3, b = 4:6)
  df2 <- data.frame(c = 7:9, d = 10:12)

  result <- remove_conflicting_columns(df1, df2)

  # No conflicts, should return original data
  expect_equal(result, df1)
  expect_equal(names(result), names(df1))
})

test_that('remove_conflicting_columns handles all columns conflicting', {
  df1 <- data.frame(a = 1:3, b = 4:6, row.names = paste0("row", 1:3))
  df2 <- data.frame(a = 7:9, b = 10:12)

  result <- remove_conflicting_columns(df1, df2)

  # All columns conflict, should return empty data frame with same row structure
  expect_equal(ncol(result), 0)
  expect_equal(nrow(result), 0)  # The [FALSE, , drop = FALSE] creates 0 rows
  expect_true(is.data.frame(result))
})

test_that('remove_conflicting_columns handles no other dataframes', {
  df1 <- data.frame(a = 1:3, b = 4:6)

  result <- remove_conflicting_columns(df1)

  # No other dataframes, should return original
  expect_equal(result, df1)
})

test_that('remove_conflicting_columns handles matrices via fortify', {
  df1 <- data.frame(a = 1:3, b = 4:6, c = 7:9)
  mat1 <- matrix(10:15, ncol = 2, dimnames = list(NULL, c("b", "x")))
  mat2 <- matrix(16:21, ncol = 2, dimnames = list(NULL, c("c", "y")))

  result <- remove_conflicting_columns(df1, mat1, mat2)
  expect_equal(names(result), "a")
  expect_equal(result$a, df1$a)

  result <- remove_conflicting_columns(mat1, df1)
  expect_equal(names(result), "x")
  expect_equal(result$x, mat1[,"x"])
})

test_that('remove_conflicting_columns preserves row names and structure', {
  df1 <- data.frame(a = 1:3, b = 4:6, c = 7:9, row.names = paste0("sample_", 1:3))
  df2 <- data.frame(b = 10:12, x = 13:15)

  result <- remove_conflicting_columns(df1, df2)

  # Should preserve row names
  expect_equal(rownames(result), rownames(df1))
  expect_equal(names(result), c("a", "c"))
  expect_equal(result$a, df1$a)
  expect_equal(result$c, df1$c)
})

test_that('remove_conflicting_columns handles partial conflicts', {
  df1 <- data.frame(PC1 = 1:4, PC2 = 5:8, Species = letters[1:4], extra = 9:12)
  df2 <- data.frame(PC1 = 101:104, PC3 = 105:108)  # Only PC1 conflicts

  result <- remove_conflicting_columns(df1, df2)

  # Should remove only PC1, keep PC2, Species, extra
  expect_equal(names(result), c("PC2", "Species", "extra"))
  expect_equal(result$PC2, df1$PC2)
  expect_equal(result$Species, df1$Species)
  expect_equal(result$extra, df1$extra)
})

test_that('remove_conflicting_columns handles multiple conflicting dataframes', {
  df1 <- data.frame(a = 1:3, b = 4:6, c = 7:9, d = 10:12, e = 13:15)
  df2 <- data.frame(a = 101:103, x = 104:106)
  df3 <- data.frame(b = 201:203, y = 204:206)
  df4 <- data.frame(c = 301:303, z = 304:306)

  result <- remove_conflicting_columns(df1, df2, df3, df4)

  # Should remove a, b, c but keep d, e
  expect_equal(names(result), c("d", "e"))
  expect_equal(result$d, df1$d)
  expect_equal(result$e, df1$e)
})

test_that('remove_conflicting_columns handles empty dataframes', {
  df1 <- data.frame(a = 1:3, b = 4:6)
  df_empty <- data.frame()

  result <- remove_conflicting_columns(df1, df_empty)

  # Empty dataframe has no column names, so no conflicts
  expect_equal(result, df1)
})

test_that('remove_conflicting_columns preserves data types', {
  df1 <- data.frame(
    numeric_col = 1:3,
    character_col = c("a", "b", "c"),
    logical_col = c(TRUE, FALSE, TRUE),
    factor_col = factor(c("x", "y", "z")),
    stringsAsFactors = FALSE
  )
  df2 <- data.frame(conflicting_col = 4:6)

  result <- remove_conflicting_columns(df1, df2)

  # Should preserve all original columns and their types
  expect_equal(result, df1)
  expect_equal(sapply(result, class), sapply(df1, class))
})

test_that('remove_conflicting_columns handles mixed object types via fortify', {
  df1 <- data.frame(PC1 = 1:4, PC2 = 5:8, Species = letters[1:4])
  mat1 <- matrix(101:108, ncol = 2, dimnames = list(NULL, c("PC1", "NewCol")))
  mat2 <- matrix(201:208, ncol = 2, dimnames = list(NULL, c("PC2", "AnotherCol")))

  result <- remove_conflicting_columns(df1, mat1, mat2)
  expect_equal(names(result), "Species")
  expect_equal(result$Species, df1$Species)

  result <- remove_conflicting_columns(mat1, mat2, df1)
  expect_equal(names(result), "NewCol")
  expect_equal(result$NewCol, mat1[, "NewCol"])

})

test_that('remove_conflicting_columns handles matrices without column names', {
  df1 <- data.frame(a = 1:3, b = 4:6)
  mat_no_names <- matrix(1:6, ncol = 2)  # No column names

  result <- remove_conflicting_columns(df1, mat_no_names)

  # Matrix without column names should not cause conflicts
  expect_equal(result, df1)
})

test_that('remove_conflicting_columns handles edge case with duplicate column names in ...', {
  df1 <- data.frame(a = 1:3, b = 4:6, c = 7:9)
  df2 <- data.frame(a = 10:12, x = 13:15)
  df3 <- data.frame(a = 16:18, y = 19:21)  # 'a' appears in both df2 and df3

  result <- remove_conflicting_columns(df1, df2, df3)

  # Should remove 'a' only once, keep 'b' and 'c'
  expect_equal(names(result), c("b", "c"))
  expect_equal(result$b, df1$b)
  expect_equal(result$c, df1$c)
})

#' Convert \code{maps::map} to \code{data.frame}.
#'
#' @param model \code{maps::map} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @export
fortify_map <- function(model, data = NULL, ...) {
  # ggplot2::fortify.map cannot handle maps retrieved by fill = FALSE properly
  df <- as.data.frame(model[c("x", "y")])
  names(df) <- c("long", "lat")
  df$group <- cumsum(is.na(df$long) & is.na(df$lat)) + 1
  df$order <- 1:nrow(df)
  names <- do.call("rbind", lapply(strsplit(model$names, "[:,]"), "[", 1:2))
  if (max(df$group) <= length(names)) {
    df$region <- names[df$group, 1]
    df$subregion <- names[df$group, 2]
  } else {
    df$region <- df$group
    df$subregion <- NA
  }
  df <- df[complete.cases(df$lat, df$long), ]
  post_fortify(df)
}

#' Autoplot \code{maps::map}
#'
#' @param object \code{maps::map} instance
#' @param p \code{ggplot2::ggplot} instance
#' @param geom geometric string for map. 'path', 'point' or 'polygon'
#' @param group key for grouping geoms
#' @param colour line colour
#' @param size point size
#' @param linetype line type
#' @param alpha alpha
#' @param fill fill colour
#' @param shape point shape
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @export
autoplot.map <- function(object, p = NULL, geom = 'path', group = 'group',
                         colour = 'black', size = NULL, linetype = NULL,
                         alpha = NULL, fill = NULL, shape = NULL,
                         xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                         main = NULL, xlab = '', ylab = '', asp = NULL,
                         ...) {
  geomfunc <- get_geom_function(geom, allowed = c('path', 'point', 'polygon'))
  plot.data <- fortify_map(object)

  # create ggplot instance if not passed
  if (is.null(p)) {
    mapping <- ggplot2::aes_string(x = 'long', y = 'lat')
    p <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    # ToDo: allow data to be none. ggplot instance should be
    # passed to factory to check its data
    p <- p + geom_factory(geomfunc, data = plot.data, group = group,
                          colour = colour, size = size, linetype = linetype,
                          alpha = alpha, fill = fill, shape = shape)
  } else {
    p <- p + geom_factory(geomfunc, data = plot.data,
                          x = 'long', y = 'lat', group = group,
                          colour = colour, size = size, linetype = linetype,
                          alpha = alpha, fill = fill, shape = shape)
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert a spline basis to a tibble
#'
#' @param x object of class "basis"
#' @param ... Ignored.
#' @return A tibble constructed from the underlying matrix of the
#'     basis object. Each column will possess all the attributes from
#'     the source object, except that the "class" attribute will be
#'     renamed to "basis.class" to avoid interfering with dplyr
#'     operations.
#' @details This function is needed because the default method for
#'     converting a matrix object with an additional class attribute
#'     to a tibble causes issues because each column of the resulting
#'     tibble has the attributes, including the matrix class, copied
#'     from the source. Having matrices as columns in a tibble causes
#'     dplyr to throw errors, so a special method is needed to avoid
#'     copying the class attribute.
#' @examples
#' library(splines)
#' library(tibble)
#' x <- seq(0, 1, by=0.001)
#' spl <- bs(x, df=6)
#' as_tibble(spl)
#' @importFrom tibble as_tibble
as_tibble.basis <- function(x, ...) {
    attr(x, "basis.class") <- attr(x, "class")
    as_tibble(unclass(x))
}

#' Convert spline basis instances to \code{data.frame}
#'
#' @param model spline basis object
#' @param data x-values at which to evaluate the splines. Optional. By
#'     default, an evenly spaced sequence of 256 values covering the
#'     range of the splines will be used.
#' @param n If data is not provided, instead use an evenly-spaced
#'     sequence of x-values of this length (plus one, since both
#'     endpoints are included). If data is provided, this argument is
#'     ignored.
#' @inheritParams fortify_base
#' @return data.frame with 3 columns: Spline (character), x (numeric),
#'     and y (numeric); giving the interpolated x and y values for
#'     each of the splines in the basis.
#' @examples
#' library(splines)
#' x <- seq(0, 1, by=0.001)
#' spl <- bs(x, df=6)
#' fortify(spl)
#' @importFrom tibble as_tibble
#' @importFrom stats predict
#' @export
fortify.basis <- function(model, data, n=256, ...) {
    attrs <- attributes(model)
    bounds <- attrs$Boundary.knots
    if (missing(data)) {
        ## Add 1 to length.out because both endpoints are included
        data <- seq(from=bounds[1], to=bounds[2], length.out=n + 1)
    }
    predict(model, data) %>%
        dplyr::as_tibble() %>%
        dplyr::mutate(x=data) %>%
        tidyr::pivot_longer(colnames(model), names_to="Spline", values_to="y") %>%
        dplyr::arrange(Spline, x) %>%
        dplyr::select(Spline, x, y)
}

#' Autoplot spline basis instances
#'
#' @param object spline basis object
#' @param data x-values at which to evaluate the splines. Optional. By
#'     default, an evenly spaced sequence of 256 values covering the
#'     range of the splines will be used.
#' @param n If data is not provided, instead use an evenly-spaced
#'     sequence of x-values of this length (plus one, since both
#'     endpoints are included). If data is provided, this argument is
#'     ignored.
#' @param ... Ignored.
#' @return ggplot
#' @examples
#' library(splines)
#' x <- seq(0, 1, by=0.001)
#' spl <- bs(x, df=6)
#' autoplot(spl)
#' autoplot(spl, n=5)
#' @export
autoplot.basis <- function(object, data, n=256, ...) {
    fortified <- ggplot2::fortify(object, data, n)
    all.knots <- c(attr(object, "Boundary.knots"),
                   attr(object, "knots")) %>%
        unname %>% sort
    knot.df <- ggplot2::fortify(object,
                                data=all.knots)
    ggplot(fortified) +
        aes_string(x="x", y="y", group="Spline", color="Spline") +
        geom_line() +
        geom_point(data=knot.df) +
        scale_color_discrete(guide=FALSE)
}

#' Convert list to data.frame
#'
#' @param model \code{list} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @export
fortify.list <- function(model, data = NULL, ...) {
  klass <- infer(model)
  if (klass == 'mds-like') {
    return(ggplot2::fortify(model$points))
  } else if (klass == 'dlmSmooth') {
    s <- dlm::dropFirst(model$s)
    if (!is.univariate(s, raise = FALSE)) {
      s <- s[, 1]
    }
    return(ggplot2::fortify(s))
  } else if (klass == 'KFASSignal') {
    return(ggplot2::fortify(model$signal))
  }
  stop('Unable to infer class from input list')
}

#' Autoplot list
#'
#' @param object \code{list} instance
#' @param data original dataset, if needed
#' @param ... other arguments passed to methods
#' @inheritParams apply_facets
#' @return ggplot
#' @export
autoplot.list <- function(object, data = NULL,
                          nrow = NULL, ncol = NULL, scales = 'free_y',
                          ...) {
  if (length(object) ==0) {
    stop('list length = 0, contains nothing')
  }

  klass <- infer(object)
  if (klass == 'mds-like') {
    return(ggplot2::autoplot(object$points[, 1:2], geom = 'point', ...))
  } else if (klass == 'dlmSmooth') {
    s <- dlm::dropFirst(object$s)
    if (!is.univariate(s, raise = FALSE)) {
      s <- s[, 1]
    }
    return(ggplot2::autoplot(s, ...))
  } else if (klass == 'KFASSignal') {
    return(ggplot2::autoplot(object$signal, ...))
  }

  # if model is a list of a single class instances, try to plot them with facets
  if (all(sapply(object, support_autoplot))) {

    p <- lapply(object, function(x) autoplot(x, data = data, nrow = nrow,
                                             ncol = ncol, scales = scales, ...))
    if (is(p[[1]], 'ggmultiplot')) {
      p <- unlist(lapply(p, function(x) x@plots), recursive = FALSE)
    }
    # set default
    if (is.null(ncol)) { ncol <- 0 }
    if (is.null(nrow)) { nrow <- 0 }
    p <- new('ggmultiplot', plots = p, nrow = nrow, ncol = ncol)
    return(p)
  }

  stop('Unable to infer class from input list')
}

#' Infer class name
#'
#' @param data list instance
#' @return character
infer <- function(data) {
  if (check_names(data, c('points', 'eig', 'x', 'ac', 'GOF'))) {
    # cmdscale
    return('mds-like')
  } else if (check_names(data, c('points', 'stress'))) {
    # isoMDS
    return('mds-like')
  } else if (check_names(data, c('points', 'stress', 'call'))) {
    # sammon
    return('mds-like')
  } else if (check_names(data, c('s', 'U.S', 'D.S'))) {
    # dlm::dlmSmooth
    return('dlmSmooth')
  } else if (check_names(data, c('signal', 'variance'))) {
    # KFAS::signal
    return('KFASSignal')
  } else {
    return('list')
  }
}

#' Check data names are equal with expected
#'
#' @param data \code{list} instance to be checked
#' @param expected expected character vector
#' @return logical
check_names <- function(data, expected) {
  n <- names(data)
  if (length(n) == length(expected) && all(n == expected)) {
    return(TRUE)
  }
  return (FALSE)
}

context('test base-infer')

test_that('infer works for MDS-likes', {
  skip_on_cran()
  skip_on_travis()
  # MDS
  data(eurodist)
  expect_equal(infer(cmdscale(eurodist, eig = TRUE)), 'mds-like')
  expect_equal(infer(cmdscale(eurodist, add = TRUE)), 'mds-like')
  expect_equal(infer(cmdscale(eurodist, x.ret = TRUE)), 'mds-like')
  library(MASS)
  expect_equal(infer(isoMDS(eurodist)), 'mds-like')
  expect_equal(infer(sammon(eurodist)), 'mds-like')
})

test_that('fortify works for MDS-likes', {
  skip_on_cran()
  skip_on_travis()
  # MDS
  data(eurodist)
  fortified <- fortify(cmdscale(eurodist, eig = TRUE))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('1', '2'))
  expect_equal(nrow(fortified), 21)

  fortified <- fortify(cmdscale(eurodist, x.ret = TRUE))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('1', '2'))
  expect_equal(nrow(fortified), 21)

  fortified <- fortify(cmdscale(eurodist, k = 4, add = TRUE))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('1', '2', '3', '4'))
  expect_equal(nrow(fortified), 21)

  library(MASS)
  expect_equal(infer(isoMDS(eurodist)), 'mds-like')
  fortified <- fortify(cmdscale(eurodist, k = 3, add = TRUE))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('1', '2', '3'))
  expect_equal(nrow(fortified), 21)

  expect_equal(infer(sammon(eurodist)), 'mds-like')
  fortified <- fortify(cmdscale(eurodist, k = 3, add = TRUE))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('1', '2', '3'))
  expect_equal(nrow(fortified), 21)
})

test_that('infer, fortify and autoplot works for dlm::dlmSmooth', {
  skip_on_cran()
  skip_on_travis()
  skip_if_not_installed("dlm")
  nile_fortified <- fortify(Nile)

  library(dlm)
  form <- function(theta){
    dlm::dlmModPoly(order=1, dV=exp(theta[1]), dW=exp(theta[2]))
  }
  model <- form(dlm::dlmMLE(Nile, parm=c(1, 1), form)$par)
  filtered <- dlm::dlmFilter(Nile, model)
  fortified <- fortify(filtered)
  expect_equal(colnames(fortified), c('Index', 'Data', 'Fitted', 'Residuals'))
  expect_equal(fortified$Index, nile_fortified$Index)

  smoothed <- dlm::dlmSmooth(filtered)
  expect_equal(infer(smoothed), 'dlmSmooth')

  fortified <- fortify(smoothed)
  expect_equal(colnames(fortified), c('Index', 'Data'))
  expect_equal(fortified$Index, nile_fortified$Index)

  autoplot(filtered)
  autoplot(smoothed)

  form <- function(parm){
    dlmModPoly(order = 2, dV = exp(parm[1]), dW = exp(parm[2:3]))
  }
  y <- log(Nile)
  model <- form(dlm::dlmMLE(y, parm=log(c(var(y), 0.001, 0.001)), form)$par)
  parm <- log(c(var(y), 0.001, 0.001))

  filtered <- dlm::dlmFilter(y, model)
  fortified <- fortify(filtered)
  expect_equal(colnames(fortified), c('Index', 'Data', 'Fitted', 'Residuals'))
  expect_equal(fortified$Index, nile_fortified$Index)

  smoothed <- dlm::dlmSmooth(filtered)
  expect_equal(infer(smoothed), 'dlmSmooth')

  fortified <- fortify(smoothed)
  expect_equal(colnames(fortified), c('Index', 'Data'))
  expect_equal(fortified$Index, nile_fortified$Index)

  autoplot(filtered)
  autoplot(smoothed)
})

test_that('infer, fortify and autoplot works for KFAS::signal', {
  skip_on_cran()
  skip_on_travis()
  skip_if_not_installed("KFAS")
  nile_fortified <- fortify(Nile)

  library(KFAS)
  model <- SSModel(
    Nile ~ SSMtrend(degree=1, Q=matrix(NA)), H=matrix(NA)
  )
  fit <- fitSSM(model=model, inits=c(log(var(Nile)), log(var(Nile))), method="BFGS")

  smoothed <- KFS(fit$model)
  fortified <- fortify(smoothed)
  expect_equal(colnames(fortified), c('Index', 'Data', 'Fitted', 'Residuals'))
  expect_equal(fortified$Index, nile_fortified$Index)

  filtered <- KFS(fit$model, filtering="mean", smoothing='none')
  fortified <- fortify(filtered)
  expect_equal(colnames(fortified), c('Index', 'Data', 'Fitted', 'Residuals'))
  expect_equal(fortified$Index, nile_fortified$Index)

  trend <- signal(smoothed, states="trend")
  expect_equal(infer(trend), 'KFASSignal')

  fortified <- fortify(trend)
  expect_equal(colnames(fortified), c('Index', 'Data'))
  expect_equal(fortified$Index, nile_fortified$Index)

  p <- autoplot(filtered)
  expect_true(inherits(p, 'ggplot'))

  p <- autoplot(smoothed)
  expect_true(inherits(p, 'ggplot'))

  p <- autoplot(trend)
  expect_true(inherits(p, 'ggplot'))
})

#' Convert \code{survival::survfit} to \code{data.frame}
#'
#' @param model \code{survival::survfit} instance
#' @param surv.connect logical frag indicates whether connects survival curve to the origin
#' @param fun an arbitrary function defining a transformation of the survival curve
#' @inheritParams fortify_base
#' @return data.frame
#' @aliases fortify.survfit.cox
#' @examples
#' library(survival)
#' fortify(survfit(Surv(time, status) ~ sex, data = lung))
#' fortify(survfit(Surv(time, status) ~ 1, data = lung))
#' fortify(survfit(coxph(Surv(time, status) ~ sex, data = lung)))
#' fortify(survfit(coxph(Surv(time, status) ~ 1, data = lung)))
#' @export
fortify.survfit <- function(model, data = NULL, surv.connect = FALSE,
                            fun = NULL, ...) {
  d <- data.frame(time = model$time,
                  n.risk = model$n.risk,
                  n.event = model$n.event,
                  n.censor = model$n.censor,
                  std.err = model$std.err,
                  upper = model$upper,
                  lower = model$lower)

  if (is(model, 'survfit.cox')) {
    d <- cbind_wraps(d, data.frame(surv = model$surv, cumhaz = model$cumhaz))
  } else if (is(model, 'survfit')) {
    if (is(model, 'survfitms')) {
      d <- cbind_wraps(d, data.frame(pstate = model$pstate))

      varying.names <- c('n.risk', 'n.event', 'pstate', 'std.err', 'upper', 'lower')
      varying.i <- lapply(varying.names, function(x) which(startsWith(colnames(d), x)))
      d <- reshape(d, varying = varying.i, v.names = varying.names, timevar = NULL, direction = 'long')
      d <- subset(d, select = -c(id))
      rownames(d) <- NULL

      if (length(model$states) > 1) {
        ev.names <- model$states
        ev.names[ev.names == ''] <- 'any'
        ev <- factor(rep(ev.names, each = length(model$time)), levels = ev.names)
        d <- cbind_wraps(d, data.frame(event = ev))
      }
    } else {
      d <- cbind_wraps(d, data.frame(surv = model$surv))
    }

    if ('strata' %in% names(model)) {
      groupIDs <- gsub("[^,]*=", '', names(model$strata))
      groupIDs <- factor(rep(groupIDs, model$strata), levels = groupIDs)
      if ('states' %in% names(model)) {
        groupIDs <- rep(groupIDs, length(model$states))
      }
      d <- cbind_wraps(d, data.frame(strata = groupIDs))
    }
  } else {
    stop(paste0('Unsupported class for fortify.survfit: ', class(model)))
  }

  # connect to the origin for plotting
  if (surv.connect) {
    base <- d[1, ]
    # cumhaz is for survfit.cox cases
    base[intersect(c('time', 'n.event', 'n.censor', 'std.err', 'cumhaz'), colnames(base))] <- 0
    if ('pstate' %in% colnames(d)) {
      base[c('pstate', 'upper', 'lower')] <- 0
    } else {
      base[c('surv', 'upper', 'lower')] <- 1.0
    }
    if ('strata' %in% colnames(d)) {
      strata <- levels(d$strata)
      base <- base[rep(seq_len(nrow(base)), length(strata)), ]
      rownames(base) <- NULL
      base$strata <- strata
      base$strata <- factor(base$strata, levels = base$strata)
    }
    if ('event' %in% colnames(d)) {
      events <- levels(d$event)
      base <- base[rep(seq_len(nrow(base)), length(events)), ]
      rownames(base) <- NULL
      base$event <- events
      base$event <- factor(base$event, levels = events)
      base[base$event == 'any', c('pstate', 'upper', 'lower')] <- 1.0
    }
    d <- rbind(base, d)
  }

  if (!is.null(fun)) {
    if (is.character(fun)) {
      fun <- switch(fun, log = function(x) log(x),
                     event = function(x) 1 - x,
                     cumhaz = function(x) -log(x),
                     cloglog = function(x) log(-log(x)),
                     pct = function(x) x * 100,
                     logpct = function(x) 100 * x,
                     identity = function(x) x,
                     stop("Unrecognized function argument"))
    }
    else if (!is.function(fun)) {
      stop("Invalid 'fun' argument")
    }
    d$surv <- fun(d$surv)
    d$upper <- fun(d$upper)
    d$lower <- fun(d$lower)
  }

  d <- d[, intersect(c(
    'time', 'n.risk', 'n.event', 'n.censor',
    'surv', 'pstate',
    'std.err', 'upper', 'lower',
    'strata', 'event',
    'cumhaz'
  ), colnames(d))]

  post_fortify(d)
}

#' Autoplot \code{survival::survfit}
#'
#' @param object \code{survival::survfit} instance
#' @param fun an arbitrary function defining a transformation of the survival curve
#' @param surv.geom geometric string for survival curve. 'step', 'line' or 'point'
#' @param surv.colour line colour for survival curve
#' @param surv.size point size for survival curve
#' @param surv.linetype line type for survival curve
#' @param surv.alpha alpha for survival curve
#' @param surv.fill fill colour survival curve
#' @param surv.shape point shape survival curve
#' @inheritParams fortify.survfit
#' @inheritParams plot_confint
#' @param censor Logical flag indicating whether to plot censors
#' @param censor.colour colour for censors
#' @param censor.size size for censors
#' @param censor.alpha alpha for censors
#' @param censor.shape shape for censors
#' @inheritParams apply_facets
#' @param grid Logical flag indicating whether to draw grid
#' @inheritParams apply_grid
#' @param strip_swap swap facet or grid strips
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @aliases autoplot.survfit.cox
#' @examples
#' library(survival)
#' autoplot(survfit(Surv(time, status) ~ sex, data = lung))
#' autoplot(survfit(Surv(time, status) ~ sex, data = lung), facets = TRUE)
#' autoplot(survfit(Surv(time, status) ~ 1, data = lung))
#' autoplot(survfit(Surv(time, status) ~ sex, data=lung), conf.int = FALSE, censor = FALSE)
#' autoplot(survfit(coxph(Surv(time, status) ~ sex, data = lung)))
#' @importFrom scales percent
#' @export
autoplot.survfit <- function(object, fun = NULL,
                             surv.geom = 'step',
                             surv.colour = NULL, surv.size = NULL, surv.linetype = NULL,
                             surv.alpha = NULL, surv.fill = NULL, surv.shape = NULL,
                             surv.connect = TRUE,
                             conf.int = TRUE,
                             conf.int.colour = '#0000FF', conf.int.linetype = 'none',
                             conf.int.fill = '#000000', conf.int.alpha = 0.3,
                             censor = TRUE, censor.colour = NULL, censor.size = 3,
                             censor.alpha = NULL, censor.shape = '+',
                             facets = FALSE, nrow = NULL, ncol = 1,
                             grid = FALSE, strip_swap = FALSE,
                             scales = 'free_y',
                             xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                             main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                             ...) {

  if (is_derived_from(object, 'aareg')) {
    # for autoplot.aareg, object must be a data.frame
    plot.data <- object
    mapping <- aes_string(x = 'time', y = 'value')
    strips_formula <- ~ variable
    if (is.null(surv.colour)) {
      surv.colour <- 'variable'
    }
    # use default
    scale_labels <- ggplot2::waiver()
  } else {
    plot.data <- fortify(object, surv.connect = surv.connect, fun = fun)

    if (is_derived_from(object, 'survfitms')) {
      mapping <- aes_string(x = 'time', y = 'pstate')
    } else {
      mapping <- aes_string(x = 'time', y = 'surv')
    }

    group <- c()

    if ('strata' %in% colnames(plot.data)) {
      group <- c(group, 'strata')
      if (is.null(surv.colour)) {
        surv.colour <- 'strata'
      }
    }

    if ('event' %in% colnames(plot.data)) {
      group <- c(group, 'event')
      if (is.null(surv.linetype)) {
        surv.linetype <- 'event'
      }
    }

    if (length(group) == 1) {
      plot.data[, 'group'] <- plot.data[, group]
    } else {
      group.levels <- lapply(plot.data[, group], levels)
      group.levels <- apply(expand.grid(group.levels), 1, function(x) paste(x, collapse = ' '))
      group.data <- factor(apply(plot.data[, group], 1, function(x) paste(x, collapse = ' ')), levels = group.levels)
      plot.data[, 'group'] <- group.data
    }

    strips_formula <- c(
      if ('event' %in% colnames(plot.data)) 'event' else if (grid) '.',
      if ('strata' %in% colnames(plot.data)) 'strata' else if (grid) '.')
    if (strip_swap) strips_formula <- rev(strips_formula)
    if (!is.null(strips_formula)) {
      if (grid) {
        strips_formula <- as.formula(paste(strips_formula, collapse = ' ~ '))
      } else if (facets) {
        strips_formula <- as.formula(c('~', paste(strips_formula, collapse = ' + ')))
      }
    }

    if (is.null(fun) || identical(fun, 'identity') || identical(fun, 'event')) {
      scale_labels <- scales::percent
    } else {
      scale_labels <- ggplot2::waiver()
    }
  }

  if (missing(conf.int.fill) & !is.null(surv.colour)) {
    conf.int.fill <- surv.colour
  }

  geomfunc <- get_geom_function(surv.geom, allowed = c('step', 'line', 'point'))

  p <- ggplot(data = plot.data, mapping = mapping) +
    scale_y_continuous(labels = scale_labels)
  p <- p + geom_factory(geomfunc, plot.data,
                        colour = surv.colour, size = surv.size, linetype = surv.linetype,
                        alpha = surv.alpha, fill = surv.fill, shape = surv.shape)
  if (surv.geom == 'step') {
    conf.int.geom <- 'step'
  } else {
    conf.int.geom <- 'line'
  }
  p <- plot_confint(p, data = plot.data,
                    conf.int = conf.int, conf.int.geom = conf.int.geom,
                    conf.int.group = 'group',
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  if (censor & 'n.censor' %in% colnames(plot.data)) {
    p <- p + geom_factory(geom_point, plot.data[plot.data$n.censor > 0, ],
                          colour = censor.colour, size = censor.size,
                          alpha = censor.alpha, shape = censor.shape)
  }
  if (facets) {
    p <- apply_facets(p, strips_formula, nrow = nrow, ncol = ncol, scales = scales)
  } else if (grid) {
    p <- apply_grid(p, strips_formula, scales = scales)
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{survival::aareg} to \code{data.frame}
#'
#' @param model \code{survival::aareg} instance
#' @param maxtime truncate the input to the model at time "maxtime"
#' @inheritParams fortify.survfit
#' @param melt Logical flag indicating whether to melt each timeseries as variable
#' @return data.frame
#' @examples
#' library(survival)
#' fortify(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1))
#' fortify(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1), melt = TRUE)
#' @export
fortify.aareg <- function(model, data = NULL,
                          maxtime = NULL,
                          surv.connect = TRUE,
                          melt = FALSE, ...) {

  if (is.null(maxtime)) {
    keep <- 1:length(model$time)
  } else {
    keep <- 1:sum(model$time <= maxtime)
  }

  if (is.matrix(model$coefficient) && ncol(model$coefficient) > 1) {
    coefs <- model$coefficient[keep, ]
  } else {
    coefs <- model$coefficient[keep]
  }
  rownames(coefs) <- NULL
  coefs <- as.data.frame(coefs)
  cols <- colnames(coefs)

  if (melt) {
    d <- cbind(data.frame(time = model$time[keep]), coefs)
    if (surv.connect) {
      d <- rbind(0, d)
    }
    d <- tidyr::gather_(d, 'variable', 'coef', cols)
    d <- d %>%
      dplyr::group_by(variable) %>%
      dplyr::mutate(se = sqrt(cumsum(coef ^ 2)),
                     value = cumsum(coef),
                     upper = value + se * 1.96,
                     lower = value - se * 1.96)
  } else {
    d <- cbind_wraps(data.frame(time = model$time[keep]),
                     apply(coefs, 2, cumsum))
    indexer <- 1 + length(d$time) - rev(match(unique(rev(d$time)), rev(d$time)))
    d <- d[indexer, ]
    if (surv.connect) {
      d <- rbind(0, d)
    }
  }
  post_fortify(d, klass = model)
}

#' Autoplot \code{survival::aareg}
#'
#' @param object \code{survival::aareg} instance
#' @param maxtime truncate the input to the model at time "maxtime"
#' @inheritParams autoplot.survfit
#' @param ... other arguments passed to \code{autoplot.survfit}
#' @return ggplot
#' @examples
#' library(survival)
#' autoplot(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1))
#' @export
autoplot.aareg <- function (object, maxtime = NULL,
                            surv.connect = TRUE,
                            facets = TRUE, ncol = NULL,
                            xlab = '', ylab = '',
                            ...) {

  plot.data <- fortify(object, maxtime = maxtime,
                       surv.connect = surv.connect, melt = TRUE)
  autoplot.survfit(plot.data, facets = facets, ncol = ncol,
                   xlab = xlab, ylab = ylab, ...)
}

#' Convert \code{stats::density} to \code{data.frame}
#'
#' @param model \code{stats::density} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(stats::density(stats::rnorm(1:50)))
#' @export
fortify.density <- function(model, data = NULL, ...) {
  d <- data.frame(x = model$x, y = model$y)
  post_fortify(d)
}

#' Autoplot \code{stats::density}
#'
#' @param object \code{stats::density} instance
#' @param p \code{ggplot2::ggplot} instance to plot
#' @param colour Line colour
#' @param linetype Line type
#' @param fill Fill colour
#' @param alpha Alpha
#' @inheritParams post_autoplot
#' @param ... other arguments passed to PDC/CDF func
#' @return ggplot
#' @examples
#' autoplot(stats::density(stats::rnorm(1:50)))
#' autoplot(stats::density(stats::rnorm(1:50)), fill = 'blue')
#' @importFrom scales percent
#' @export
autoplot.density <- function (object, p = NULL,
                              colour = '#000000', linetype = NULL,
                              fill = NULL, alpha = NULL,
                              xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                              main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                              ...)  {
  if (!is.data.frame(object)) {
    object <- ggplot2::fortify(object)
    object$ymin <- rep(0, nrow(object))
  }
  mapping <- ggplot2::aes_string(x = 'x', y = 'y', ymin = 'ymin', ymax = 'y')
  if (is.null(p)) {
    if (is.null(xlab)) {
      xlab = ''
    }
    if (is.null(ylab)) {
      ylab = ''
    }
    p <- ggplot2::ggplot(mapping = mapping) +
      ggplot2::scale_x_continuous(name = xlab) +
      ggplot2::scale_y_continuous(name = ylab)
  }
  if (!is.null(fill)) {

    if (is.null(alpha)) {
      # specify default which should not affect to geom_line
      alpha <- 0.3
    }

    p <- p + geom_factory(geom_ribbon, object, colour = colour,
                          linetype = linetype, fill = fill, alpha = alpha)
  } else {
    # not to draw bottom line
    p <- p + geom_factory(geom_line, object, colour = colour,
                          linetype = linetype, alpha = alpha)
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Plot distribution
#'
#' @param func PDF or CDF function
#' @param x Numeric vector to be passed to func
#' @param p \code{ggplot2::ggplot} instance to plot
#' @param colour Line colour
#' @param linetype Line type
#' @param fill Fill colour
#' @param alpha Alpha
#' @param ... Keywords passed to PDC/CDF func
#' @return ggplot
#' @examples
#' ggdistribution(dnorm, seq(-3, 3, 0.1), mean = 0, sd = 1)
#' ggdistribution(ppois, seq(0, 30), lambda = 20)
#'
#' p <- ggdistribution(pchisq, 0:20, df = 7, fill = 'blue')
#' ggdistribution(pchisq, 0:20, p = p, df = 9, fill = 'red')
#' @export
ggdistribution <- function (func, x, p = NULL,
                            colour = '#000000', linetype = NULL,
                            fill = NULL, alpha = NULL, ...)  {
  data <- data.frame(x = x, y = func(x, ...),
                     ymin = rep(0, length(x)))
  p <- autoplot.density(data, p = p, colour = colour, linetype = linetype,
                        fill = fill, alpha = alpha, ...)
  p
}

## ----global_options, include=FALSE--------------------------------------------
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/surv-', warning=FALSE)

## ---- message = FALSE---------------------------------------------------------
library(ggfortify)
library(survival)
fit <- survfit(Surv(time, status) ~ sex, data = lung)
autoplot(fit)

## ---- message = FALSE---------------------------------------------------------
autoplot(fit, surv.linetype = 'dashed', conf.int = FALSE,
         censor.shape = '*', censor.size = 5, facets = TRUE, ncol = 2)

autoplot(survfit(Surv(time, status) ~ 1, data = lung), surv.colour = 'orange', censor.colour = 'red')

autoplot(survfit(Surv(time, status) ~ sex, data = lung), fun = 'event')

d.coxph <- survfit(coxph(Surv(time, status) ~ sex, data = lung))
autoplot(d.coxph, surv.linetype = 'dashed', surv.colour = 'blue',
         conf.int.fill = 'dodgerblue3', conf.int.alpha = 0.5, censor = FALSE)

## ---- fig.width = 8, fig.height = 4-------------------------------------------
autoplot(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung))

#' Attach confidence interval to \code{ggplot2::ggplot}
#'
#' @param p \code{ggplot2::ggplot} instance
#' @param data data contains lower and upper confidence intervals
#' @param lower column name for lower confidence interval
#' @param upper column name for upper confidence interval
#' @param conf.int Logical flag indicating whether to plot confidence intervals
#' @param conf.int.geom geometric string for confidence interval. 'line' or 'step'
#' @param conf.int.group name of grouping variable for confidence intervals
#' @param conf.int.colour line colour for confidence intervals
#' @param conf.int.linetype line type for confidence intervals
#' @param conf.int.fill fill colour for confidence intervals
#' @param conf.int.alpha alpha for confidence intervals
#' @return ggplot
#' @examples
#' d <- fortify(stats::acf(AirPassengers, plot = FALSE))
#' p <- ggplot(data = d, mapping = aes(x = Lag))
#' ggfortify:::plot_confint(p, data = d)
plot_confint <- function (p, data = NULL, lower = 'lower', upper = 'upper',
                          conf.int = TRUE, conf.int.geom = 'line',
                          conf.int.group = NULL,
                          conf.int.colour = '#0000FF', conf.int.linetype = 'none',
                          conf.int.fill = '#000000', conf.int.alpha = 0.3) {

  if (missing(conf.int) && (!missing(conf.int.colour) ||
                            !missing(conf.int.linetype) ||
                            !missing(conf.int.fill) ||
                            !missing(conf.int.alpha))) {
    # if conf.int is missing but other options are specified, turn conf.in to TRUE
    conf.int <- TRUE
  }

  if (is.null(data)) {
    stop("Internal Error: 'data' must be provided to plot_confint")
  }

  if (conf.int.geom == 'step') {
    ribbon_func <- geom_confint
    line_func <- geom_step
  } else {
    ribbon_func <- geom_ribbon
    line_func <- geom_line
  }

  if (conf.int) {
    if (!is.null(conf.int.fill)) {
      p <- p + geom_factory(ribbon_func, data, ymin = lower, ymax = upper,
                           group = conf.int.group,
                           fill = conf.int.fill, alpha = conf.int.alpha, na.rm = TRUE)
    }
    if (conf.int.linetype != 'none') {
      p <- p + geom_factory(line_func, data, y = lower,
                            group = conf.int.group,
                            colour = conf.int.colour, linetype = conf.int.linetype,
                            na.rm = TRUE)
      p <- p + geom_factory(line_func, data, y = upper,
                     group = conf.int.group,
                     colour = conf.int.colour, linetype = conf.int.linetype,
                     na.rm = TRUE)
    }
  }
  p
}

#' Attach label to \code{ggplot2::ggplot}
#'
#' @param p \code{ggplot2::ggplot} instance
#' @param data Data contains text label
#' @param x x coordinates for label
#' @param y y coordinates for label
#' @param label Logical value whether to display labels
#' @param label.label Column name used for label text
#' @param label.colour Colour for text labels
#' @param label.alpha Alpha for text labels
#' @param label.size Size for text labels
#' @param label.angle Angle for text labels
#' @param label.family Font family for text labels
#' @param label.fontface Fontface for text labels
#' @param label.lineheight Lineheight for text labels
#' @param label.hjust Horizontal adjustment for text labels
#' @param label.vjust Vertical adjustment for text labels
#' @param label.repel Logical flag indicating whether to use \code{ggrepel}, enabling this may take some time for plotting
#' @param label.show.legend Logical value indicating whether to show the legend of the text labels
#' @param label.position Character or a position function
#' @return ggplot
plot_label <- function(p, data, x = NULL, y = NULL, label = TRUE, label.label = 'rownames',
                       label.colour = NULL, label.alpha = NULL,
                       label.size = NULL, label.angle = NULL,
                       label.family = NULL, label.fontface = NULL,
                       label.lineheight = NULL,
                       label.hjust = NULL, label.vjust = NULL,
                       label.repel = FALSE, label.show.legend = NA,
                       label.position = "identity") {

  if (!is.data.frame(data)) {
    stop(paste0('Unsupported class: ', class(data)))
  }

  if (!missing(label.colour) && !is.null(label.colour) && missing(label)) {
    # if flag is missing but colour is specified, turn flag to TRUE
    label <- TRUE
  }

  if (label || label.repel) {
    # user wants label if they enables repel
    if (is.null(label.colour)) {
      # NULL may be explicitly passed from parent functions
      label.colour <- '#000000'
    }
    if (label.repel && 'ggrepel' %in% rownames(installed.packages())) {
      textfunc <- ggrepel::geom_text_repel
    } else {
      textfunc <- ggplot2::geom_text
    }
    p <- p + geom_factory(textfunc, data, x = x, y = y,
                          label = label.label,
                          colour = label.colour, alpha = label.alpha,
                          size = label.size, angle = label.angle,
                          family = label.family, fontface = label.fontface,
                          lineheight = label.lineheight,
                          hjust = label.hjust, vjust = label.vjust,
                          position = label.position,
                          show.legend = label.show.legend)
  }
  p
}


#' Apply facets to to \code{ggplot2::ggplot}
#'
#' @param p \code{ggplot2::ggplot} instance
#' @param formula \code{stats::formula} instance
#' @param facets Logical value to specify use facets
#' @param nrow Number of facet/subplot rows
#' @param ncol Number of facet/subplot columns
#' @param scales Scale value passed to \code{ggplot2}
#' @param ... other arguments passed to methods
#' @return ggplot
apply_facets <- function(p, formula, facets = TRUE, nrow = NULL, ncol = 1,
                         scales = 'free_y', ...) {
  if (!is.null(formula)) {
    if (is.character(formula)) {
      formula <- formula(paste('~', formula))
    }
    p <- p + ggplot2::facet_wrap(formula, scales = scales,
                                 nrow = nrow, ncol = ncol)
  }
  return(p)
}


#' Apply grid to to \code{ggplot2::ggplot}
#'
#' @param p \code{ggplot2::ggplot} instance
#' @param formula \code{stats::formula} instance
#' @param scales Scale value passed to \code{ggplot2}
#' @param ... other arguments passed to methods
apply_grid <- function(p, formula, scales = 'free_y', ...) {
  if (!is.null(formula) & deparse(formula) != '. ~ .') {
    p <- p + ggplot2::facet_grid(formula, scales = scales)
  }
  return(p)
}


#' Factory function to control \code{ggplot2::geom_xxx} functions
#'
#' @param geom string representation of \code{ggplot2::geom_xxx} function
#' @param allowed character vector contains allowed values
#' @return function
#' @examples
#' ggfortify:::get_geom_function('point')
#' ggfortify:::get_geom_function('line', allowed = c('line'))
get_geom_function <- function(geom, allowed = c('line', 'bar', 'point')) {

  if (!(geom %in% allowed)) {
    stop(paste("Invalid geom is specified. Use", paste(allowed, collapse=', ')))
  }
  # ToDo: may better to use layer directly?
  if (geom == 'line') {
    return(ggplot2::geom_line)
  } else if (geom == 'bar') {
    return(ggplot2::geom_bar)
  } else if (geom == 'point') {
    return(ggplot2::geom_point)
  } else if (geom == 'step') {
    return(ggplot2::geom_step)
  } else if (geom == 'ribbon') {
    return(ggplot2::geom_ribbon)
  } else if (geom == 'polygon') {
    return(ggplot2::geom_polygon)
  } else if (geom == 'path') {
    return(ggplot2::geom_path)
  }
  stop(paste("Invalid geom is specified. Use", paste(allowed, collapse=', ')))
}

#' Factory function to control \code{ggplot2::geom_xxx} functions
#'
#' @param geomfunc \code{ggplot2::geom_xxx} function
#' @param data plotting data
#' @param position A position function or character
#' @param ... other arguments passed to methods
#' @return proto
geom_factory <- function(geomfunc, data = NULL, position = NULL, ...) {
  mapping <- list()
  option <- list()

  columns <- colnames(data)
  for (key in names(list(...))) {
      value <- list(...)[[key]]
      if (is.null(value)) {
        # pass
      } else if (!(is.vector(value) && length(value) > 1L) &&
                 value %in% columns) {
        # check for length so that vectors of colours are supported
         mapping[[key]] <- value
      } else {
        option[[key]] <- value
      }
  }
  if (!is.null(data)) {
    option[['data']] <- data
  }
  if (!is.null(position)) {
    option[['position']] <- position
  }
  option[['mapping']] <- do.call(ggplot2::aes_string, mapping)
  return(do.call(geomfunc, option))
}

#' An S4 class to hold multiple \code{ggplot2::ggplot} instances
#'
#' @name ggmultiplot-class
#' @rdname ggmultiplot-class
#' @exportClass ggmultiplot
#'
#' @slot plots List of \code{ggplot2::ggplot} instances
#' @slot ncol Number of grid columns
#' @slot nrow Number of grid rows
setClass('ggmultiplot',
         representation(plots = 'list', ncol = 'numeric', nrow = 'numeric'),
         prototype = list(ncol = 0, nrow = 0))

# ref:
# https://stat.ethz.ch/R-manual/R-devel/library/methods/html/setMethod.html
# https://github.com/variani/pckdev/wiki/Documenting-with-roxygen2

#' Generic add operator for \code{ggmultiplot}
#'
#' @param e1 first argument
#' @param e2 second argument
#' @return \code{ggmultiplot}
setMethod('+', c('ggmultiplot', 'ANY'),
  function(e1, e2) {
    if (is(e2, 'ggmultiplot')) {
      # concat 2 ggmultiplots using lhs ncol / nrow
      plots <- c(e1@plots, e2@plots)
    } else if (is(e2, 'ggplot')) {
      plots <- c(e1@plots, list(e2))
    } else {
      # apply theme / add geom
      plots <- lapply(e1@plots, function(x) { x + e2 })
    }
    new('ggmultiplot', plots = plots,
        ncol = e1@ncol, nrow = e1@nrow)
})

#' @param x \code{ggmultiplot}
#'
#' @rdname ggmultiplot-class
#' @aliases length,ggmultiplot-method
setMethod('length', 'ggmultiplot',
  function(x) {
    return(length(x@plots))
})

#' @param i elements to extract or replace
#' @param j not used
#' @param ... not used
#' @param drop not used
#'
#' @rdname ggmultiplot-class
#' @aliases [,ggmultiplot-method
setMethod("[", signature(x = "ggmultiplot"),
  function(x, i, j, ..., drop) {
    new('ggmultiplot', plots = x@plots[i],
        ncol = x@ncol, nrow = x@nrow)
})

#' @rdname ggmultiplot-class
#' @aliases [[,ggmultiplot-method
setMethod("[[", signature(x = "ggmultiplot"),
  function(x, i, j, ..., drop) {
    x@plots[[i]]
})

#' @param value value to be set
#'
#' @rdname ggmultiplot-class
#' @aliases [<-,ggmultiplot-method
setMethod("[<-", signature(x = "ggmultiplot"),
  function(x, i, j, ..., value) {

    if (is(value, 'ggmultiplot')) {
      if (length(value) == length(x@plots[i])) {
        x@plots[i] <- value@plots
      } else {
        stop(paste('Unable to set value, length mismatch:', length(value)))
      }
    } else if (is(value, 'ggplot')) {
      if (length(x@plots[i]) == 1) {
        x@plots[[i]] <- value
      } else {
        stop(paste('Unable to set ggplot to multiple slice'))
      }
    } else {
      stop(paste('Unable to set type, unsupported type:', class(value)))
    }
    x
})

#' @rdname ggmultiplot-class
#' @aliases [[<-,ggmultiplot-method
setMethod("[[<-", signature(x = "ggmultiplot"),
  function(x, i, j, ..., value) {

    if (is(value, 'ggmultiplot')) {
      if (length(value) == 1) {
        x@plots[[i]] <- value@plots[[1]]
      } else {
        stop(paste('Unable to set value, length mismatch:', length(value)))
      }
    } else if (is(value, 'ggplot')) {
      x@plots[[i]] <- value
    } else {
      stop(paste('Unable to set type, unsupported type:', class(value)))
    }
    x
})

#' Calcurate layout matrix for \code{ggmultiplot}
#'
#' @param nplots Number of plots
#' @param ncol Number of grid columns
#' @param nrow Number of grid rows
#' @return matrix
#' @examples
#' ggfortify:::get.layout(3, 2, 2)
get.layout <- function(nplots, ncol, nrow) {
  if (ncol == 0 && nrow == 0) {
    ncol <- 2
  } else if (ncol == 0 && nrow != 0) {
    ncol <- ceiling(nplots / nrow)
  }

  if (nrow == 0) {
    nrow <- ceiling(nplots / ncol)
  } else {
    nrow <- nrow
  }

  if (nrow * ncol < nplots) {
    message <- paste('nrow * ncol (', nrow, ' * ', ncol,
                     ') must be larger than number of plots', nplots)
    stop(message)
  }

  t(matrix(1:(ncol * nrow), ncol = nrow, nrow = ncol))
}

#' Generic print function for \code{ggmultiplot}
#'
#' @param x \code{ggmultiplot}
#' @return NULL
#'
#' @importFrom gridExtra grid.arrange
setMethod('print', 'ggmultiplot',
  function(x) {
    layout <- get.layout(length(x@plots), x@ncol, x@nrow)
    args <- c(x@plots, list(ncol = ncol(layout), nrow = nrow(layout)))
    do.call(gridExtra::grid.arrange, args)
})

#' Generic show function for \code{ggmultiplot}
#'
#' @param object \code{ggmultiplot}
#' @return NULL
setMethod('show', 'ggmultiplot', function(object) { print(object) })

#' The implemented grid.draw method for ggmultiplot, in order to work
#' with ggsave() properly
#'
#' @export
#' @method grid.draw ggmultiplot
#' @importFrom grid grid.draw
#' @importFrom gridExtra arrangeGrob
#' @param plot \code{ggmultiplot}
grid.draw.ggmultiplot <- function(plot) {
  grid::grid.draw(gridExtra::arrangeGrob(grobs = plot@plots))
}

#' Post process for fortify. Based on \code{ggplot2::qplot}
#'
#' @param p \code{ggplot2::ggplot} instances
#' @param xlim limits for x axis
#' @param ylim limits for y axis
#' @param log which variables to log transform ("x", "y", or "xy")
#' @param main character vector or expression for plot title
#' @param xlab character vector or expression for x axis label
#' @param ylab character vector or expression for y axis label
#' @param asp the y/x aspect ratio
#' @return data.frame
#' @examples
#' p <- qplot(Petal.Length, Petal.Width, data = iris)
#' ggfortify:::post_autoplot(p, xlim = c(1, 5), ylim = c(1, 5), log = 'xy', main = 'title',
#'                           xlab = 'x', ylab = 'y', asp = 1.5)
post_autoplot <- function(p, xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                          main = NULL, xlab = NULL, ylab = NULL, asp = NULL) {
  logv <- function(var) var %in% strsplit(log, "")[[1]]
  if (logv("x"))
    p <- p + scale_x_log10()
  if (logv("y"))
    p <- p + scale_y_log10()
  if (!is.null(asp))
    p <- p + ggplot2::theme(aspect.ratio = asp)
  if (!is.null(main))
    p <- p + ggplot2::ggtitle(main)
  if (!is.null(xlab))
    p <- p + ggplot2::xlab(xlab)
  if (!is.null(ylab))
    p <- p + ggplot2::ylab(ylab)
  if (!all(is.na(xlim)))
    p <- p + ggplot2::xlim(xlim)
  if (!all(is.na(ylim)))
    p <- p + ggplot2::ylim(ylim)
  p
}

#' Check if passed object is supported by \code{ggplot2::autoplot}
#'
#' @param obj object
#' @return logical
support_autoplot <- function(obj) {
  maybe_autoplot <- paste0('autoplot.', class(obj))
  return(any(sapply(maybe_autoplot, function(x) x %in% utils::methods('autoplot'))))
}


#' Autoplot \code{ggplot} instances.
#' It returns the passed instance as it is.
#'
#' @param object ggplot instance
#' @param ... Not used.
#' @return ggplot
#' @export
autoplot.ggplot <- function(object, ...) {
  return (object)
}

#' Autoplot \code{ggmultiplot} instances.
#' It returns the passed instance as it is.
#'
#' @param object ggmultiplot instance
#' @param ... Not used.
#' @return ggmultiplot
#' @export
autoplot.ggmultiplot <- function(object, ...) {
  return (object)
}

#' Draw \code{biplot} using \code{ggplot2}.
#'
#' @param plot.data data.frame
#' @param loadings.data data.frame
#' @param colour colour
#' @param size size
#' @param linetype line type
#' @param alpha alpha
#' @param fill fill
#' @param shape shape
#' @param label Logical value whether to display data labels
#' @inheritParams plot_label
#' @param loadings Logical value whether to display loadings arrows
#' @param loadings.arrow An arrow definition
#' @param loadings.colour Point colour for data
#' @param loadings.label Logical value whether to display loadings labels
#' @param loadings.label.label Column name used for loadings text labels
#' @param loadings.label.colour Colour for loadings text labels
#' @param loadings.label.alpha Alpha for loadings text labels
#' @param loadings.label.size Size for loadings text labels
#' @param loadings.label.angle Angle for loadings text labels
#' @param loadings.label.family Font family for loadings text labels
#' @param loadings.label.fontface Fontface for loadings text labels
#' @param loadings.label.lineheight Lineheight for loadings text labels
#' @param loadings.label.hjust Horizontal adjustment for loadings text labels
#' @param loadings.label.vjust Vertical adjustment for loadings text labels
#' @param loadings.label.repel Logical flag indicating whether to use \code{ggrepel} automatically
#' @param label.show.legend Logical value indicating whether to show the legend of text labels
#' @param frame Logical value whether to draw outliner convex / ellipse
#' @param frame.type Character specifying frame type.
#' 'convex' or types supporeted by \code{ggplot2::stat_ellipse} can be used.
#' @param frame.colour Colour for frame
#' @param frame.level Passed for \code{ggplot2::stat_ellipse} 's level. Ignored in 'convex'.
#' @param frame.alpha Alpha for frame
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @export
ggbiplot <- function(plot.data, loadings.data = NULL,
                     colour = NULL, size = NULL, linetype = NULL,
                     alpha = NULL, fill = NULL, shape = NULL,
                     label = FALSE,
                     label.label = 'rownames',
                     label.colour = colour,
                     label.alpha = NULL,
                     label.size = NULL,
                     label.angle = NULL,
                     label.family = NULL,
                     label.fontface = NULL,
                     label.lineheight = NULL,
                     label.hjust = NULL,
                     label.vjust = NULL,
                     label.repel = FALSE,
                     label.position = "identity",
                     loadings = FALSE,
                     loadings.arrow = grid::arrow(length = grid::unit(8, 'points')),
                     loadings.colour = '#FF0000',
                     loadings.label = FALSE,
                     loadings.label.label = 'rownames',
                     loadings.label.colour = '#FF0000',
                     loadings.label.alpha = NULL,
                     loadings.label.size = NULL,
                     loadings.label.angle = NULL,
                     loadings.label.family = NULL,
                     loadings.label.fontface = NULL,
                     loadings.label.lineheight = NULL,
                     loadings.label.hjust = NULL,
                     loadings.label.vjust = NULL,
                     loadings.label.repel = FALSE,
                     label.show.legend = NA,
                     frame = FALSE, frame.type = NULL,
                     frame.colour = colour, frame.level = 0.95,
                     frame.alpha = 0.2,
                     xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                     main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                     ...) {
#  print(label.position)

  plot.columns <- colnames(plot.data)
  mapping <- ggplot2::aes_string(x = plot.columns[1L], y = plot.columns[2L])

  if (is.logical(shape) && !shape && missing(label)) {
    # if shape=FALSE, turn label to TRUE
    label <- TRUE
  }

  p <- ggplot2::ggplot(data = plot.data, mapping = mapping)
  if (!is.logical(shape) || shape) {
    p <- p + geom_factory(ggplot2::geom_point, plot.data,
                          colour = colour, size = size, linetype = linetype,
                          alpha = alpha, fill = fill, shape = shape)
  }
  p <- plot_label(p = p, data = plot.data, label = label,
                  label.label = label.label,
                  label.colour = label.colour,
                  label.alpha = label.alpha,
                  label.size = label.size,
                  label.angle = label.angle,
                  label.family = label.family,
                  label.fontface = label.fontface,
                  label.lineheight = label.lineheight,
                  label.hjust = label.hjust,
                  label.vjust = label.vjust,
                  label.repel = label.repel,
                  label.show.legend = label.show.legend,
                  label.position = label.position)

  if (loadings.label && !loadings) {
    # If loadings.label is TRUE, draw loadings
    loadings <- TRUE
  }
  if (loadings && !is.null(loadings.data)) {

    scaler <- min(max(abs(plot.data[, 1L])) / max(abs(loadings.data[, 1L])),
                  max(abs(plot.data[, 2L])) / max(abs(loadings.data[, 2L])))

    loadings.columns <- colnames(loadings.data)
    loadings.mapping <- ggplot2::aes_string(x = 0, y = 0,
                                            xend = loadings.columns[1L],
                                            yend = loadings.columns[2L])
    loadings.data[, 1L:2L] <- loadings.data[, 1L:2L] * scaler * 0.8

    p <- p + geom_segment(data = loadings.data,
                          mapping = loadings.mapping,
                          arrow = loadings.arrow,
                          colour = loadings.colour)
    p <- plot_label(p = p, data = loadings.data, label = loadings.label,
                    label.label = loadings.label.label,
                    label.colour = loadings.label.colour,
                    label.alpha = loadings.label.alpha,
                    label.size = loadings.label.size,
                    label.angle = loadings.label.angle,
                    label.family = loadings.label.family,
                    label.fontface = loadings.label.fontface,
                    label.lineheight = loadings.label.lineheight,
                    label.hjust = loadings.label.hjust,
                    label.vjust = loadings.label.vjust,
                    label.repel = loadings.label.repel,
                    label.show.legend = label.show.legend,
                    label.position = label.position)
  }

  if (missing(frame) && !is.null(frame.type)) {
    # if frame is missing but frame.type is specified, turn frame to TRUE
    frame <- TRUE
  }

  # dummy to solve "no visible binding for global variable '.'" warnings
  . <- NULL

  if (frame) {
    if (is.null(frame.type) || frame.type == 'convex') {
      if (is.null(frame.colour) || !(frame.colour %in% colnames(plot.data))) {
        hulls <- plot.data[grDevices::chull(plot.data[, 1L:2L]), ]
      } else {
        hulls <- plot.data %>%
          dplyr::group_by(.data[[frame.colour]]) %>%
          dplyr::do(.[grDevices::chull(.[, 1L:2L]), ])
      }
      mapping <- aes_string(colour = frame.colour, fill = frame.colour)
      p <- p + ggplot2::geom_polygon(data = hulls, mapping = mapping,
                                     alpha = frame.alpha)
    } else if (frame.type %in% c('t', 'norm', 'euclid')) {
      mapping <- aes_string(colour = frame.colour, fill = frame.colour)
      p <- p + ggplot2::stat_ellipse(mapping = mapping,
                                     level = frame.level, type = frame.type,
                                     geom = 'polygon', alpha = frame.alpha)
    } else {
      stop('frame.type must be convex, t, norm or euclid')
    }
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  return(p)
}

context('test plotlib')

test_that('Check ggmultiplot arithmetics', {
  skip_on_cran()
  skip_on_travis()
  p1 <- autoplot(lm(Petal.Width ~ Petal.Length, data = iris))
  p2 <- autoplot(lm(Sepal.Width ~ Sepal.Length, data = iris))
  expect_true(is(p1, 'ggmultiplot'))
  expect_true(is(p2, 'ggmultiplot'))

  res <- p1 + p2
  expect_true(is(res, 'ggmultiplot'))
  expect_equal(length(res@plots), 8)

  res <- p1 + (ggplot(iris, aes(x=Sepal.Width, y=Sepal.Length)) + geom_point())
  expect_true(is(res, 'ggmultiplot'))
  expect_equal(length(res@plots), 5)

  res <- res + theme_bw()
  expect_true(is(res, 'ggmultiplot'))
  expect_equal(length(res@plots), 5)
})


test_that('Check ggmultiplot extraction', {
  skip_on_cran()
  skip_on_travis()
  p <- autoplot(lm(Petal.Width ~ Petal.Length, data = iris))
  expect_equal(length(p), 4)

  # getter

  res <- p[1]
  expect_true(is(res, 'ggmultiplot'))
  expect_equal(length(res), 1)

  res <- p[2:3]
  expect_true(is(res, 'ggmultiplot'))
  expect_equal(length(res), 2)

  res <- p[[1]]
  expect_true(is(res, 'ggplot'))

  # setter
  p[1] <- p[1] # same length
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 4)

  p[2:3] <- p[1:2] # same length
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 4)

  p[1] <- p[[1]] # same length (set ggplot)
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 4)

  # different length
  temp <- function(x) {
    x[2:4] <- x[1:2]
  }
  expect_error(temp(p), 'Unable to set value, length mismatch')

  # different length (set ggplot to slice)
  temp <- function(x) {
    x[2:4] <- x[[1]]
  }
  expect_error(temp(p), 'Unable to set ggplot to multiple slice')

  # invalid value
  temp <- function(x) {
    x[2:4] <- 'xxx'
  }
  expect_error(temp(p), 'Unable to set type, unsupported type')

  # setter
  p[[1]] <- p[1] # same length
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 4)

  p[[1]] <- p[[1]] # same length (set ggplot)
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 4)

  # different length
  temp <- function(x) {
    x[[1]] <- x[1:2]
  }
  expect_error(temp(p), 'Unable to set value, length mismatch')

  # invalid value
  temp <- function(x) {
    x[2:4] <- 'xxx'
  }
  expect_error(temp(p), 'Unable to set type, unsupported type')
})

test_that('Check ggmultiplot multiple instances', {
  skip_on_cran()
  skip_on_travis()
  res <- lapply(c(3, 4, 5), function(x) kmeans(iris[-5], x))
  p <- autoplot(res, data = iris[-5])
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 3)

  p <- autoplot(list(a = AirPassengers, b = AirPassengers))
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 2)

  library(survival)
  sf <- survfit(Surv(time, status) ~ sex, data = lung)
  res <- list(a = sf, b = sf, c = sf)
  p <- autoplot(res)
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 3)

  res <- list(a = lm(Sepal.Width ~ Sepal.Length, data = iris),
              b = lm(Petal.Width ~ Petal.Length, data = iris))
  p <- autoplot(res, ncol = 4)
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 8)

  p <- autoplot(list(a=Canada, b=AirPassengers))
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p), 2)
})

test_that('Check get.layout works', {
  skip_on_cran()
  skip_on_travis()
  expect_equal(ggfortify:::get.layout(5, 2, 0), t(matrix(1:6, 2, 3)))
  expect_equal(ggfortify:::get.layout(1, 2, 0), t(matrix(1:2, 2, 1)))
  expect_equal(ggfortify:::get.layout(2, 2, 0), t(matrix(1:2, 2, 1)))
  expect_equal(ggfortify:::get.layout(3, 2, 0), t(matrix(1:4, 2, 2)))

  expect_equal(ggfortify:::get.layout(8, 3, 0), t(matrix(1:9, 3, 3)))
  expect_equal(ggfortify:::get.layout(2, 3, 4), t(matrix(1:12, 3, 4)))

  expect_equal(ggfortify:::get.layout(5, 0, 3), t(matrix(1:6, 2, 3)))
  expect_equal(ggfortify:::get.layout(1, 0, 2), t(matrix(1:2, 1, 2)))
  expect_equal(ggfortify:::get.layout(2, 0, 3), t(matrix(1:3, 1, 3)))
  expect_equal(ggfortify:::get.layout(3, 0, 2), t(matrix(1:4, 2, 2)))

  expect_equal(ggfortify:::get.layout(3, 2, 2), t(matrix(1:4, 2, 2)))
  expect_equal(ggfortify:::get.layout(2, 1, 3), t(matrix(1:3, 1, 3)))
  expect_equal(ggfortify:::get.layout(3, 1, 3), t(matrix(1:3, 1, 3)))

})

test_that('Check geom_factory works', {
  skip_on_cran()
  skip_on_travis()
  # Unable to compare geom_xxx each other, because it is an environment variable
  # Thus, capture the printed result and check equalities

  result <- ggfortify:::geom_factory(geom_point, iris, shape = 'Species')
  expected <- c("mapping: shape = ~Species ",
                "geom_point: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_point, iris, shape = 'Species', size = 10)
  expected <- c("mapping: shape = ~Species ",
                "geom_point: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_point, iris, shape = 'Species',
                                     size = 'Sepal.Width')
  expected <- c("mapping: shape = ~Species, size = ~Sepal.Width ",
                "geom_point: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_point, iris, shape = 2,
                                     colour = 'Species', size = 'Sepal.Width')
  expected <- c("mapping: colour = ~Species, size = ~Sepal.Width ",
                "geom_point: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_line, iris)
  expected <- c("mapping:  ",
                "geom_line: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_line, iris, linetype = 'dashed')
  expected <- c("mapping:  ",
                "geom_line: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)

  result <- ggfortify:::geom_factory(geom_line, iris, linetype = 'dashed',
                                     colour = 'Species')
  expected <- c("mapping: colour = ~Species ",
                "geom_line: na.rm = FALSE",
                "stat_identity: na.rm = FALSE",
                "position_identity ")

  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(capture.output(print(result)), expected)
})

test_that('Check autoplot works for list of ggplot', {
  skip_on_cran()
  skip_on_travis()
  library(dplyr)
  p <- iris %>% ggplot(aes(Sepal.Length, Petal.Length)) + geom_point()
  plots <- iris %>% group_by(Species) %>%
    do(plots = p %+% . + facet_wrap(~Species))
  p <- autoplot(plots$plots)
  expect_true(inherits(p, 'ggmultiplot'))
})

test_that('Check autoplot works for list of ggpmultilot', {
  skip_on_cran()
  skip_on_travis()
  library(dplyr)
  plots <- iris %>% group_by(Species) %>%
    do(plots = autoplot(lm(Petal.Width ~ Petal.Length, data = .)))
  p <- autoplot(plots$plots)
  expect_true(inherits(p, 'ggmultiplot'))
})

## ----global_options, include=FALSE--------------------------------------------
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/pca-', warning=FALSE)
hasDep <- require("cluster", quietly = TRUE) && require("lfda", quietly = TRUE) && require("MASS", quietly = TRUE)

## ---- message = FALSE, eval=hasDep--------------------------------------------
library(ggfortify)
df <- iris[1:4]
pca_res <- prcomp(df, scale. = TRUE)

autoplot(pca_res)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, data = iris, colour = 'Species')

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, data = iris, colour = 'Species', label = TRUE, label.size = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, data = iris, colour = 'Species', shape = FALSE, label.size = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, data = iris, colour = 'Species', loadings = TRUE)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, data = iris, colour = 'Species',
         loadings = TRUE, loadings.colour = 'blue',
         loadings.label = TRUE, loadings.label.size = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(pca_res, scale = 0)

## ---- message = FALSE, eval=hasDep--------------------------------------------
d.factanal <- factanal(state.x77, factors = 3, scores = 'regression')
autoplot(d.factanal, data = state.x77, colour = 'Income')
autoplot(d.factanal, label = TRUE, label.size = 3,
         loadings = TRUE, loadings.label = TRUE, loadings.label.size  = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
set.seed(1)
autoplot(kmeans(USArrests, 3), data = USArrests)

autoplot(kmeans(USArrests, 3), data = USArrests, label = TRUE, label.size = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
library(cluster)
autoplot(clara(iris[-5], 3))

## ---- eval=hasDep-------------------------------------------------------------
autoplot(fanny(iris[-5], 3), frame = TRUE)

## ---- eval=hasDep-------------------------------------------------------------
autoplot(pam(iris[-5], 3), frame = TRUE, frame.type = 'norm')

## ---- eval=hasDep-------------------------------------------------------------
autoplot(silhouette(pam(iris[-5], 3L)))

## ---- message = FALSE, eval=hasDep--------------------------------------------
library(lfda)

# Local Fisher Discriminant Analysis (LFDA)
model <- lfda(iris[-5], iris[, 5], r = 3, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')

## ---- message = FALSE, eval=hasDep--------------------------------------------
# Semi-supervised Local Fisher Discriminant Analysis (SELF)
model <- self(iris[-5], iris[, 5], beta = 0.1, r = 3, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(eurodist)

## ---- eval=hasDep-------------------------------------------------------------
autoplot(cmdscale(eurodist, eig = TRUE))

## ---- eval=hasDep-------------------------------------------------------------
autoplot(cmdscale(eurodist, eig = TRUE), label = TRUE, label.size = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
library(MASS)
autoplot(isoMDS(eurodist), colour = 'orange', size = 4, shape = 3)

## ---- message = FALSE, eval=hasDep--------------------------------------------
autoplot(sammon(eurodist), shape = FALSE, label.colour = 'blue', label.size = 3)

#' Convert cluster instances to \code{data.frame}
#'
#' @param model Clustered instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(stats::kmeans(iris[-5], 3))
#' fortify(stats::kmeans(iris[-5], 3), data = iris)
#' fortify(cluster::clara(iris[-5], 3))
#' fortify(cluster::fanny(iris[-5], 3))
#' fortify(cluster::pam(iris[-5], 3), data = iris)
#' @export
fortify.kmeans <- function(model, data = NULL, ...) {

  if (is(model, 'kmeans')) {
    d <- data.frame(cluster = as.factor(model$cluster))
  } else if (is(model, 'partition')) {
    d <- data.frame(ggplot2::fortify(model$data),
                    cluster = as.factor(model$cluster))
  } else {
    stop(paste0('Unsupported class for fortify.kmeans: ', class(model)))
  }
  d <- cbind_wraps(data, d)
  post_fortify(d)
}


#' Autoplot cluster instances
#'
#' @param object Clustered instance
#' @param data Original data used for clustering. Mandatory for \code{stats::kmeans}.
#' @param colour line colour for points
#' @param ... other arguments passed to \code{autoplot::prcomp}
#' @return ggplot
#' @examples
#' autoplot(stats::kmeans(iris[-5], 3), data = iris)
#' autoplot(cluster::clara(iris[-5], 3), label = TRUE)
#' autoplot(cluster::fanny(iris[-5], 3))
#' autoplot(cluster::fanny(iris[-5], 3), frame = TRUE)
#' autoplot(cluster::pam(iris[-5], 3), data = iris, colour = 'Species')
#' autoplot(cluster::pam(iris[-5], 3), data = iris, frame = TRUE, frame.type = 't')
#' @export
autoplot.kmeans <- function(object, data = NULL,
                            colour = 'cluster', ...) {

  if (is_derived_from(object, 'kmeans')) {
    if (is.null(data)) {
      stop("'data' is mandatory for plotting kmeans instance")
    }
    dots <- 'centers'
  } else if (is_derived_from(object, 'partition')) {
    dots <- 'data'
  } else {
    stop(paste0('Unsupported class for autoplot.kmeans: ', class(object)))
  }

  plot.data <- ggplot2::fortify(object, data = data)
  dots <- colnames(object[[dots]])
  plot.data$rownames <- rownames(plot.data)
  pca.data <- dplyr::select(plot.data, all_of(dots))
  p <- ggplot2::autoplot(stats::prcomp(pca.data), data = plot.data,
                         colour = colour, ...)
  p
}

#' @export
fortify.partition <- fortify.kmeans

#' @export
fortify.clara <- fortify.partition

#' @export
fortify.fanny <- fortify.partition

#' @export
fortify.pam <- fortify.partition

#' @export
autoplot.partition <- autoplot.kmeans

#' @export
autoplot.clara <- autoplot.partition

#' @export
autoplot.fanny <- autoplot.partition

# @export
autoplot.pam <- autoplot.partition

#' Convert \code{cluster::silhouette} to \code{data.frame}
#'
#' @param model Silhouette instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(cluster::silhouette(cluster::pam(iris[-5], 3)))
#' fortify(cluster::silhouette(cluster::clara(iris[-5], 3)))
#' fortify(cluster::silhouette(cluster::fanny(iris[-5], 3)))
#'
#' mod = stats::kmeans(iris[-5], 3)
#' fortify(cluster::silhouette(mod$cluster, stats::dist(iris[-5])))
#' @export
fortify.silhouette <- function(model, data = NULL, ...) {

  if (is(model, 'silhouette')) {
    d <- as.data.frame(unclass(model))
    d <- data.frame(cluster = as.factor(d$cluster),
                    sil_width = d$sil_width)
    d <- d[order(d$cluster, d$sil_width), ]
    d$id <- seq_len(nrow(d))
  } else {
    stop(paste0('Unsupported class for fortify.silhouette: ', class(model)))
  }
  post_fortify(d)
}

#' Autoplot silhouette instances
#'
#' @param object Silhouette instance
#' @param colour reference line color
#' @param linetype reference line type
#' @param size reference line size
#' @param bar.width bar width
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' model = cluster::pam(iris[-5], 3L)
#' sil = cluster::silhouette(model)
#' autoplot(sil)
#'
#' autoplot(cluster::silhouette(cluster::clara(iris[-5], 3)))
#' autoplot(cluster::silhouette(cluster::fanny(iris[-5], 3)))
#'
#' model = stats::kmeans(iris[-5], 3)
#' sil = cluster::silhouette(model$cluster, stats::dist(iris[-5]))
#' autoplot(sil)
#' @export
autoplot.silhouette <- function(object, colour = "red",
                                linetype = "dashed", size = 0.5,
                                bar.width = 1, ...) {

  if (!is_derived_from(object, 'silhouette')) {
    stop(paste0('Unsupported class for autoplot.silhouette: ', class(object)))
  }

  plot.data <- ggplot2::fortify(object)
  mapping <- aes_string(x = 'id', y = 'sil_width', fill = 'cluster')

  min.y <- if(min(plot.data$sil_width) < 0) min(plot.data$sil_width) else 0
  ylim <- c(min.y, 1)
  yinter <- round(mean(plot.data$sil_width), 2)

  geomfunc <- get_geom_function("bar")

  p <- ggplot(plot.data, mapping = mapping)
  p <- p + geom_factory(geomfunc, plot.data, stat = "identity",
                        width = bar.width) +
    geom_hline(yintercept = yinter, colour = colour,
                      linetype = linetype, size = size)

  p <- post_autoplot(p = p, ylim = ylim, xlab = "Observations", ylab = "Silhouette Values") +
    coord_flip()
  p
}

#' Convert \code{changepoint::cpt} and \code{strucchange::breakpoints} to \code{data.frame}
#'
#' @param model \code{chantepoint::cpt} or \code{strucchange::breakpoints} instance
#' @inheritParams fortify_base
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @return data.frame
#' @aliases fortify.breakpointsfull fortify.breakpoints
#' @examples
#' library(changepoint)
#' fortify(cpt.mean(AirPassengers))
#' fortify(cpt.var(AirPassengers))
#' fortify(cpt.meanvar(AirPassengers))
#'
#' library(strucchange)
#' bp.nile <- breakpoints(Nile ~ 1)
#' fortify(bp.nile)
#' fortify(breakpoints(bp.nile, breaks = 2))
#' fortify(breakpoints(bp.nile, breaks = 2), data = Nile)
#' @export
fortify.cpt <- function(model, data = NULL,
                        is.date = NULL, ...) {
  # changepoint::cpt is S4 class
  if (is(model, 'cpt')) {
    d <- ggplot2::fortify(model@data.set, is.date = is.date)

    cptindex <- get.dtindex(model@data.set, is.date = is.date)[model@cpts]
    cptd <- data.frame(Index = cptindex)
    if ('mean' %in% names(model@param.est)) {
      cptd$mean <- model@param.est$mean
    }
    if ('variance' %in% names(model@param.est)) {
      cptd$variance <- model@param.est$variance
    }
    d <- dplyr::left_join(d, cptd, by = 'Index')
  } else if (is(model, 'breakpointsfull') || is(model, 'breakpoints')) {
      dtindex <- get.dtindex(model$datatsp, is.tsp = TRUE, is.date = is.date)
      breakpoints <- rep(NA, length(dtindex))
      breakpoints[model$breakpoints] <- 1
      d <- data.frame(Index = dtindex, Breaks = breakpoints)
      if (!is.null(data)) {
        data <- ggplot2::fortify(data, is.date = is.date)
        d <- dplyr::left_join(data, d, by = 'Index')
      } else if ('y' %in% names(model)) {
        # breakpointsfull should have y
        data <- data.frame(Index = dtindex, Data = model$y)
        d <- dplyr::left_join(data, d, by = 'Index')
      }
  } else {
    stop(paste0('Unsupported class for autoplot.pca_common: ', class(model)))
  }
  post_fortify(d)
}

#' @export
fortify.breakpoints <- fortify.cpt

#' @export
fortify.breakpointsfull <- fortify.cpt

#' Autoplot \code{changepoint::cpt}
#'
#' @param object \code{changepoint::cpt} instance
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @param cpt.colour Line colour for changepoints
#' @param cpt.linetype Line type for changepoints
#' @param ... other arguments passed \code{autoplot.ts}
#' @return ggplot
#' @examples
#' library(changepoint)
#' autoplot(cpt.mean(AirPassengers))
#' autoplot(cpt.meanvar(AirPassengers))
#' @export
autoplot.cpt <- function(object, is.date = NULL,
                         cpt.colour = '#FF0000', cpt.linetype = 'dashed',
                         ...) {
  plot.data <- ggplot2::fortify(object, is.date = is.date)
  y <- 'Data'
  p <- autoplot.ts(plot.data, columns = y, ...)

  if ('mean' %in% names(plot.data)) {
    d <- dplyr::filter(plot.data, !is.na(mean))
    p <- p + ggplot2::geom_vline(xintercept = as.integer(d$Index),
                                 colour = cpt.colour, linetype = cpt.linetype)
  }
  if ('variance' %in% names(plot.data)) {
    d <- dplyr::filter(plot.data, !is.na(variance))
    p <- p + ggplot2::geom_vline(xintercept = as.integer(d$Index),
                                 colour = cpt.colour, linetype = cpt.linetype)
  }
  p
}

#' Autoplot \code{strucchange::breakpoints}
#'
#' @param object \code{strucchange::breakpoints} or \code{strucchange::breakpointsfull} instance.
#' @param data Original time series. Mandatory for plotting \code{strucchange::breakpoints} instance.
#' @param cpt.colour Line colour for changepoints
#' @param cpt.linetype Line type for changepoints
#' @param ... other arguments passed to \code{autoplot.ts}
#' @return ggplot
#' @aliases autoplot.breakpointsfull
#' @examples
#' library(strucchange)
#' bp.nile <- breakpoints(Nile ~ 1)
#' autoplot(bp.nile)
#' autoplot(bp.nile, is.date = TRUE)
#' autoplot(breakpoints(bp.nile, breaks = 2), data = Nile)
#' @export
autoplot.breakpoints <- function(object, data = NULL,
                                 cpt.colour = '#FF0000', cpt.linetype = 'dashed',
                                 ...) {
  plot.data <- ggplot2::fortify(object, data = data)
  y <- 'Data'
  if (y %in% names(plot.data)) {
    p <- autoplot.ts(plot.data, columns = y, ...)
  } else {
    stop("'data' is mandatory for plotting breakpoints instance")
  }

  d <- dplyr::filter(plot.data, !is.na(Breaks))
  p <- p + ggplot2::geom_vline(xintercept = as.integer(d$Index),
                               colour = cpt.colour, linetype = cpt.linetype)
  p
}

#' @export
fortify.breakpointsfull <- fortify.breakpoints

#' Convert \code{vars::varprd} to \code{data.frame}
#'
#' @param model \code{vars::varprd} instance
#' @inheritParams fortify_base
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @param melt Logical flag indicating whether to melt each timeseries as variable
#' @return data.frame
#' @examples
#' data(Canada, package = 'vars')
#' d.var <- vars::VAR(Canada, p = 3, type = 'const')
#' fortify(stats::predict(d.var, n.ahead = 50))
#' @export
fortify.varprd <- function(model, data = NULL, is.date = NULL,
                           ts.connect = FALSE, melt = FALSE, ...){
  fitted <- ggplot2::fortify(model$model$y)

  fcst <- model$fcst
  dtindex.cont <- get.dtindex.continuous(model$model$y, length = nrow(fcst[[1]]),
                                         is.date = is.date)
  cols <- names(fcst)

  if (melt) {
    # for autoplot conversion
    for (col in cols){
      pred <- data.frame(fcst[[col]])
      pred$Index <- dtindex.cont
      obs <- fitted[, c('Index', col)]
      colnames(obs) <- c('Index', 'Data')
      binded <- ggfortify::rbind_ts(pred, obs, ts.connect = ts.connect)
      binded$variable <- col
      fcst[[col]] <- binded
    }
    return(dplyr::bind_rows(fcst))
  } else {
    for (col in cols){
      colnames(fcst[[col]]) <- paste0(col, '.', colnames(fcst[[col]]))
    }
    pred <- data.frame(do.call(cbind, fcst))
    pred$Index <- dtindex.cont
    binded <- ggfortify::rbind_ts(pred, fitted, ts.connect = ts.connect)
    return(binded)
  }
}

#' Autoplot \code{vars::varprd}
#'
#' @param object \code{vars::varpred} instance
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @param scales Scale value passed to \code{ggplot2}
#' @inheritParams autoplot.tsmodel
#' @inheritParams plot_confint
#' @param ... other arguments passed to \code{autoplot.ts}
#' @return ggplot
#' @examples
#' data(Canada, package = 'vars')
#' d.var <- vars::VAR(Canada, p = 3, type = 'const')
#' autoplot(stats::predict(d.var, n.ahead = 50), is.date = TRUE)
#' autoplot(stats::predict(d.var, n.ahead = 50), conf.int = FALSE)
#' @export
autoplot.varprd <- function(object, is.date = NULL, ts.connect = TRUE,
                            scales = 'free_y',
                            predict.geom = 'line',
                            predict.colour = '#0000FF', predict.size = NULL,
                            predict.linetype = NULL, predict.alpha = NULL,
                            predict.fill = NULL, predict.shape = NULL,
                            conf.int = TRUE,
                            conf.int.colour = '#0000FF', conf.int.linetype = 'none',
                            conf.int.fill = '#000000', conf.int.alpha = 0.3,
                            ...) {

  plot.data <- ggplot2::fortify(object, is.date = is.date,
                                ts.connect = ts.connect, melt = TRUE)

  # Filter existing values to avoid warnings
  original.data <- dplyr::filter(plot.data, !is.na(Data))
  predict.data <- dplyr::filter(plot.data, !is.na(fcst))

  p <- autoplot.ts(original.data, columns = 'Data', ...)

  p <- autoplot.ts(predict.data, columns = 'fcst', p = p,
                   geom = predict.geom,
                   colour = predict.colour, size = predict.size,
                   linetype = predict.linetype, alpha = predict.alpha,
                   fill = predict.fill, shape = predict.shape)

  p <- p + ggplot2::facet_grid(variable ~ ., scales = scales)

  p <- plot_confint(p = p, data = predict.data, conf.int = conf.int,
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  p
}

# Avoid false positives in R CMD CHECK:
utils::globalVariables(c("ACF", "plot_group", "Breaks", "variance", "plot_group",
	"Data", "Point_Forecast", ".resid", ".wresid", ".cooksd", "fcst", "variable",
	"value", "se", "Spline", "x", "y", "Frequency"))

#' Convert \code{ROCR::performance} objects to \code{data.frame}
#'
#' @param model \code{performance} instances
#' @inheritParams fortify_base
#' @return data.frame
#' @export
fortify.performance <- function(model, data = NULL, ...) {
  if (is(model, 'performance')) {

    ## Get metadata

    # number of repetitions (bootstrapped, CV, etc)
    n_rep <- length(model@y.values)
    # repetition names
    rep_names <- paste0('Rep', seq(n_rep))
    # length of each repetition
    rep_lengths <- vapply(model@y.values, length, integer(1))

    ## Extract values

    # unlist in long form
    df <- data.frame(y = unlist(model@y.values))
    # label each data point with its repetition name
    df$repn <- as.factor(rep(rep_names, rep_lengths))
    # add x and alpha, which may or may not be there
    df$x <- unlist(model@x.values)
    df$alpha <- unlist(model@alpha.values)

    # Rename/reorder cols
    renames <- c(repn = 'Repetition.Number',
                 x = make.names(model@x.name),
                 y = make.names(model@y.name),
                 alpha = make.names(model@alpha.name))

    names(df) <- renames[names(df)]
    renames <- intersect(renames, names(df))
    df <- df[, renames]

  } else {
    stop(paste0('Unsupported class for fortify.performance: ', class(model)))
  }

  post_fortify(df, klass = model)
}




#' Autoplot \code{ROCR::performance}
#'
#' @param object \code{ROCR::performance} instance
#' @param p \code{ggplot2::ggplot} instances
#' @param bins If \code{object} represents a measure whose value is just a
#'        scalar (e.g. \code{performance(predObj, 'auc')}), a histogram will be
#'        plotted of this scalar's values for different runs. \code{bins}
#'        is the number of bins for this histogram.
#' @param ... other arguments passed to methods
#' @return ggplot
#' @export
autoplot.performance <- function(object, p = NULL,
                                 bins = 5, ...) {

  plot.data <- ggplot2::fortify(object)
  plot.names <- names(plot.data)

  # Three different plots depending on how many columns there are (2, 3, or 4)

  if (length(plot.names) == 2) {

    if (nrow(plot.data) == 1) {
      warning(paste('This histogram is more useful with multiple runs.',
                    'See ?ROCR::prediction'))
    }

    if (is.null(p)) {
      p <- geom_factory(ggplot2::ggplot, data = plot.data,
                        x = plot.names[2])
    }

    p <- p + geom_factory(ggplot2::geom_histogram, data = plot.data,
                          x = plot.names[2], bins = bins)
    p <- p + ggplot2::ggtitle(paste('Histogram of', plot.names[2]))

  } else if (length(plot.names) == 3) {

    if (is.null(p)) {
      p <- geom_factory(ggplot2::ggplot, data = plot.data,
                        x = plot.names[2], y = plot.names[3],
                        group = plot.names[1])
    }

    p <- p + geom_factory(ggplot2::geom_line, data = plot.data,
                          x = plot.names[2], y = plot.names[3],
                          group = plot.names[1])
    p <- p + ggplot2::ggtitle(paste(plot.names[3], 'vs', plot.names[2]))

  } else if (length(plot.names) == 4) {

    if (is.null(p)) {
      p <- geom_factory(ggplot2::ggplot, data = plot.data,
                        x = plot.names[2], y = plot.names[3],
                        group = plot.names[1], col = plot.names[4])
    }

    p <- p + geom_factory(ggplot2::geom_line, data = plot.data,
                          x = plot.names[2], y = plot.names[3],
                          group = plot.names[1], col = plot.names[4])
    p <- p + ggplot2::ggtitle(paste(plot.names[3], 'vs', plot.names[2]))

  }

  p <- post_autoplot(p = p, ...)
  return(p)
}

## ----global_options, include=FALSE--------------------------------------------
library(knitr)
opts_chunk$set(fig.width=3, fig.height=3, fig.path='figures/map-', warning=FALSE)

## ---- message = FALSE---------------------------------------------------------
library(mapdata)
library(ggplot2)

jp <- ggplot2::map_data('world2', 'japan')
class(jp)
head(jp)
ggplot(jp, aes(x = long, y = lat, group = group)) +
  geom_polygon()

## ---- message = FALSE---------------------------------------------------------
library(ggfortify)
jp <-  map('world2', 'japan', plot = FALSE, fill = TRUE)
class(jp)
autoplot(jp)

p <- autoplot(jp, geom = 'polygon', fill = 'subregion') + 
  theme(legend.position="none")
p

## ---- message = FALSE---------------------------------------------------------
cities <- get('world.cities')
cities <- cities[cities$country.etc == 'Japan', ]
head(cities)

p + geom_point(data = cities, aes(x = long, y = lat),
               colour = 'blue', size = 0.1)

## ---- message = FALSE---------------------------------------------------------
p + geom_point(data = cities, colour = 'blue', size = 0.1)

## ---- message = FALSE---------------------------------------------------------
library(sp)
df <- data.frame(long = c(139.691704, 135.519711),
                 lat = c(35.689521, 34.686316),
                 label = c('Tokyo', 'Osaka'),
                 population = c(1335, 886))
coordinates(df) <- ~ long + lat
class(df)
autoplot(df, p = p, colour = 'red', size = 10)

## ---- message = FALSE---------------------------------------------------------
autoplot(df, p = p, colour = 'red', size = 'population') +
  scale_size_area()

## ---- message = FALSE, eval = FALSE-------------------------------------------
#  library(ggmap)
#  bbox <- c(130.0, 30.0, 145.0, 45.0)
#  map <- get_openstreetmap(bbox = bbox, scale = 47500000)
#  p <- ggmap(map)
#  autoplot(df, p = p, colour = 'red', size = 'population') +
#    scale_size_area() +
#    theme(legend.justification = c(1, 0), legend.position = c(1, 0))

#' ggfortify
#'
#' Define Fortify and Autoplot to Allow 'ggplot2' to Draw Some Popular Packages
#'
#' @name ggfortify
#' @docType package
#' @import methods ggplot2 dplyr
#' @importFrom tidyr gather_
#' @importFrom gridExtra grid.arrange
#' @importFrom utils installed.packages
#' @importFrom stats complete.cases residuals weights as.formula reshape coef sd
#' @importFrom stringr str_wrap
NULL

#' Convert time-series-like to data.frame
#'
#' @param model time-series-like instance
#' @inheritParams fortify_base
#' @param columns character vector specifies target column name(s)
#' @param is.date logical frag indicates whether the \code{stats::ts} is date or not
#' If not provided, regard the input as date when the frequency is 4 or 12
#' @param index.name specify column name for time series index
#' @param data.name specify column name for univariate time series data. Ignored in multivariate time series.
#' @param scale logical flag indicating whether to perform scaling each timeseries
#' @param melt logical flag indicating whether to melt each timeseries as variable
#' @return data.frame
#' @examples
#' fortify(AirPassengers)
#' fortify(timeSeries::as.timeSeries(AirPassengers))
#' fortify(tseries::irts(cumsum(rexp(10, rate = 0.1)), matrix(rnorm(20), ncol=2)))
#' fortify(stats::stl(UKgas, s.window = 'periodic'))
#' fortify(stats::decompose(UKgas))
#' @export
fortify.ts <- function(model, data = NULL, columns = NULL, is.date = NULL,
                       index.name = 'Index', data.name = 'Data',
                       scale = FALSE, melt = FALSE, ...) {
  # no need to define `fortify.xts` because zoo package has `fortify.zoo`
  if (is(model, 'timeSeries')) {
    d <- as.data.frame(model)
    dtindex <- as.POSIXct(rownames(d))
  } else if (is(model, 'irts')) {
    d <- as.data.frame(model$value)
    dtindex <- model$time
  } else if (is(model, 'ts')) {
    d <- as.data.frame(as.matrix(model))
    dtindex <- get.dtindex(model, is.date = is.date)
  } else if (is(model, 'stl')) {
      # stl allows only univariate series
      ts.data <- model$time.series
      orig <- drop(ts.data %*% rep(1, ncol(ts.data)))

      dtindex <- get.dtindex(ts.data, is.date = is.date)
      d <- cbind(data.frame(Data = orig),
                 data.frame(model$time.series))
    } else if (is(model, 'decomposed.ts')) {
      dtindex <- get.dtindex(model$x, is.date = is.date)
      dtframe <- ggplot2::fortify(model$x)

      # for tbl_df
      dtframe <- data.frame(Data = dtframe[['Data']])

      # trend and random can be multivariate
      rndframe <- model$random
      colnames(rndframe) <- NULL
      dcframe <- data.frame(seasonal = model$seasonal,
                            trend = model$trend,
                            remainder = rndframe)
      d <- cbind(dtframe, dcframe)
  } else {
    stop(paste0('Unsupported class for fortify.ts: ', class(model)))
  }
  dtframe <- data.frame(Index = dtindex)
  colnames(dtframe) <- index.name
  if (ncol(d) == 1) {
    colnames(d) <- data.name
  }
  d <- cbind(dtframe, d)

  # filtering columns
  if (is.null(columns)) {
    data.names <- names(d)
    columns <- data.names[data.names != index.name]
  } else {
    d <- dplyr::select_(d, .dots = c(index.name, columns))
  }

  # scaling
  if (scale) {
    for (col in columns) {
      d[[col]] <- base::scale(d[[col]], center = TRUE, scale = TRUE)
    }
  }

  # unpivot
  if (melt) {
    d <- tidyr::gather_(d, 'variable', 'value', columns)
  }
  post_fortify(d)
}

#' @export
fortify.timeSeries <- fortify.ts

#' @export
fortify.irts <- fortify.ts

#' Autoplot time-series-like
#'
#' @param object time-series-like instance
#' @param columns Character vector specifies target column name(s)
#' @param group Character vector specifies grouping
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not
#' If not provided, regard the input as date when the frequency is 4 or 12
#' @param index.name Specify column name for time series index when passing \code{data.frame} via data.
#' @param p \code{ggplot2::ggplot} instance
#' @param ts.scale Logical flag indicating whether to perform scaling each timeseries
#' @param stacked Logical flag indicating whether to stack multivariate timeseries
#' @inheritParams apply_facets
#' @param ts.geom geometric string for time-series. 'line', 'bar', 'ribbon', or 'point'
#' @param ts.colour line colour for time-series
#' @param ts.size point size for time-series
#' @param ts.linetype line type for time-series
#' @param ts.alpha alpha for time-series
#' @param ts.fill fill colour for time-series
#' @param ts.shape point shape for time-series
#' @param geom same as ts.geom
#' @param colour same as ts.colour
#' @param size same as ts.size
#' @param linetype same as ts.linetype
#' @param alpha same as ts.alpha
#' @param fill same as ts.fill
#' @param shape same as ts.shape
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @aliases autoplot.xts autoplot.timeSeries autoplot.irts autoplot.stl autoplot.decomposed.ts
#' @examples
#' \dontrun{
#' data(Canada, package = 'vars')
#' autoplot(AirPassengers)
#' autoplot(UKgas, ts.geom = 'bar')
#' autoplot(Canada)
#' autoplot(Canada, facets = FALSE)
#'
#' library(zoo)
#' autoplot(xts::as.xts(AirPassengers))
#' autoplot(timeSeries::as.timeSeries(AirPassengers))
#' its <- tseries::irts(cumsum(rexp(10, rate = 0.1)), matrix(rnorm(20), ncol=2))
#' autoplot(its)
#'
#' autoplot(stats::stl(UKgas, s.window = 'periodic'))
#' autoplot(stats::decompose(UKgas))
#' }
#' @export
autoplot.ts <- function(object, columns = NULL, group = NULL,
                        is.date = NULL, index.name = 'Index',
                        p = NULL, ts.scale = FALSE, stacked = FALSE,
                        facets = TRUE, nrow = NULL, ncol = 1, scales = 'free_y',
                        ts.geom = 'line', ts.colour = NULL, ts.size = NULL, ts.linetype = NULL,
                        ts.alpha = NULL, ts.fill = NULL, ts.shape = NULL,
                        geom = ts.geom, colour = ts.colour, size = ts.size, linetype = ts.linetype,
                        alpha = ts.alpha, fill = ts.fill, shape = ts.shape,
                        xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                        main = NULL, xlab = '', ylab = '', asp = NULL,
                        ...) {
  geomfunc <- get_geom_function(geom, allowed = c('line', 'bar', 'point', 'ribbon'))

  # fortify data
  if (is.data.frame(object)) {
    plot.data <- object
  } else {
    plot.data <- ggplot2::fortify(object, scale = ts.scale,
                                  is.date = is.date, index.name = index.name)
  }

  if (is.null(columns)) {
    if (is(object, 'bats') || is(object, 'ets')) {
      # for forecast::bats and forecast::ets
      columns <- c('Data', 'Level', 'Slope', 'Season')
      # Slope and Season can be optionals
      columns <- columns[columns %in% names(plot.data)]
    } else {
      data.names <- names(plot.data)
      columns <- data.names[data.names != index.name]
    }
  }

  if (is.null(colour) && !is.null(fill)) {
    colour <- fill
  } else if (!is.null(colour) && is.null(fill)) {
    if (geom %in% c('bar', 'ribbon')) {
      # do not set for line / point to handle as NULL in geom_factory
      fill <- colour
    }
  }

  if (length(columns) > 1) {
    .is.univariate <- FALSE
  } else {
    .is.univariate <- TRUE
    facets <- FALSE
    stacked <- FALSE
  }
  # required for shift op
  tslen <- nrow(plot.data)

  if (!facets && stacked && geom != 'bar') {
    for (i in seq_len(length(columns) - 1)) {
      plot.data[columns[i + 1]] <- plot.data[columns[i + 1]] + plot.data[columns[i]]
    }
  }
  # must be done here, because fortify.zoo is defined in zoo package
  ts.column <- plot.data[[index.name]]
  if (is(ts.column, 'yearmon') || is(ts.column, 'yearqtr')) {
    plot.data[[index.name]] <- zoo::as.Date(plot.data[[index.name]])
  }

  group_key <- 'plot_group'  # gets used later. Don't know why its defined here.
  plot.data <- tidyr::pivot_longer(plot.data, names_to=group_key, values_to='value', columns) %>%
    arrange(plot_group)      # somewhere later the sort order matters.

  # create ggplot instance if not passed
  if (is.null(p)) {
    null.p <- TRUE
    mapping <- ggplot2::aes_string(x = index.name)
    p <- ggplot2::ggplot(data = plot.data, mapping = mapping)
  } else {
    null.p <- FALSE
  }

  if (!facets && stacked) {
    # using dplyr::lag may be easier, but it likely to
    # cause a trouble in CMD check
    value <- plot.data$value
    shifted <- c(rep(0, times = tslen), value[1:(length(value) - tslen)])
    plot.data[['base']] <- shifted
  } else {
    plot.data[['base']] <- 0
  }

  args <- list(geomfunc, plot.data, colour = colour, size = size,
               linetype = linetype, alpha = alpha, fill = fill,
               shape = shape, stat = 'identity')
  if (geom == 'ribbon') {
    args['ymin'] <- 'base'
    args['ymax'] <- 'value'
  } else {
    args['y'] <- 'value'
  }

  if (facets) {
    args['group'] <- group_key
    p <- p + do.call(geom_factory, args)
    p <- apply_facets(p, ~ plot_group, nrow = nrow, ncol = ncol, scales = scales)
  } else {
    if (!.is.univariate) {
      # ts.colour cannot be used
      if (!is.null(colour)) {
        warning('multivariate timeseries with facets=FALSE are colorized by variable, colour is ignored')
      }
      args['colour'] <- group_key
      if (geom %in% c('bar', 'ribbon')) {
        args['fill'] <- group_key
      }
      if (geom == 'ribbon' && !stacked && is.null(alpha)) {
        args['alpha'] <- 0.5
      }
      if (geom == 'bar' && !stacked) {
        args['position'] <- 'dodge'
      }
    }
    p <- p + do.call(geom_factory, args)
  }
  if (null.p) {
    p <- p + ggplot2::scale_y_continuous()
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' @export
autoplot.zooreg <- autoplot.ts

#' @export
autoplot.xts <- autoplot.ts

#' @export
autoplot.timeSeries <- autoplot.ts

#' @export
autoplot.irts <- autoplot.ts

#' Convert time series models (like AR, ARIMA) to \code{data.frame}
#'
#' @param model Time series model instance
#' @param data original dataset, needed for \code{stats::ar}, \code{stats::Arima}
#' @inheritParams fortify_base
#' @param predict Predicted \code{stats::ts}
#' If not provided, try to retrieve from current environment using variable name.
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @return data.frame
#' @aliases fortify.ar fortify.Arima fortify.fracdiff
#' fortify.nnetar fortify.HoltWinters fortify.fGARCH
#' @examples
#' fortify(stats::ar(AirPassengers))
#' fortify(stats::arima(UKgas))
#' fortify(stats::arima(UKgas), data = UKgas, is.date = TRUE)
#' fortify(forecast::auto.arima(austres))
#' fortify(forecast::arfima(AirPassengers))
#' fortify(forecast::nnetar(UKgas))
#' fortify(stats::HoltWinters(USAccDeaths))
#'
#' data(LPP2005REC, package = 'timeSeries')
#' x = timeSeries::as.timeSeries(LPP2005REC)
#' d.Garch = fGarch::garchFit(LPP40 ~ garch(1, 1), data = 100 * x, trace = FALSE)
#' fortify(d.Garch)
fortify.tsmodel <- function(model, data = NULL,
                            predict = NULL,
                            is.date = NULL,
                            ts.connect = TRUE, ...) {

  if (is(model, 'Arima') || is(model, 'ar')) {
    if (is.null(data)) {
      data <- forecast::getResponse(model)
      fit <- stats::fitted(model)
    } else {
      fit <- data - stats::residuals(model)
    }
    d <- ggplot2::fortify(data, is.date = is.date)
    fit <- ggplot2::fortify(fit, data.name = 'Fitted', is.date = is.date)
    resid <- ggplot2::fortify(stats::residuals(model), data.name = 'Residuals', is.date = is.date)

    if (!is.null(predict)) {
      pred <- ggplot2::fortify(predict$pred, data.name = 'Predicted')
      se <- as.vector(predict$se)
      pred$lower <- pred$Predicted - se
      pred$upper <- pred$Predicted + se
    }
  } else if (is(model, 'HoltWinters')) {
    # same as fracdiff and nnetar
    d <- ggplot2::fortify(model$x, is.date = is.date)
    fit <- ggplot2::fortify(stats::fitted(model), data.name = 'Fitted', is.date = is.date)
    resid <- ggplot2::fortify(stats::residuals(model), data.name = 'Residuals', is.date = is.date)

    if (!is.null(predict)) {
      pred <- ggplot2::fortify(predict)
      if (! 'upr' %in% names(pred)) {
        pred$upr <- pred$Data
        pred$lwr <- pred$Data
      }
      colnames(pred) <- c('Index', 'Predicted', 'upper', 'lower')
    }
  } else if (is(model, 'fracdiff') || is(model, 'nnetar')) {
    d <- ggplot2::fortify(model$x, is.date = is.date)
    fit <- ggplot2::fortify(stats::fitted(model), data.name = 'Fitted', is.date = is.date)
    resid <- ggplot2::fortify(stats::residuals(model), data.name = 'Residuals', is.date = is.date)
  } else if (is(model, 'fGARCH')) {
    index <- attr(model@data, 'names')
    index <- as.vector(index)
    d <- data.frame(Index = index, Data = model@data)
    fit <- data.frame(Index = index, Fitted = model@fitted)
    resid <- data.frame(Index = index, Residuals = model@residuals)

    if (!is.null(predict)) {
      pred <- data.frame(Predicted = predict$meanForecast)
      pred$lower <- pred$Predicted - predict$meanError
      pred$upper <- pred$Predicted + predict$meanError
    }
  } else if (is(model, 'dlmFiltered')) {
    d <- ggplot2::fortify(model$y, is.date = is.date)
    m <- dlm::dropFirst(model$m)
    if (!is.univariate(m, raise = FALSE)) {
      m <- m[, 1]
    }
    fit <- ggplot2::fortify(m, data.name = 'Fitted', is.date = is.date)
    resid <- ggplot2::fortify(model$y - m, data.name = 'Residuals', is.date = is.date)
  } else if (is(model, 'KFS')) {
    d <- ggplot2::fortify(model$model$y, is.date = is.date)
    m <- model$alphahat
    if (is.null(m)) {
      m <- model$m
      if (is.null(m)) {
        stop('Object does not contain smoothed estimates of states.')
      }
      m[1] <- model$model$y[1]
    }
    fit <- ggplot2::fortify(m, data.name = 'Fitted', is.date = is.date)
    resid <- ggplot2::fortify(model$model$y - m,
                              data.name = 'Residuals', is.date = is.date)
  } else {
    stop(paste0('Unsupported class for fortify.Arima: ', class(model)))
  }
  d <- dplyr::left_join(d, fit, by = 'Index')
  d <- dplyr::left_join(d, resid, by = 'Index')
  if (!is.null(predict)) {
    d <- rbind_ts(pred, d, ts.connect = ts.connect)
  }
  post_fortify(d)
}

#' @export
fortify.ar <- fortify.tsmodel

#' @export
fortify.Arima <- fortify.tsmodel

#' @export
fortify.tsmodel <- fortify.tsmodel

#' @export
fortify.HoltWinters <- fortify.tsmodel

#' @export
fortify.fracdiff <- fortify.tsmodel

#' @export
fortify.nnetar <- fortify.tsmodel

#' @export
fortify.fGARCH <- fortify.tsmodel

#' @export
fortify.dlmFiltered <- fortify.tsmodel

#' @export
fortify.KFS <- fortify.tsmodel

#' Autoplot time series models (like AR, ARIMA)
#'
#' @param object Time series model instance
#' @param data original dataset, needed for \code{stats::ar}, \code{stats::Arima}
#' @param predict Predicted \code{stats::ts}
#' If not provided, try to retrieve from current environment using variable name.
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @param fitted.geom geometric string for fitted time-series
#' @param fitted.colour line colour for fitted time-series
#' @param fitted.size point size for fitted time-series
#' @param fitted.linetype line type for fitted time-series
#' @param fitted.alpha alpha for fitted time-series
#' @param fitted.fill fill colour for fitted time-series
#' @param fitted.shape point shape for fitted time-series
#' @param predict.geom geometric string for predicted time-series
#' @param predict.colour line colour for predicted time-series
#' @param predict.size point size for predicted time-series
#' @param predict.linetype line type for predicted time-series
#' @param predict.alpha alpha for predicted time-series
#' @param predict.fill fill colour for predicted time-series
#' @param predict.shape point shape for predicted time-series
#' @inheritParams plot_confint
#' @param ... Keywords passed to \code{autoplot.ts}
#' @return ggplot
#' @aliases autoplot.ar autoplot.fracdiff autoplot.nnetar autoplot.HoltWinters autoplot.fGARCH
#' @examples
#' d.ar <- stats::ar(AirPassengers)
#' autoplot(d.ar)
#' autoplot(d.ar, predict = predict(d.ar, n.ahead = 5))
#' autoplot(stats::arima(UKgas), data = UKgas)
#' autoplot(forecast::arfima(AirPassengers))
#' autoplot(forecast::nnetar(UKgas), is.date = FALSE)
#'
#' d.holt <- stats::HoltWinters(USAccDeaths)
#' autoplot(d.holt)
#' autoplot(d.holt, predict = predict(d.holt, n.ahead = 5))
#' autoplot(d.holt, predict = predict(d.holt, n.ahead = 5, prediction.interval = TRUE))
#' @export
autoplot.tsmodel <- function(object, data = NULL,
                             predict = NULL,
                             is.date = NULL, ts.connect = TRUE,
                             fitted.geom = 'line',
                             fitted.colour = '#FF0000', fitted.size = NULL,
                             fitted.linetype = NULL, fitted.alpha = NULL,
                             fitted.fill = NULL, fitted.shape = NULL,
                             predict.geom = 'line',
                             predict.colour = '#0000FF', predict.size = NULL,
                             predict.linetype = NULL, predict.alpha = NULL,
                             predict.fill = NULL, predict.shape = NULL,
                             conf.int = TRUE,
                             conf.int.colour = '#0000FF', conf.int.linetype = 'none',
                             conf.int.fill = '#000000', conf.int.alpha = 0.3,
                             ...) {
  fcol <- ifelse(is(object, 'HoltWinters'), 'xhat', 'Fitted')
  plot.data <- ggplot2::fortify(object, predict = predict,
                                data = data, is.date = is.date)
  p <- autoplot.ts(plot.data, columns = 'Data', ...)

  # must be passed by ts.<option>
  p <- autoplot.ts(plot.data, columns = fcol, p = p,
                   ts.geom = fitted.geom,
                   ts.colour = fitted.colour, ts.size = fitted.size,
                   ts.linetype = fitted.linetype, ts.alpha = fitted.alpha,
                   ts.fill = fitted.fill, ts.shape = fitted.shape)
  if (!is.null(predict)) {
    predict.data <- dplyr::filter_(plot.data, '!is.na(Predicted)')
    p <- autoplot.ts(predict.data, columns = 'Predicted', p = p,
                     ts.geom = predict.geom,
                     ts.colour = predict.colour, ts.size = predict.size,
                     ts.linetype = predict.linetype, ts.alpha = predict.alpha,
                     ts.fill = predict.fill, ts.shape = predict.shape)
    p <- plot_confint(p = p, data = predict.data, conf.int = conf.int,
                      conf.int.colour = conf.int.colour,
                      conf.int.linetype = conf.int.linetype,
                      conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  }
  p
}

#' @export
autoplot.ar <- autoplot.tsmodel

#' @export
autoplot.Arima <- autoplot.tsmodel

#' @export
autoplot.HoltWinters <- autoplot.tsmodel

#' @export
autoplot.fracdiff <- autoplot.tsmodel

#' @export
autoplot.nnetar <- autoplot.tsmodel

#' @export
autoplot.fGARCH <- autoplot.tsmodel

#' @export
autoplot.dlmFiltered <- autoplot.tsmodel

#' @export
autoplot.KFS <- autoplot.tsmodel

## ----global_options, include=FALSE, eval=FALSE--------------------------------
#  library(knitr)
#  opts_chunk$set(fig.width=8, fig.height=6, fig.path='figures/lm-', warning=FALSE)

## ---- message = FALSE, fig.width = 5, fig.height = 5, eval=FALSE--------------
#  library(ggfortify)
#  autoplot(lm(Petal.Width ~ Petal.Length, data = iris), label.size = 3)

## ---- message = FALSE, fig.width = 7, fig.height = 5, eval=FALSE--------------
#  par(mfrow = c(1, 2))
#  m <- lm(Petal.Width ~ Petal.Length, data = iris)
#  
#  autoplot(m, which = 1:6, ncol = 3, label.size = 3)

## ---- message = FALSE, fig.width = 5, fig.height = 6, eval=FALSE--------------
#  m <- glm(Murder ~ Assault + UrbanPop + Rape,
#           family = gaussian, data = USArrests)
#  
#  autoplot(m, which = 1:6, label.size = 3)

## ---- message = FALSE, fig.width = 5, fig.height = 5, eval=FALSE--------------
#  class(autoplot(m))
#  autoplot(m, label.size = 3) + theme_bw()

## ---- message = FALSE, fig.width = 8, fig.height = 5, eval=FALSE--------------
#  autoplot(m, which = 1:6, colour = 'dodgerblue3',
#           smooth.colour = 'black', smooth.linetype = 'dashed',
#           ad.colour = 'blue',
#           label.size = 3, label.n = 5, label.colour = 'blue',
#           ncol = 3)

## ---- message = FALSE, fig.width = 8, fig.height = 5, eval=FALSE--------------
#  autoplot(lm(Petal.Width ~ Petal.Length, data = iris), data = iris,
#           colour = 'Species', label.size = 3)

## ---- message = FALSE, fig.width = 5, fig.height = 3, eval=FALSE--------------
#  library(glmnet)
#  data(QuickStartExample)
#  fit <- glmnet::glmnet(x, y)
#  autoplot(fit)
#  
#  fit <- glmnet::cv.glmnet(x, y)
#  autoplot(fit, colour = 'blue')

#' Convert \code{forecast::forecast} to \code{data.frame}
#'
#' @param model \code{forecast::forecast} instance
#' @inheritParams fortify_base
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @return data.frame
#' @export
#' @examples
#' d.arima <- forecast::auto.arima(AirPassengers)
#' d.forecast <- forecast::forecast(d.arima, level = c(95), h = 50)
#' fortify(d.forecast)
#' fortify(d.forecast, ts.connect = TRUE)
fortify.forecast <- function(model, data = NULL, is.date = NULL,
                             ts.connect = FALSE, ...) {
  forecasted <- as.data.frame(model)
  forecasted$Index <- get.dtindex(model$mean, is.date = is.date)

  d <- ggplot2::fortify(model$x, is.date = is.date)
  fitted <- ggplot2::fortify(model$fitted, data.name = 'Fitted', is.date = is.date)
  d <- dplyr::left_join(d, fitted, by = 'Index')
  d <- ggfortify::rbind_ts(forecasted, d, ts.connect = ts.connect)
  post_fortify(d)
}

#' Autoplot \code{forecast::forecast}
#'
#' @param object \code{forecast::forecast} instance
#' @inheritParams autoplot.tsmodel
#' @inheritParams plot_confint
#' @param ... other arguments passed to \code{autoplot.ts}
#' @return ggplot
#' @examples
#' d.arima <- forecast::auto.arima(AirPassengers)
#' autoplot(forecast::forecast(d.arima, h = 10))
#' autoplot(forecast::forecast(d.arima, level = c(85), h = 10))
#' autoplot(forecast::forecast(d.arima, h = 5), conf.int = FALSE, is.date = FALSE)
#' autoplot(forecast::forecast(stats::HoltWinters(UKgas), h = 10))
#' \dontrun{
#' autoplot(forecast::forecast(forecast::ets(UKgas), h = 5))
#' }
#' @export
autoplot.forecast <- function(object, is.date = NULL, ts.connect = TRUE,
                              predict.geom = 'line',
                              predict.colour = '#0000FF', predict.size = NULL,
                              predict.linetype = NULL, predict.alpha = NULL,
                              predict.fill = NULL, predict.shape = NULL,
                              conf.int = TRUE,
                              conf.int.colour = '#0000FF', conf.int.linetype = 'none',
                              conf.int.fill = '#000000', conf.int.alpha = 0.3,
                              ...) {
  plot.data <- ggplot2::fortify(object, is.date = is.date, ts.connect = ts.connect)
  # replace whitespace to underscore to make column name handling easie
  colnames(plot.data) <- sub(' ', '_', colnames(plot.data))
  lower <- 'Lo_95'  # prioritize to use 95%
  upper <- 'Hi_95'

  if (! 'Lo 95' %in% names(plot.data)) {
    # escape by backquote
    lower <- names(plot.data)[5]
  }
  if (! 'Hi 95' %in% names(plot.data)) {
    upper <- names(plot.data)[6]
  }

  # Filter existing values to avoid warnings
  original.data <- dplyr::filter(plot.data, !is.na(Data))
  predict.data <- dplyr::filter(plot.data, !is.na(`Point_Forecast`))

  p <- autoplot.ts(original.data, columns = 'Data', ...)
  p <- autoplot.ts(predict.data, columns = 'Point_Forecast', p = p,
                   ts.geom = predict.geom,
                   ts.colour = predict.colour, ts.size = predict.size,
                   ts.linetype = predict.linetype, ts.alpha = predict.alpha,
                   ts.fill = predict.fill, ts.shape = predict.shape)
  p <- plot_confint(p = p, data = predict.data, lower = lower,
                    upper = upper, conf.int = conf.int,
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  p
}

#' Convert \code{forecast::bats} and \code{forecast::ets} to \code{data.frame}
#'
#' @param model \code{forecast::bats} or \code{forecast::ets} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' \dontrun{
#' fortify(forecast::bats(UKgas))
#' fortify(forecast::ets(UKgas))
#' }
#' @export
fortify.ets <- function(model, data = NULL, ...) {
  if (is(model, 'ets')) {
    d <- ggplot2::fortify(model$x)
    resid <- ggplot2::fortify(model$residuals, data.name = 'Residuals')
    fitted <- ggplot2::fortify(model$fitted, data.name = 'Fitted')
    d <- dplyr::left_join(d, fitted, by = 'Index')
    d <- dplyr::left_join(d, resid, by = 'Index')

    level <- ggplot2::fortify(model$states[, 'l'], data.name = 'Level')
    d <- dplyr::left_join(d, level, by = 'Index')
    if ('b' %in% colnames(model$states)) {
      slope <- ggplot2::fortify(model$states[, 'b'], data.name = 'Slope')
      d <- dplyr::left_join(d, slope, by = 'Index')
    }
    if ('s1' %in% colnames(model$states)) {
      season <- ggplot2::fortify(model$states[, 's1'], data.name = 'Season')
      d <- dplyr::left_join(d, season, by = 'Index')
    }
  } else if (is(model, 'bats')) {
    if (!is.null(model$lambda))
      y <- forecast::BoxCox(model$y, model$lambda)
    else y <- model$y
    d <- ggplot2::fortify(y)
    resid <- ggplot2::fortify(model$errors, data.name = 'Residuals')
    fitted <- ggplot2::fortify(model$fitted.values, data.name = 'Fitted')
    d <- dplyr::left_join(d, fitted, by = 'Index')
    d <- dplyr::left_join(d, resid, by = 'Index')

    d <- cbind(d, Level = model$x[1, ])
    if (!is.null(model$beta))
      d <- cbind(d, Slope = model$x[2, ])

    nonseas <- 2 + (!is.null(model$beta))
    nseas <- length(model$gamma.values)
    if (!is.null(model$gamma)) {
      seas.states <- model$x[- (1:(1 + (!is.null(model$beta)))), ]
      j <- cumsum(c(1, model$seasonal.periods))
      for (i in 1:nseas) d <- cbind(d, Season = seas.states[j[i], ])
      if (nseas > 1)
        colnames(d)[nonseas + 1:nseas] <- paste0("Season", 1:nseas)
    }
  } else {
    stop(paste0('Unsupported class for fortify.ets: ', class(model)))
  }
  post_fortify(d)
}

#' @export
fortify.bats <- fortify.ets

#' @export
autoplot.ets <- autoplot.ts

#' @export
autoplot.bats <- autoplot.ts

#' Convert \code{raster} to \code{data.frame}
#'
#' @param model \code{raster} instances
#' @inheritParams fortify_base
#' @param maxpixels number of pixels for resampling
#' @param rename logical flag indicating whether to rename coordinates to long and lat
#' @return data.frame
#' @export
fortify.RasterCommon <- function(model, data = NULL, maxpixels = 100000,
                                rename = TRUE, ...) {
  if (is(model, 'RasterLayer')) {

    # Resample if necessary (raster too big)
    model <- raster::sampleRegular(model, maxpixels, asRaster = TRUE)
    # Get coordinates
    coords <- raster::xyFromCell(model, seq_len(raster::ncell(model)))

    ## Extract values
    dat <- as.data.frame(raster::getValues(model))

    dat <- cbind(coords, dat)
    names(dat)[3] <- names(model)
    df <- as.data.frame(dat)

  } else if (is(model, 'RasterBrick') || is(model, 'RasterStack')) {

    nl <- raster::nlayers(model)
    model <- raster::sampleRegular(model, maxpixels, asRaster = TRUE)
    coords <- raster::xyFromCell(model, seq_len(raster::ncell(model)))
    ## Extract values
    dat <- as.data.frame(raster::getValues(model))
    dat <- cbind(coords, dat)
    names(dat)[3:(nl + 2)] <- names(model)

    df <- as.data.frame(dat)

  } else {
    stop(paste0('Unsupported class for fortify.RasterCommon: ', class(model)))
  }

  if (rename) {
    colnames(df)[1:2] <- c("long", "lat")
  }

  post_fortify(df, klass = model)
}

#' @export
fortify.RasterLayer <- fortify.RasterCommon

#' @export
fortify.RasterBrick <- fortify.RasterCommon

#' @export
fortify.RasterStack <- fortify.RasterCommon

#' Autoplot \code{raster::raster}
#'
#' Only plot the first layer of the given raster
#' @param object \code{raster::raster} instance
#' @param p \code{ggplot2::ggplot} instance
#' @param raster.layer name of the layer to plot
#' @param alpha alpha
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @export
autoplot.RasterCommon <- function(object, raster.layer = NULL, p = NULL,
                                  alpha = NULL, xlim = c(NA, NA),
                                  ylim = c(NA, NA), log = "", main = NULL,
                                  xlab = '', ylab = '', asp = NULL, ...) {
  plot.data <- ggplot2::fortify(object)
  layer_names <- names(object)

  if (length(layer_names) > 1) {
    message(paste0("More than one layer, plotting the first one only: ",
                   layer_names[1]))
  }

  if (is.null(p)) {
    mapping <- ggplot2::aes_string(x = 'long', y = 'lat', fill = layer_names[1])
    p <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    p <- p + geom_factory(ggplot2::geom_tile, data = plot.data, alpha = alpha)
  } else {
    p <- p + geom_factory(ggplot2::geom_tile, data = plot.data,
                          x = 'long', y = 'lat', fill = layer_names[1],
                          alpha = alpha)
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' @export
autoplot.RasterLayer <- autoplot.RasterCommon

#' @export
autoplot.RasterBrick <- autoplot.RasterCommon

#' @export
autoplot.RasterStack <- autoplot.RasterCommon

#' Autoplot \code{stats::lm} and \code{stats::glm}
#'
#' @param object \code{stats::lm} instance
#' @param which If a subset of the plots is required, specify a subset of the numbers 1:6.
#' @param data original dataset, if needed
#' @param colour line colour
#' @param size point size
#' @param linetype line type
#' @param alpha alpha
#' @param fill fill colour
#' @param shape point shape
#' @param label.n Number of points to be laeled in each plot, starting with the most extreme
#' @inheritParams plot_label
#' @param smooth.colour Line colour for smoother lines
#' @param smooth.linetype Line type for smoother lines
#' @param ad.colour Line colour for additional lines
#' @param ad.linetype Line type for additional lines
#' @param ad.size Fill colour for additional lines
#' @param nrow Number of facet/subplot rows
#' @param ncol Number of facet/subplot columns
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' \dontrun{
#' autoplot(lm(Petal.Width ~ Petal.Length, data = iris))
#' autoplot(glm(Petal.Width ~ Petal.Length, data = iris), which = 1:6)
#' autoplot(lm(Petal.Width~Petal.Length, data = iris), data = iris, colour = 'Species')
#' }
#' @export
autoplot.lm <- function(object, which = c(1:3, 5), data = NULL,
                        colour = '#444444', size = NULL, linetype = NULL,
                        alpha = NULL, fill = NULL, shape = NULL,
                        label = TRUE, label.label = '.label',
                        label.colour = '#000000', label.alpha = NULL,
                        label.size = NULL, label.angle = NULL,
                        label.family = NULL, label.fontface = NULL,
                        label.lineheight = NULL,
                        label.hjust = NULL, label.vjust = NULL, label.repel = FALSE,
                        label.n = 3,
                        smooth.colour = '#0000FF', smooth.linetype = 'solid',
                        ad.colour = '#888888', ad.linetype = 'dashed', ad.size = .2,
                        nrow = NULL, ncol = NULL, ...) {
  # initialization
  p1 <- p2 <- p3 <- p4 <- p5 <- p6 <- NULL

  dropInf <- function(x, h) {
    if (any(isInf <- h >= 1)) {
      warning(gettextf("not plotting observations with leverage one:\n  %s",
                       paste(which(isInf), collapse = ", ")), call. = FALSE,
              domain = NA)
      x[isInf] <- NaN
    }
    x
  }

  show <- rep(FALSE, 6)
  show[which] <- TRUE

  # ggplot2 can't rescale residuals with type AsIs
  # See https://github.com/sinhrks/ggfortify/issues/202
  class(object$residuals) <- NULL

  if (is.null(data)) {
    # ggplot2::fortify can't handle NULL properly
    plot.data <- ggplot2::fortify(object)
  } else {
    plot.data <- ggplot2::fortify(object, data = data)
  }
  n <- nrow(plot.data)

  plot.data$.index <- 1:n
  plot.data$.label <- rownames(plot.data)

  is_glm <- inherits(object, "glm")
  r <- residuals(object)
  w <- weights(object)
  if (any(show[2L:6L])) {
    s <- if (inherits(object, "rlm")) {
      object$s
    } else if (is_glm) {
      sqrt(summary(object)$dispersion)
    } else {
      sqrt(stats::deviance(object) / stats::df.residual(object))
    }
    hii <- stats::lm.influence(object, do.coef = FALSE)$hat
    r.hat <- range(hii, na.rm = TRUE)
    is_const_lev <- all(r.hat == 0) ||
      diff(r.hat) < 1e-10 * mean(hii, na.rm = TRUE)

    fs <- dplyr::select_if(plot.data,
                    function(x) is.character(x) | is.factor(x))
    fs[[".label"]] <- NULL
    if (is_const_lev & ncol(fs) > 0){
      plot.data$.nf <- stringr::str_wrap(interaction(fs, sep = ":"), width = 10)
    }

    if (any(show[2L:3L])) {
      plot.data$.wresid <- if (is.null(w)) {
        r
      } else {
        sqrt(w) * r
      }
      plot.data$.wstdresid <- plot.data$.wresid / (s * sqrt(1 - hii))
    }
    if (show[2L]) {
      ylim <- range(plot.data$.wstdresid, na.rm = TRUE)
      ylim[2L] <- ylim[2L] + diff(ylim) * 0.075
      qn <- stats::qqnorm(plot.data$.wstdresid, ylim = ylim, plot.it = FALSE)
      plot.data$.qqx <- qn$x
      plot.data$.qqy <- qn$y
    }
  }

  label.fitted <- ifelse(is_glm, 'Predicted values', 'Fitted values')
  label.y23 <- ifelse(is_glm, 'Std. deviance resid.', 'Standardized residuals')

  if (is.logical(shape) && !shape) {
    if (missing(label)) {
      # if label is missing and shape=FALSE, turn label to TRUE
      label <- TRUE
    }
    if (missing(label.n)) {
      label.n <- nrow(plot.data)
    }
  }

  # target or explanatory variables may be nested, e.g. binomial
  plot.data <- flatten(plot.data)

  if (label.n > 0L) {
    if (show[1L]) {
      r.data <- dplyr::arrange(plot.data, dplyr::desc(abs(.resid)))
      r.data <- utils::head(r.data, label.n)
    }
    if (".wresid" %in% colnames(plot.data)) {
      wr.data <- dplyr::arrange(plot.data, dplyr::desc(abs(.wresid)))
      wr.data <- utils::head(wr.data, label.n)
    }
    if (any(show[4L:6L])) {
      cd.data <- dplyr::arrange(plot.data, dplyr::desc(abs(.cooksd)))
      cd.data <- utils::head(cd.data, label.n)
    }
  }

  .smooth <- function(x, y) {
    stats::lowess(x, y, f = 2 / 3, iter = 3)
  }

  .decorate.label <- function(p, data) {
    if (label & label.n > 0) {
      p <- plot_label(p = p, data = data,
                      label = label,
                      label.label = label.label,
                      label.colour = label.colour,
                      label.alpha = label.alpha,
                      label.size = label.size,
                      label.angle = label.angle,
                      label.family = label.family,
                      label.fontface = label.fontface,
                      label.lineheight = label.lineheight,
                      label.hjust = label.hjust,
                      label.vjust = label.vjust,
                      label.repel = label.repel)
    }
    p
  }

  .decorate.plot <- function(p, xlab = NULL, ylab = NULL, title = NULL) {
    p +
      ggplot2::xlab(xlab) +
      ggplot2::ylab(ylab) +
      ggplot2::ggtitle(title)
  }

  smoother_m <- ggplot2::aes_string(x = 'x', y = 'y')

  if (show[1L]) {
    t1 <- 'Residuals vs Fitted'
    mapping <- ggplot2::aes_string(x = '.fitted', y = '.resid')
    smoother <- .smooth(plot.data$.fitted, plot.data$.resid)
    smoother <- as.data.frame(smoother)
    p1 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    if (!is.logical(shape) || shape) {
      p1 <- p1 + geom_factory(geom_point, plot.data,
                              colour = colour, size = size, linetype = linetype,
                              alpha = alpha, fill = fill, shape = shape)
    }
    p1 <- p1 +
      ggplot2::geom_line(data = smoother, mapping = smoother_m,
                         colour = smooth.colour, linetype = smooth.linetype) +
      ggplot2::geom_hline(yintercept = 0L, linetype = ad.linetype,
                          size = ad.size, colour = ad.colour)
    p1 <- .decorate.label(p1, r.data)
    p1 <- .decorate.plot(p1, xlab = label.fitted, ylab = 'Residuals', title = t1)
  }

  if (show[2L]) {
    t2 <- 'Normal Q-Q'
    qprobs <- c(0.25, 0.75)

    qy <- stats::quantile(plot.data$.wstdresid, probs = qprobs, names = FALSE,
                          type = 7, na.rm = TRUE)
    qx <- stats::qnorm(qprobs)
    slope <- diff(qy) / diff(qx)
    int <- qy[1L] - slope * qx[1L]

    mapping <- ggplot2::aes_string(x = '.qqx', y = '.qqy')
    p2 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    if (!is.logical(shape) || shape) {
      # Do not use stat_qq here for labeling
      p2 <- p2 + geom_factory(geom_point, plot.data,
                              colour = colour, size = size, linetype = linetype,
                              alpha = alpha, fill = fill, shape = shape)
    }
    p2 <- p2 + ggplot2::geom_abline(intercept=int, slope=slope,
                                    linetype = ad.linetype, size = ad.size,
                                    colour = ad.colour)
    p2 <- .decorate.label(p2, wr.data)
    p2 <- .decorate.plot(p2, xlab = 'Theoretical Quantiles',
                         ylab = label.y23, title = t2)
  }

  if (show[3L]) {
    t3 <- 'Scale-Location'
    mapping <- ggplot2::aes_string(x = '.fitted', y = 'sqrt(abs(.wstdresid))')
    smoother <- .smooth(plot.data$.fitted, sqrt(abs(plot.data$.wstdresid)))
    smoother <- as.data.frame(smoother)
    p3 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    if (!is.logical(shape) || shape) {
      p3 <- p3 + geom_factory(geom_point, plot.data,
                              colour = colour, size = size, linetype = linetype,
                              alpha = alpha, fill = fill, shape = shape)
    }
    p3 <- p3 + ggplot2::geom_line(data = smoother, mapping = smoother_m,
                                  colour = smooth.colour, linetype = smooth.linetype)
    p3 <- .decorate.label(p3, wr.data)
    label.y3 <- ifelse(is_glm, expression(sqrt(abs(`Std. deviance resid.`))),
                       expression(sqrt(abs(`Standardized residuals`))))
    p3 <- .decorate.plot(p3, xlab = label.fitted, ylab = label.y3,
                         title = t3)
  }

  if (show[4L]) {
    t4 <- "Cook's distance"
    mapping <- ggplot2::aes_string(x = '.index', y = '.cooksd',
                                   ymin = 0, ymax = '.cooksd')
    p4 <-  ggplot2::ggplot(data = plot.data, mapping = mapping)
    if (!is.logical(shape) || shape) {
      p4 <- p4 + geom_factory(geom_linerange, plot.data,
                              colour = colour, size = size, linetype = linetype,
                              alpha = alpha, fill = fill, shape = shape)
    }
    p4 <- .decorate.label(p4, cd.data)
    p4 <- .decorate.plot(p4, xlab = 'Obs. Number',
                         ylab = "Cook's distance", title = t4)
  }

  if (show[5L]) {
    if (is_const_lev & ncol(fs) > 0){
      t5 <- 'Constant Leverage:\nResiduals vs Factor Levels'
      mapping <- ggplot2::aes_string(x = '.nf', y = '.stdresid')

      p5 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
      if (!is.logical(shape) || shape) {
        p5 <- p5 + geom_factory(geom_point, plot.data,
                                colour = colour, size = size, linetype = linetype,
                                alpha = alpha, fill = fill, shape = shape)
      }
      p5 <- p5 +
        ggplot2::geom_hline(yintercept = 0L, linetype = ad.linetype,
                            size = ad.size, colour = ad.colour) +
        ggplot2::expand_limits(x = 0)
      p5 <- .decorate.label(p5, cd.data)
      label.y5 <- ifelse(is_glm, 'Std. Pearson resid.', 'Standardized Residuals')
      p5 <- .decorate.plot(p5, xlab = 'Factor Level Combination',
                           ylab = label.y5, title = t5)
    } else {
      t5 <- 'Residuals vs Leverage'
      mapping <- ggplot2::aes_string(x = '.hat', y = '.stdresid')
      smoother <- .smooth(plot.data$.hat, plot.data$.stdresid)
      smoother <- as.data.frame(smoother)
      p5 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
      if (!is.logical(shape) || shape) {
        p5 <- p5 + geom_factory(geom_point, plot.data,
                                colour = colour, size = size, linetype = linetype,
                                alpha = alpha, fill = fill, shape = shape)
      }
      p5 <- p5 + ggplot2::geom_line(data = smoother, mapping = smoother_m,
                                    colour = smooth.colour, linetype = smooth.linetype) +
        ggplot2::geom_hline(yintercept = 0L, linetype = ad.linetype,
                            size = ad.size, colour = ad.colour) +
        ggplot2::expand_limits(x = 0)
      p5 <- .decorate.label(p5, cd.data)
      label.y5 <- ifelse(is_glm, 'Std. Pearson resid.', 'Standardized Residuals')
      p5 <- .decorate.plot(p5, xlab = 'Leverage', ylab = label.y5, title = t5)
    }
  }

  if (show[6L]) {
    t6 <- "Cook's dist vs Leverage"
    mapping <- ggplot2::aes_string(x = '.hat', y = '.cooksd')
    smoother <- .smooth(plot.data$.hat, plot.data$.cooksd)
    smoother <- as.data.frame(smoother)
    p6 <- ggplot2::ggplot(data = plot.data, mapping = mapping)
    if (!is.logical(shape) || shape) {
      p6 <- p6 + geom_factory(geom_point, plot.data,
                              colour = colour, size = size, linetype = linetype,
                              alpha = alpha, fill = fill, shape = shape)
    }
    p6 <- p6 + ggplot2::geom_line(data = smoother, mapping = smoother_m,
                                  colour = smooth.colour, linetype = smooth.linetype) +
      ggplot2::expand_limits(x = 0, y = 0)
    p6 <- .decorate.label(p6, cd.data)
    p6 <- .decorate.plot(p6, xlab = 'Leverage', ylab = "Cook's distance",
                         title = t6)

    g <- dropInf(hii / (1 - hii), hii)
    p <- length(stats::coef(object))
    bval <- pretty(sqrt(p * plot.data$.cooksd / g), 5)
    for (i in seq_along(bval)) {
      bi2 <- bval[i] ^ 2
      p6 <- p6 + ggplot2::geom_abline(intercept=0, slope=bi2,
                                      linetype = ad.linetype, size = ad.size,
                                      colour = ad.colour)
    }
  }

  if (is.null(ncol)) { ncol <- 0 }
  if (is.null(nrow)) { nrow <- 0 }
  plot.list <- list(p1, p2, p3, p4, p5, p6)[which]
  new('ggmultiplot', plots = plot.list, nrow = nrow, ncol = ncol)
}

#' @export
autoplot.glm <- autoplot.lm

## ----global_options, include=FALSE--------------------------------------------
library(knitr)
opts_chunk$set(fig.width=6, fig.height=4, fig.path='figures/basics-', warning=FALSE)

## ---- message = FALSE---------------------------------------------------------
library(ggfortify)

## -----------------------------------------------------------------------------
autoplot(AirPassengers)

## -----------------------------------------------------------------------------
autoplot(AirPassengers, ts.colour = 'blue')

## -----------------------------------------------------------------------------
p <- autoplot(AirPassengers)
class(p)
# plot as it is
p

# add title and labels
p + ggtitle('AirPassengers') + xlab('Year') + ylab('Passengers')

## -----------------------------------------------------------------------------
# these common options are supported as keywords 
autoplot(AirPassengers, title = 'AirPassengers', xlab = 'Year', ylab = 'Passengers')

## -----------------------------------------------------------------------------
set.seed(1)
p <- autoplot(kmeans(iris[-5], 3), data = iris)
# plot as it is
p

# change colour mapping
p + scale_colour_brewer()

## -----------------------------------------------------------------------------
df <- fortify(kmeans(iris[-5], 3), data = iris)
head(df)

ggplot(df, aes(x= cluster, fill = cluster)) + geom_bar()

## ---- message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE------------
#  res <- lm(Volume ~ Girth, data = trees)
#  mp <- autoplot(res, ncol = 4)
#  mp

## ---- message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE------------
#  class(mp)
#  mp + theme_bw()

## ---- message = FALSE, fig.width = 9, fig.height = 5, eval=FALSE--------------
#  mp +
#    (ggplot(trees, aes(Girth, Volume)) + geom_point()) +
#    (ggplot(trees, aes(Girth, Height)) + geom_point())

## ---- message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE------------
#  mp[2:3]

## ---- message = FALSE, fig.width = 2.25, fig.height = 2.5, eval=FALSE---------
#  mp[[1]]

## ---- message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE------------
#  mp[2:3] <- mp[2:3] + theme_bw()
#  mp

## ---- message = FALSE, fig.width = 9, fig.height = 2.5------------------------
res <- lapply(c(3, 4, 5), function(x) kmeans(iris[-5], x))
autoplot(res, data = iris[-5], ncol = 3)

## ---- message = FALSE, fig.width = 8, fig.height = 2.5------------------------
library(survival)
res <- list(a = survfit(Surv(time, status) ~ 1, data = lung),
            b = survfit(Surv(time, status) ~ sex, data = lung))
autoplot(res)

## ---- message = FALSE, fig.width = 8, fig.height = 2.5------------------------
library(vars)
data(Canada)
res <- list(a = Canada, b = AirPassengers)
autoplot(res)

## ---- message = FALSE, fig.width = 4.5, fig.height = 2.5----------------------
p1 <- ggplot(iris, aes(Petal.Width, Petal.Length)) + geom_point()
p2 <- ggplot(iris, aes(Petal.Width, Petal.Length)) + geom_point()
new('ggmultiplot', plots = list(p1, p2))

## ---- message = FALSE, fig.width = 2.25, fig.height = 5-----------------------
new('ggmultiplot', plots = list(p1, p2), ncol = 1)

#' Convert \code{MSwM::MSM.lm} to \code{data.frame}
#'
#' @param model \code{MSwM::MSM.lm} instance
#' @inheritParams fortify_base
#' @param melt Logical flag indicating whether to melt each models
#' @return data.frame
#' @examples
#' library(MSwM)
#' d <- data.frame(Data = c(rnorm(50, mean = -10), rnorm(50, mean = 10)),
#'                 exog = cos(seq(-pi/2, pi/2, length.out = 100)))
#' d.mswm <- MSwM::msmFit(lm(Data ~.-1, data = d), k=2, sw=rep(TRUE, 2),
#'                        control = list(parallelization = FALSE))
#' fortify(d.mswm)
#' @export
fortify.MSM.lm <- function(model, data = NULL, melt = FALSE, ...) {
  probable <- apply(model@Fit@smoProb[-1, ], 1, which.max)
  idx <- names(model@model$fitted.values)

  if (melt) {
    models <- seq(1, model@k)
    .model <- function(k) {
      md <- cbind(data.frame(Index = idx),
                  model@model$model,
                  data.frame(Fitted = model@model$fitted.values,
                             Residuals = model@model$residuals,
                             FiltProb = model@Fit@filtProb[, k],
                             SmoProb = model@Fit@smoProb[-1, k]))
      md$Index <- factor(md$Index, levels = idx)
      md$Model <- rep(k, nrow(md))
      md$ProbableModel <- probable
      md
    }
    d <- dplyr::bind_rows(lapply(models, .model))
    d$Model <- as.factor(d$Model)
  } else {
    d <- cbind(data.frame(Index = idx),
               model@model$model,
               data.frame(Fitted = model@model$fitted.values,
                          Residuals = model@model$residuals,
                          FiltProb = model@Fit@filtProb,
                          SmoProb = model@Fit@smoProb[-1, ],
                          ProbableModel = probable))
  }
  post_fortify(d)
}

#' Autoplot \code{MSwM::MSM.lm}
#'
#' @param object \code{MSwM::MSM.lm} instance
#' @param prob.colour Line colour for probabilities
#' @param prob.linetype Line type for probabilities
#' @param ... other arguments passed to \code{autoplot.ts}
#' @return ggplot
#' @examples
#' \dontrun{
#' library(MSwM)
#' d <- data.frame(Data = c(rnorm(50, mean = -10), rnorm(50, mean = 10)),
#'                 exog = cos(seq(-pi/2, pi/2, length.out = 100)))
#' d.mswm <- MSwM::msmFit(lm(Data ~.-1, data = d), k=2, sw=rep(TRUE, 2),
#'                        control = list(parallelization = FALSE))
#' autoplot(d.mswm)
#' }
#' @importFrom scales percent
#' @export
autoplot.MSM.lm <- function(object, prob.colour = '#FF0000',
                            prob.linetype = 'dashed', ...) {
  plot.data <- ggplot2::fortify(object, melt = TRUE)

  y <- 'SmoProb'
  p <- autoplot.ts(plot.data, columns = y, group = 'Model', ...)
  p <- p + ggplot2::geom_linerange(mapping = ggplot2::aes_string(ymax = 'FiltProb'), ymin = 0) +
    ggplot2::facet_wrap(~Model, ncol = 1) +
    ggplot2::scale_y_continuous(name = 'Smoothed Probabilities', labels = scales::percent)

  p
}

#' @export
fortify.stl <- fortify.ts

#' @export
fortify.decomposed.ts <- fortify.ts

#' @export
autoplot.stl <- autoplot.ts

#' @export
autoplot.decomposed.ts <- autoplot.ts

#' Convert \code{stats::acf} to \code{data.frame}
#'
#' @param model \code{stats::acf} instance
#' @inheritParams fortify_base
#' @param conf.int Logical flag indicating whether to attach confidence intervals
#' @param conf.int.value Coverage probability for confidence interval
#' @param conf.int.type Type of confidence interval, 'white' for white noise or 'ma' MA(k-1) model
#' @return data.frame
#' @examples
#' fortify(stats::acf(AirPassengers))
#' fortify(stats::pacf(AirPassengers))
#' fortify(stats::ccf(AirPassengers, AirPassengers))
#'
#' fortify(stats::acf(AirPassengers), conf.int = TRUE)
#' @export
fortify.acf <- function(model, data = NULL,
                        conf.int = TRUE, conf.int.value = 0.95,
                        conf.int.type = 'white', ...) {
  d <- data.frame(Lag = model$lag, ACF = model$acf)
  if (conf.int) {
    cf <- confint.acf(model, ci = conf.int.value, ci.type = conf.int.type)
    cfd <- data.frame(lower = -cf, upper = cf)
    d <- cbind(d, cfd)
  }
  post_fortify(d)
}

#' Autoplot \code{stats::acf}. Note to pass `plot = FALSE` to original function to suppress
#' standard plot output
#'
#' @param object \code{stats::acf} instance
#' @param colour Line colour
#' @param linetype Line type
#' @inheritParams plot_confint
#' @param conf.int.value Coverage probability for confidence interval
#' @param conf.int.type Type of confidence interval, 'white' for white noise or 'ma' MA(k-1) model
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(stats::acf(AirPassengers, plot = FALSE))
#' autoplot(stats::pacf(AirPassengers, plot = FALSE))
#' autoplot(stats::ccf(AirPassengers, AirPassengers, plot = FALSE))
#' @export
autoplot.acf <- function(object,
                         colour = '#000000', linetype = 'solid',
                         conf.int = TRUE,
                         conf.int.colour = '#0000FF', conf.int.linetype = 'dashed',
                         conf.int.fill = NULL, conf.int.alpha = 0.3,
                         conf.int.value = 0.95, conf.int.type = 'white',
                         xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                         main = NULL, xlab = NULL, ylab = 'ACF', asp = NULL,
                         ...) {
  plot.data <- ggplot2::fortify(object, conf.int = conf.int,
                                conf.int.value = conf.int.value,
                                conf.int.type = conf.int.type)

  # Prepare ymax and ymin used for geom_linerange
  plot.data <- dplyr::mutate(plot.data,
                              ymax = ifelse(ACF > 0, ACF, 0),
                              ymin = ifelse(ACF < 0, ACF, 0))

  p <- ggplot2::ggplot(data = plot.data, mapping = ggplot2::aes_string(x = 'Lag')) +
    ggplot2::geom_linerange(mapping = ggplot2::aes_string(ymin = 'ymin', ymax = 'ymax'),
                            colour = colour, linetype = linetype)

  p <- plot_confint(p = p, data = plot.data, conf.int = conf.int,
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{stats::spec} to \code{data.frame}
#'
#' @param model \code{stats::spec} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(spectrum(AirPassengers))
#' fortify(stats::spec.ar(AirPassengers))
#' fortify(stats::spec.pgram(AirPassengers))
#' @export
fortify.spec <- function(model, data = NULL, ...) {
  d <- data.frame(Frequency = model$freq,
                  Spectrum = model$spec)
  post_fortify(d)
}

#' Autoplot \code{stats::spec}
#'
#' @param object \code{stats::spec} instance
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(stats::spec.ar(AirPassengers))
#' autoplot(stats::spec.pgram(AirPassengers))
#' @export
autoplot.spec <- function(object,
                          xlim = c(NA, NA), ylim = c(NA, NA), log = "y",
                          main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                          ...) {
  plot.data <- ggplot2::fortify(object)
  mapping <- ggplot2::aes_string(x = 'Frequency', y = 'Spectrum')
  p <- ggplot2::ggplot(data = plot.data) +
    ggplot2::geom_line(mapping = mapping, stat = 'identity')
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{stats::prcomp}, \code{stats::princomp} to \code{data.frame}
#'
#' @param model \code{stats::prcomp} or \code{stats::princomp} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @aliases fortify.princomp
#' @examples
#' fortify(stats::prcomp(iris[-5]))
#' fortify(stats::prcomp(iris[-5]), data = iris)
#'
#' fortify(stats::princomp(iris[-5]))
#' fortify(stats::princomp(iris[-5]), data = iris)
#' @export
fortify.prcomp <- function(model, data = NULL, ...) {

  if (is(model, 'prcomp')) {
    d <- as.data.frame(model$x)
    values <- model$x %*% t(model$rotation)
  } else if (is(model, 'princomp')) {
    d <- as.data.frame(model$scores)
    values <- model$scores %*% t(model$loadings[, ])
  } else {
    stop(paste0('Unsupported class for fortify.pca_common: ', class(model)))
  }

  values <- ggfortify::unscale(values, center = model$center,
                               scale = model$scale)
  values <- cbind_wraps(data, values)
  d <- cbind_wraps(values, d)
  post_fortify(d)
}

#' @export
fortify.princomp <- fortify.prcomp

#' Convert \code{stats::factanal} to \code{data.frame}
#'
#' @param model \code{stats::factanal} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' d.factanal <- stats::factanal(state.x77, factors = 3, scores = 'regression')
#' fortify(d.factanal)
#' fortify(d.factanal, data = state.x77)
#' @export
fortify.factanal <- function(model, data = NULL, ...) {

  if (is.null(model$scores)) {
    stop(paste0('Unable to fortify factanal result without scores, ',
                'specify scores="regression", or "Bartlett" when calling factanal'))
  }
  d <- as.data.frame(model$scores)
  d <- cbind_wraps(data, d)
  post_fortify(d)
}

#' Convert \code{lfda::lfda} or \code{lfda::klfda} or \code{lfda::self} to \code{data.frame}
#'
#' @param model \code{lfda::lfda} or \code{lfda::klfda} or \code{lfda::self} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' \dontrun{
#' model <- lfda::lfda(iris[, -5], iris[, 5], 3, metric = "plain")
#' fortify(model)
#' }
#' @export
fortify.lfda <- function(model, data = NULL, ...) {

  if (!is(model, 'lfda')) {
    stop('model is not a lfda object')
  }

  model <- stats::prcomp(model$Z)

  d <- as.data.frame(model$x)
  values <- model$x %*% t(model$rotation)

  values <- ggfortify::unscale(values, center = model$center,
                               scale = model$scale)
  values <- cbind_wraps(data, values)
  d <- cbind_wraps(values, d)
  post_fortify(d)
}

#' Autoplot PCA-likes
#'
#' @param object PCA-like instance
#' @param data Joined to fitting result if provided.
#' @param scale scaling parameter, disabled by 0
#' @param x principal component number used in x axis
#' @param y principal component number used in y axis
#' @param variance_percentage show the variance explained by the principal component?
#' @param ... other arguments passed to [ggbiplot()]
#' @inheritParams ggbiplot
#' @inheritParams plot_label
#' @inheritParams post_autoplot
#' @aliases autoplot.prcomp autoplot.princomp autoplot.factanal
#' @examples
#' autoplot(stats::prcomp(iris[-5]))
#' autoplot(stats::prcomp(iris[-5]), data = iris)
#' autoplot(stats::prcomp(iris[-5]), data = iris, colour = 'Species')
#' autoplot(stats::prcomp(iris[-5]), label = TRUE, loadings = TRUE, loadings.label = TRUE)
#' autoplot(stats::prcomp(iris[-5]), frame = TRUE)
#' autoplot(stats::prcomp(iris[-5]), data = iris, frame = TRUE,
#'          frame.colour = 'Species')
#' autoplot(stats::prcomp(iris[-5]), data = iris, frame = TRUE,
#'          frame.type = 't', frame.colour = 'Species')
#'
#' autoplot(stats::princomp(iris[-5]))
#' autoplot(stats::princomp(iris[-5]), data = iris)
#' autoplot(stats::princomp(iris[-5]), data = iris, colour = 'Species')
#' autoplot(stats::princomp(iris[-5]), label = TRUE, loadings = TRUE, loadings.label = TRUE)
#'
#' #Plot PC 2 and 3
#' autoplot(stats::princomp(iris[-5]), x = 2, y = 3)
#'
#' #Don't show the variance explained
#' autoplot(stats::princomp(iris[-5]), variance_percentage = FALSE)
#'
#' d.factanal <- stats::factanal(state.x77, factors = 3, scores = 'regression')
#' autoplot(d.factanal)
#' autoplot(d.factanal, data = state.x77, colour = 'Income')
#' autoplot(d.factanal, label = TRUE, loadings = TRUE, loadings.label = TRUE)
#' @export
autoplot.pca_common <- function(object, data = NULL,
                                scale = 1.0, x = 1, y = 2,
                                variance_percentage = TRUE, ...) {

  plot.data <- ggplot2::fortify(object, data = data)
  plot.data$rownames <- rownames(plot.data)

  if (is_derived_from(object, 'prcomp')) {

    ve <- object$sdev^2 / sum(object$sdev^2)
    PC <- paste0("PC", c(x, y))
    x.column <- PC[1]
    y.column <- PC[2]
    loadings.column <- 'rotation'

    lam <- object$sdev[c(x, y)]
    lam <- lam * sqrt(nrow(plot.data))

  } else if (is_derived_from(object, 'princomp')) {

    ve <- object$sdev^2 / sum(object$sdev^2)
    PC <- paste0("Comp.", c(x, y))
    x.column <- PC[1]
    y.column <- PC[2]
    loadings.column <- 'loadings'

    lam <- object$sdev[c(x, y)]
    lam <- lam * sqrt(nrow(plot.data))

  } else if (is_derived_from(object, 'factanal')) {

    if (is.null(attr(object, "covariance"))) {
      p <- nrow(object$loading)
      ve <- colSums(object$loading^2) / p
    } else ve <- NULL

    PC <- paste0("Factor", c(x, y))
    x.column <- PC[1]
    y.column <- PC[2]
    scale <- 0
    loadings.column <- 'loadings'

  } else if (is_derived_from(object, 'lfda')) {

    ve <- NULL
    PC <- paste0("PC", c(x, y))
    x.column <- PC[1]
    y.column <- PC[2]
    scale <- 0
    loadings.column <- NULL

  } else {
    stop(paste0('Unsupported class for autoplot.pca_common: ', class(object)))
  }

  # scaling
  if (scale != 0) {
    lam <- lam ^ scale
    plot.data[, c(x.column, y.column)] <- t(t(plot.data[, c(x.column, y.column)]) / lam)
  }

  # move target columns to 1st and 2nd
  plot.columns <- unique(c(x.column, y.column, colnames(plot.data)))
  plot.data <- plot.data[, plot.columns]

  if (!is.null(loadings.column)) {
    loadings.data <-as.data.frame(object[[loadings.column]][, ])
    loadings.data$rownames <- rownames(loadings.data)

    loadings.columns <- unique(c(x.column, y.column, colnames(loadings.data)))
    loadings.data <- loadings.data[, loadings.columns]
  } else {
    loadings.data <- NULL
  }

  #Make labels
  if (is.null(ve) | !variance_percentage) {
    labs <- PC
  } else {
    ve <- ve[c(x, y)]
    labs <- paste0(PC, " (", round(ve * 100, 2), "%)")
  }
  xlab <- labs[1]
  ylab <- labs[2]

  p <- ggbiplot(plot.data = plot.data,
                loadings.data = loadings.data,
                xlab = xlab,
                ylab = ylab, ...)
  return(p)
}

#' @export
autoplot.prcomp <- autoplot.pca_common

#' @export
autoplot.princomp <- autoplot.pca_common

#' @export
autoplot.factanal <- autoplot.pca_common

#' @export
autoplot.lfda <- autoplot.pca_common

#' Convert \code{stats::dist} to \code{data.frame}
#'
#' @param model \code{stats::dist} instance
#' @param data original dataset, if needed
#' @param ... other arguments passed to methods
#' @return data.frame
#' @examples
#' fortify(eurodist)
#' @export
fortify.dist <- function(model, data = NULL, ...) {
  model <- as.matrix(model)
  ggplot2::fortify(model)
}

#' @export
autoplot.dist <- autoplot.matrix


#' Convert \code{stats::stepfun} to \code{data.frame}
#'
#' @param model \code{stats::stepfun} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
#' fortify(stepfun(c(1), c(4, 5)))
#' fortify(stepfun(c(1, 3, 4, 8), c(4, 5, 2, 3, 5)))
#' fortify(stepfun(c(1, 2, 3, 4, 5, 6, 7, 8, 10), c(4, 5, 6, 7, 8, 9, 10, 11, 12, 9)))
#' @export
fortify.stepfun <- function(model, data, ...) {
  x <- stats::knots(model)
  lim <- range(x)
  if (length(x) > 1L) {
    dr <- max(0.08 * diff(lim), stats::median(diff(x)))
  } else {
    dr <- abs(x) / 16
  }
  lim <- lim + dr * c(-1, 1)
  x <- c(lim[1], rep(x, each = 2), lim[2])
  y <- base::eval(expression(c(yleft, y)), envir = environment(model))
  y <- rep(y, each = 2)
  d <- data.frame(x = x, y = y)
  post_fortify(d)
}

#' Plot \code{stats::stepfun}
#'
#' @param object \code{stats::stepfun} instance
#' @param colour colour
#' @param size point size
#' @param linetype line type
#' @param alpha alpha
#' @param shape point shape
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
#' autoplot(stepfun(c(1), c(4, 5)), shape = NULL)
#' autoplot(stepfun(c(1, 3, 4, 8), c(4, 5, 2, 3, 5)), linetype = 'dashed')
#' autoplot(stepfun(c(1, 2, 3, 4, 5, 6, 7, 8, 10), c(4, 5, 6, 7, 8, 9, 10, 11, 12, 9)), colour = 'red')
#' @export
autoplot.stepfun <- function(object,
                             colour = NULL, size = NULL, linetype = NULL,
                             alpha = NULL, shape = 1,
                             xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                             main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                             ...) {
  # not accept shape kw

  plot.data <- ggplot2::fortify(object)

  mapping <- ggplot2::aes_string(x = 'x', y = 'y')
  p <- ggplot2::ggplot(plot.data, mapping = mapping) +
    geom_factory(ggplot2::geom_line, data = plot.data,
                colour = colour, size = size, linetype = linetype,
                alpha = alpha, stat = 'identity')
  if (nrow(plot.data) >= 3 & !is.null(shape)) {
    # indexer will be logical vector
    # to skip first value, start with c(FALSE, FALSE)
    indexer <- c(FALSE, FALSE, 3:nrow(plot.data) %% 2 == 1)
    point.data <- plot.data[indexer, ]
    p <- p +
      geom_factory(ggplot2::geom_point, data = point.data,
                   colour = colour, size = size,
                   alpha = alpha, shape = shape)
  }

  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{ts} index to \code{Date} \code{vector}
#'
#' @param data \code{ts} instance
#' @param is.tsp Logical frag whether data is \code{tsp} itself or not
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @return vector
#' @examples
#' ggfortify:::get.dtindex(AirPassengers)
#' ggfortify:::get.dtindex(UKgas)
#' ggfortify:::get.dtindex(Nile, is.date = FALSE)
#' ggfortify:::get.dtindex(Nile, is.date = TRUE)
get.dtindex <- function(data, is.tsp = FALSE, is.date = NULL) {
  if (is.tsp) {
    tsp <- data
  } else {
    tsp <- attr(data, which='tsp')
  }
  if (is.null(tsp)) {
    stop('Failed to convert ts object index to date')
  }
  dtindex <- seq(from = tsp[1], to = tsp[2], by= 1 / tsp[3])
  if ( (is.null(is.date) && any(tsp[3] == c(4, 12))) ||
       (!is.null(is.date) && is.date)) {
    dtindex <- zoo::as.Date.yearmon(dtindex)
  }
  dtindex
}

#' Get \code{Date} \code{vector} continue to \code{ts} index
#'
#' @param data \code{ts} instance
#' @param length A number to continue
#' @param is.tsp Logical frag whether data is \code{tsp} itself or not
#' @param is.date Logical frag indicates whether the \code{stats::ts} is date or not.
#' If not provided, regard the input as date when the frequency is 4 or 12.
#' @return vector
#' @examples
#' ggfortify:::get.dtindex.continuous(AirPassengers, length = 10)
#' ggfortify:::get.dtindex.continuous(UKgas, length = 10)
get.dtindex.continuous <- function(data, length, is.tsp = FALSE, is.date = NULL) {
  if (is.tsp) {
    tsp <- data
  } else {
    tsp <- attr(data, which='tsp')
  }
  if (is.null(tsp)) {
    stop('Failed to convert ts object index to date')
  }
  dt.by <- 1 / tsp[3]
  dtindex <- seq(from = tsp[2] + dt.by, length = length, by = dt.by)
  if ( (is.null(is.date) && any(tsp[3] == c(4, 12))) ||
       (!is.null(is.date) && is.date)) {
    dtindex <- zoo::as.Date.yearmon(dtindex)
  }
  dtindex
}

#' Check if Validates number of \code{ts} variates
#'
#' @param data \code{ts} instance
#' @param raise Logical flag whether raise an error
#' @return logical
#' @examples
#' ggfortify:::is.univariate(AirPassengers)
is.univariate <- function(data, raise = TRUE) {
  if (ncol(as.matrix(data)) > 1) {
    if (raise) {
      stop("data must be univariate time series")
    } else {
      return(FALSE)
    }
  }
  return(TRUE)
}

#' Rbind original and predicted time-series-like instances as fortified \code{data.frame}
#'
#' @param data Predicted/forecasted \code{ts} instance
#' @param original Original \code{ts} instance
#' @param ts.connect Logical frag indicates whether connects original time-series and predicted values
#' @param index.name Specify column name for time series index
#' @param data.name Specify column name for univariate time series data. Ignored in multivariate time series.
#' @return data.frame
#' @examples
#' predicted <- predict(stats::HoltWinters(UKgas), n.ahead = 5, prediction.interval = TRUE)
#' rbind_ts(predicted, UKgas, ts.connect = TRUE)
#' @export
rbind_ts <- function(data, original, ts.connect = TRUE,
                     index.name = 'Index', data.name = 'Data') {
  if (!is.data.frame(data)) {
    data <- ggplot2::fortify(data, index.name = index.name,
                             data.name = 'Point Forecast')
  }

  dnames <- names(data)
  dnames <- dnames[dnames != index.name]

  if (!is.data.frame(original)) {
    original <- ggplot2::fortify(original, index.name = index.name,
                                 data.name = data.name)
  }
  n <- nrow(original)
  rownames(data) <- NULL
  rownames(original) <- NULL

  d <- dplyr::bind_rows(original, data)
  if (ts.connect) {
    # Use fnames not to overwrite Index
    d[n, dnames] <- d[n, data.name]
  }
  post_fortify(d)
}


#' Calculate confidence interval for \code{stats::acf}
#'
#' @param x \code{stats::acf} instance
#' @param ci Float value for confidence interval
#' @param ci.type "white" or "ma"
#' @return vector
#' @examples
#' air.acf <- acf(AirPassengers, plot = FALSE)
#' ggfortify:::confint.acf(air.acf)
#' ggfortify:::confint.acf(air.acf, ci.type = 'ma')
confint.acf <- function (x, ci = 0.95, ci.type = "white") {
  if ( (nser <- ncol(x$lag)) < 1L)
    stop("x$lag must have at least 1 column")
  with.ci <- ci > 0 && x$type != "covariance"
  with.ci.ma <- with.ci && ci.type == "ma" && x$type == "correlation"

  if (with.ci.ma && x$lag[1L, 1L, 1L] != 0L) {
    warning("can use ci.type=\"ma\" only if first lag is 0")
    with.ci.ma <- FALSE
  }
  clim0 <- if (with.ci)
    stats::qnorm( (1 + ci) / 2) / sqrt(x$n.used)
  else c(0, 0)

  Npgs <- 1L
  nr <- nser

  if (nser > 1L) {
    Npgs <- nser
    nr <- ceiling(nser / Npgs)
  }

  for (I in 1L:Npgs) for (J in 1L:Npgs) {
    iind <- (I - 1) * nr + 1L:nr
    jind <- (J - 1) * nr + 1L:nr

    for (i in iind) {
      for (j in jind) {
        if (!(max(i, j) > nser)) {
          clim <- if (with.ci.ma && i == j)
            clim0 * sqrt(cumsum(c(1, 2 * x$acf[-1, i, j] ^ 2)))
          else clim0

          if (with.ci && ci.type == "white") {
            clim
          } else if (with.ci.ma && i == j) {
            clim <- clim[-length(clim)]
          }
        }
      }
    }
  }
  if (with.ci.ma && length(clim) < length(x$lag)) {
    clim <- c(NA, clim)
  }
  return(clim)
}

#' Calculate fitted values for \code{stats::ar}
#'
#' @param object \code{stats::ar} instance
#' @param ... other keywords
#' @return ts An time series of the one-step forecasts
#' @examples
#' fitted(ar(WWWusage))
#' @export
fitted.ar <- function(object, ...) {
  x <- forecast::getResponse(object)
  return(x - stats::residuals(object))
}

#' Calculate residuals for \code{stats::ar}
#'
#' @param object \code{stats::ar} instance
#' @param ... other keywords
#' @return ts Residuals extracted from the object object.
#' @examples
#' residuals(ar(WWWusage))
#' @export
residuals.ar <- function(object, ...) {
  return(object$resid)
}

#' Plots a cumulative periodogram
#'
#' @param ts \code{stats::ts} instance
#' @param taper Proportion tapered in forming the periodogram
#' @param colour Line colour
#' @param linetype Line type
#' @inheritParams plot_confint
#' @return ggplot
#' @examples
#' ggcpgram(AirPassengers)
#' @export
ggcpgram <- function (ts, taper = 0.1,
                      colour = '#000000', linetype = 'solid',
                      conf.int = TRUE,
                      conf.int.colour = '#0000FF', conf.int.linetype = 'dashed',
                      conf.int.fill = NULL, conf.int.alpha = 0.3) {
  is.univariate(ts)

  x <- as.vector(ts)
  x <- x[!is.na(x)]
  x <- stats::spec.taper(scale(x, TRUE, FALSE), p = taper)
  y <- Mod(stats::fft(x)) ^ 2 / length(x)
  y[1L] <- 0
  n <- length(x)
  x <- (0:(n / 2)) * stats::frequency(ts) / n
  if (length(x) %% 2 == 0) {
    n <- length(x) - 1
    y <- y[1L:n]
    x <- x[1L:n]
  }
  else y <- y[seq_along(x)]
  xm <- stats::frequency(ts) / 2
  mp <- length(x) - 1
  crit <- 1.358 / (sqrt(mp) + 0.12 + 0.11 / sqrt(mp))

  d <- data.frame(x = x,
                  y = cumsum(y) / sum(y),
                  upper = 1 / xm * x + crit,
                  lower = 1 / xm * x - crit)
  p <- ggplot2::ggplot(data = d, mapping = ggplot2::aes_string(x = 'x', y = 'y')) +
    geom_line(colour = colour, linetype = linetype) +
    ggplot2::scale_x_continuous(name = '', limits = c(0, xm)) +
    ggplot2::scale_y_continuous(name = '', limits = c(0, 1))

  p <- plot_confint(p = p, data = d, conf.int = conf.int,
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  p
}

#' Plots time-series diagnostics
#'
#' @param object A fitted time-series model
#' @param gof.lag The maximum number of lags for a Portmanteau goodness-of-fit test
#' @inheritParams plot_confint
#' @param ad.colour Line colour for additional lines
#' @param ad.linetype Line type for additional lines
#' @param ad.size Fill colour for additional lines
#' @param nrow Number of facet/subplot rows
#' @param ncol Number of facet/subplot columns
#' @param ... other keywords
#' @return ggplot
#' @examples
#' ggtsdiag(arima(AirPassengers))
#' @export
ggtsdiag <- function(object, gof.lag = 10,
                     conf.int = TRUE,
                     conf.int.colour = '#0000FF', conf.int.linetype = 'dashed',
                     conf.int.fill = NULL, conf.int.alpha = 0.3,
                     ad.colour = '#888888', ad.linetype = 'dashed', ad.size = .2,
                     nrow = NULL, ncol = 1, ...) {
  rs <- stats::residuals(object)
  if (is.null(rs)) {
    rs <- object$residuals
  }
  if (is.null(rs)) {
    rs <- object$resid
  }

  stdres <- rs / sqrt(object$sigma2)
  p.std <- ggplot2::autoplot(stdres) +
    ggplot2::geom_hline(yintercept = 0,
                        linetype = ad.linetype, size = ad.size,
                        colour = ad.colour) +
    ggplot2::ggtitle('Standardized Residuals')

  acfobj <- stats::acf(rs, plot = FALSE, na.action = stats::na.pass)
  p.acf <- autoplot.acf(acfobj, conf.int = conf.int,
                        conf.int.colour = conf.int.colour,
                        conf.int.linetype = conf.int.linetype,
                        conf.int.fill = conf.int.fill,
                        conf.int.alpha = conf.int.alpha)
  p.acf <- p.acf + ggplot2::ggtitle('ACF of Residuals')

  nlag <- gof.lag
  pval <- numeric(nlag)
  for (i in 1L:nlag) pval[i] <- stats::Box.test(rs, i, type = "Ljung-Box")$p.value
  lb.df <- data.frame(Lag = 1L:nlag, `p value` = pval,
                      lower = -0.05, upper = 0.05)
  # Unnable to create column with space by above expression
  colnames(lb.df) <- c('Lag', 'p value', 'lower', 'upper')
  p.lb <- ggplot2::ggplot(data = lb.df, mapping = ggplot2::aes_string(x = 'Lag')) +
    ggplot2::geom_point(mapping = ggplot2::aes_string(y = '`p value`')) +
    ggplot2::scale_y_continuous(limits=c(0, 1)) +
    ggplot2::ggtitle('p values for Ljung-Box statistic')
  p.lb <- plot_confint(p = p.lb, data = lb.df, conf.int = conf.int,
                       conf.int.colour = conf.int.colour,
                       conf.int.linetype = conf.int.linetype,
                       conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)

  if (is.null(ncol)) { ncol <- 0 }
  if (is.null(nrow)) { nrow <- 0 }
  new('ggmultiplot', plots = list(p.std, p.acf, p.lb), nrow = nrow, ncol = ncol)
}

#' Plot seasonal subseries of time series, generalization of \code{stats::monthplot}
#'
#' @param data \code{stats::ts} instance
#' @param freq Length of frequency. If not provided, use time-series frequency
#' @param nrow Number of plot rows
#' @param ncol Number of plot columns
#' @inheritParams plot_confint
#' @param conf.int.value Coverage probability for confidence interval
#' @param facet.labeller A vector used as facet labels
#' @param ... Keywords passed to autoplot.ts
#' @return ggplot
#' @examples
#' ggfreqplot(AirPassengers)
#' ggfreqplot(AirPassengers, freq = 4)
#' ggfreqplot(AirPassengers, conf.int = TRUE)
#' @export
ggfreqplot <- function(data, freq = NULL,
                       nrow = NULL, ncol = NULL,
                       conf.int = FALSE,
                       conf.int.colour = '#0000FF', conf.int.linetype = 'dashed',
                       conf.int.fill = NULL, conf.int.alpha = 0.3,
                       conf.int.value = 0.95, facet.labeller = NULL,
                       ...) {
  is.univariate(data)

  if (is.null(freq)) {
    freq <- stats::frequency(data)
  }

  if (is.null(nrow) && is.null(ncol)) {
    nrow <- ceiling(sqrt(freq))
  }

  d <- ggplot2::fortify(data)
  if (is.null(facet.labeller)) {
    freqs <- 1:freq
  } else if (length(facet.labeller)  == freq) {
    freqs <- factor(facet.labeller, levels = facet.labeller)
  } else {
    stop('facet.labeller must be a vector which has the same length as freq')
  }
  freqd <- data.frame(Frequency = rep(freqs, length.out = length(data)))
  d <- cbind(d, freqd)

  summarised <- dplyr::group_by(d, Frequency) %>%
    dplyr::summarise(m = mean(Data), s = sd(Data))

  p <- (1 - conf.int.value) / 2
  summarised$lower <- stats::qnorm(p, mean = summarised$m, sd = summarised$s)
  summarised$upper <- stats::qnorm(1 - p, mean = summarised$m, sd = summarised$s)

  d <- dplyr::left_join(d, summarised, by = 'Frequency')

  p <- autoplot.ts(d, columns = 'Data', ...)
  p <- p + ggplot2::geom_line(mapping = ggplot2::aes_string(y = 'm'),
                       colour = conf.int.colour) +
    ggplot2::facet_wrap(~Frequency, nrow = nrow, ncol = ncol)
  p <- plot_confint(p = p, data = d, conf.int = conf.int,
                    conf.int.colour = conf.int.colour,
                    conf.int.linetype = conf.int.linetype,
                    conf.int.fill = conf.int.fill, conf.int.alpha = conf.int.alpha)
  p
}

context('test tslib')

test_that('get.dtindex works for AirPassengers', {
  result <- ggfortify:::get.dtindex(AirPassengers)
  expect_equal(result[1], as.Date('1949-01-01'))
  expect_equal(result[length(result)], as.Date('1960-12-01'))
})

test_that('get.dtindex works for UKgas', {
  result <- ggfortify:::get.dtindex(UKgas)
  expect_equal(result[1], as.Date('1960-01-01'))
  expect_equal(result[length(result)], as.Date('1986-10-01'))

  result <- ggfortify:::get.dtindex(UKgas, is.date = FALSE)
  expect_equal(result[1], 1960)
  expect_equal(result[length(result)], 1986.75)

  result <- ggfortify:::get.dtindex(UKgas, is.date = TRUE)
  expect_equal(result[1], as.Date('1960-01-01'))
  expect_equal(result[length(result)], as.Date('1986-10-01'))
})

test_that('get.dtindex works for Nile', {
  result <- ggfortify:::get.dtindex(Nile)
  expect_equal(result[1], 1871)
  expect_equal(result[length(result)], 1970)

  result <- ggfortify:::get.dtindex(Nile, is.date = FALSE)
  expect_equal(result[1], 1871)
  expect_equal(result[length(result)], 1970)

  result <- ggfortify:::get.dtindex(Nile, is.date = TRUE)
  expect_equal(result[1], as.Date('1871-01-01'))
  expect_equal(result[length(result)], as.Date('1970-01-01'))
})

test_that('get.dtindex.continuous works for AirPassengers', {
  result <- ggfortify:::get.dtindex.continuous(AirPassengers, length = 18)
  expect_equal(result[1], as.Date('1961-01-01'))
  expect_equal(result[length(result)], as.Date('1962-06-01'))
})

test_that('get.dtindex.continuous works for UKgas', {
  result <- ggfortify:::get.dtindex.continuous(UKgas, length = 6)
  expect_equal(result[1], as.Date('1987-01-01'))
  expect_equal(result[length(result)], as.Date('1988-04-01'))
})

test_that('confint.acf works for AirPassengers', {
  result <- ggfortify:::confint.acf(stats::acf(AirPassengers, plot = FALSE))
  expect_equal(result, 0.1633303, tolerance = .000001)

  result <- ggfortify:::confint.acf(stats::acf(AirPassengers, plot = FALSE), ci.type = 'ma')
  expected <- c(NA, 0.1633303, 0.2731862, 0.3399018, 0.3876238, 0.4248224, 0.4556929,
                0.4821335, 0.5058641, 0.5280447, 0.5503176, 0.5737563, 0.5988899,
                0.6241140, 0.6454578, 0.6624988, 0.6761814, 0.6875039, 0.6971273,
                0.7054849, 0.7130967, 0.7203560)
  expect_equal(result, expected, tolerance = .000001)
})

test_that('fitted/residuals works for Arima/ar', {
  skip_if_not_installed("forecast")
  m <- stats::ar(UKgas)
  resid <- residuals(m)
  testthat::expect_false(is.null(resid))
  fit <- fitted(m)
  testthat::expect_false(is.null(fitted))

  original <- as.vector(UKgas)[7:length(UKgas)]
  calcurated <- as.vector(fit + resid)[7:length(UKgas)]
  testthat::expect_equal(original, calcurated)

  m <- stats::arima(UKgas)
  resid <- residuals(m)
  testthat::expect_false(is.null(resid))
  fit <- fitted(m)
  testthat::expect_false(is.null(fitted))

  original <- as.vector(UKgas)[7:length(UKgas)]
  calcurated <- as.vector(fit + resid)[7:length(UKgas)]
  testthat::expect_equal(original, calcurated)
})

test_that('ggfreqplot', {
  p <- ggfreqplot(AirPassengers)
  expect_true(inherits(p, 'ggplot'))

  p <- ggfreqplot(AirPassengers, facet.labeller = c(3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2))
  expect_true(inherits(p, 'ggplot'))
})


test_that('ggcpgram', {
  p <- ggcpgram(AirPassengers)
  expect_true(inherits(p, 'ggplot'))
})

test_that('ggtsdiag', {
  p <- ggtsdiag(arima(AirPassengers))
  expect_true(inherits(p, 'ggmultiplot'))
})

#' Convert \code{base::table} to \code{data.frame}
#'
#' @param model \code{base::table} instance
#' @param data original dataset, if needed
#' @param ... other arguments passed to methods
#' @return data.frame
fortify_base <- function(model, data, ...) {
  # the function should be base for all fortify roxygen
  as.data.frame(model)
}

#' Convert \code{base::table} to \code{data.frame}
#'
#' @param model \code{base::table} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(Titanic)
#' @export
fortify.table <- function(model, data, ...) {
  fortify_base(model, data, ...)
}

#' Convert \code{base::matrix} to \code{data.frame}
#'
#' Different from \code{as.data.frame}
#'
#' @param model \code{base::matrix} instance
#' @param data original dataset, if needed
#' @param compat Logical frag to specify the behaviour when converting matrix which has no column name.
#' If \code{FALSE}, result has character columns like c('1', '2', ...).
#' If \code{TRUE}, result has character columns like c('V1', 'V2', ...).
#' @param ... other arguments passed to methods
#' @return data.frame
#' @examples
#' fortify(matrix(1:6, nrow=2, ncol=3))
#' @export
fortify.matrix <- function(model, data = NULL, compat = FALSE, ...) {
  d <- as.data.frame(model)
  if ( (!compat) && is.null(colnames(model)) ) {
    # set numeric column names
    colnames(d) <- 1:ncol(model)
  }
  # dplyr doesn't guarantee rownames
  d <- cbind_wraps(d, data)
  post_fortify(d)
}

#' Plot \code{base::matrix}
#'
#' @param object \code{base::matrix} instance
#' @param original Combined to data by column if provided. Intended to be used for stat functions which
#' returns not containing original data.
#' @param geom Geometric string for plotting. 'tile' or 'point'.
#' @param colour colour for points ('point' only)
#' @param size point size
#' @param alpha alpha
#' @param fill fill colour. Ignored if scale keyword is passed. ('tile' Only)
#' @param shape point shape
#' @inheritParams plot_label
#' @param scale (Deprecated) \code{ggplot2::scale} instance to plot. ('tile' Only)
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(matrix(rnorm(20), nc = 5))
#' autoplot(matrix(rnorm(20), nc = 5), fill = 'red')
#' autoplot(matrix(rnorm(20), nc = 2), geom = 'point')
#' @export
autoplot.matrix <- function (object, original = NULL, geom = 'tile',
                             colour = NULL, size = NULL, alpha = NULL,
                             fill = '#0000FF', shape = NULL,
                             label = FALSE,
                             label.label = 'rownames',
                             label.colour = colour,
                             label.alpha = NULL,
                             label.size = NULL,
                             label.angle = NULL,
                             label.family = NULL,
                             label.fontface = NULL,
                             label.lineheight = NULL,
                             label.hjust = NULL,
                             label.vjust = NULL,
                             label.repel = FALSE,
                             scale = NULL,
                             xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                             main = NULL, xlab = NULL, ylab = NULL, asp = NULL,
                             ...) {
  if (geom == 'tile') {
    fortified <- ggplot2::fortify(object, original = original)
    fortified$Index <- rownames(fortified)
    cols <- colnames(fortified)
    gathered <- tidyr::gather_(fortified, 'variable', 'value',
                               cols[cols != 'Index'])

    if (is.null(scale)) {
      scale <- ggplot2::scale_fill_gradient(low = "white", high = fill)
    } else {
      message <- paste0("Argument 'scale' is being deprecated. Use + operator to returned ggplot instance.")
      warning(message, call. = FALSE)
    }
    ylim <- rev(levels(as.factor(gathered$Index)))
    mapping <- ggplot2::aes_string(x = 'variable', y = 'Index', fill = 'value')
    p <- ggplot2::ggplot(mapping = mapping) +
      geom_factory(geom_tile, gathered, alpha = alpha) +
      scale +
      xlab('Columns') + ylab('Rows') + ylim(ylim)
  } else if (geom == 'point') {
    if (ncol(object) != 2) {
      stop("Number of columns must be 2 to plot with 'geom = point'")
    }
    if (is.logical(shape) && !shape && missing(label)) {
      # if label is missing and shape=FALSE, turn label to TRUE
      label <- TRUE
    }
    plot.data <- ggplot2::fortify(object, original = original, compat = TRUE)
    plot.data$rownames <- rownames(plot.data)
    mapping <- ggplot2::aes_string(x = 'V1', y = 'V2')
    p <- ggplot2::ggplot(mapping = mapping)
    if (!is.logical(shape) || shape) {
      p <- p + geom_factory(geom_point, plot.data, colour = colour, size = size,
                            alpha = alpha, fill = fill, shape = shape)
    }
    p <- plot_label(p = p, data = plot.data,
                    label = label,
                    label.label = label.label,
                    label.colour = label.colour,
                    label.alpha = label.alpha,
                    label.size = label.size,
                    label.angle = label.angle,
                    label.family = label.family,
                    label.fontface = label.fontface,
                    label.lineheight = label.lineheight,
                    label.hjust = label.hjust,
                    label.vjust = label.vjust,
                    label.repel = label.repel)
  } else {
    stop("Invalid geom is specified. Use 'tile' or 'point'.")
  }
  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{glmnet::glmnet} to \code{data.frame}
#'
#' @param model \code{glmnet::glmnet} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(glmnet::glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
#' @export
fortify.glmnet <- function(model, data = NULL, ...) {
  beta <- as.matrix(model$beta)
  ret <- cbind_wraps(as.data.frame(t(beta)),
                     data.frame(Df = model$df, Lambda = model$lambda,
                                dev.ratio = model$dev.ratio))
  post_fortify(ret)
}

#' Autoplot \code{glmnet::glmnet}
#'
#' @param object \code{glmnet::glmnet} instance
#' @param xvar values to be dranw on the X axis. Either "norm" (L1-norm), "lambda" (log-lambda sequence) or "dev" (percent deviance)
#' @param label.n Number of Df labels
#' @inheritParams plot_label
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(glmnet::glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
#' @export
autoplot.glmnet <- function (object,
                             xvar = c("norm", "lambda", "dev"),
                             label.n = 7,
                             label = TRUE, label.label = 'Df',
                             label.colour = NULL, label.alpha = NULL,
                             label.size = NULL, label.angle = NULL,
                             label.family = NULL, label.fontface = NULL,
                             label.lineheight = NULL,
                             label.hjust = NULL, label.vjust = NULL,
                             xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                             main = NULL, xlab = NULL,
                             ylab = 'Coefficients', asp = NULL,
                             ...) {
  beta <- as.matrix(object$beta)
  xvar <- match.arg(xvar)

  switch(xvar, norm = {
    index <- apply(abs(beta), 2, sum)
    iname <- "L1 Norm"
  }, lambda = {
    index <- log(object$lambda)
    iname <- "Log Lambda"
  }, dev = {
    index <- object$dev.ratio
    iname <- "Fraction Deviance Explained"
  })
  if (is.null(xlab)) {
    xlab <- iname
  }
  plot.data <- ggplot2::fortify(object)
  cols <- rownames(beta)
  plot.data$index <- index
  indexer <- seq(0, max(plot.data$index), length = label.n)
  # take closest values
  indexer <- sapply(indexer, function(x) which.min(abs(plot.data$index - x)))
  label.data <- plot.data[indexer, ]

  # preparing plot data
  plot.data <- tidyr::gather_(plot.data, 'variable', 'value', cols)
  # preparing label data (no need to gather)
  label.data$label_y <- rep(max(plot.data$value), nrow(label.data))

  p <- ggplot2::ggplot(data = plot.data) +
    ggplot2::geom_line(aes_string(x = 'index', y = 'value', colour = 'variable'), ...)

  p <- plot_label(p = p, data = label.data, x = 'index', y = 'label_y',
                  label = label,
                  label.label = label.label, label.colour = label.colour,
                  label.alpha = label.alpha, label.size = label.size,
                  label.angle = label.angle, label.family = label.family,
                  label.fontface = label.fontface, label.lineheight = label.lineheight,
                  label.hjust = label.hjust, label.vjust = label.vjust)

  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Convert \code{glmnet::cv.glmnet} to \code{data.frame}
#'
#' @param model \code{glmnet::cv.glmnet} instance
#' @inheritParams fortify_base
#' @return data.frame
#' @examples
#' fortify(glmnet::cv.glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
#' @export
fortify.cv.glmnet <- function(model, data = NULL, ...) {
  d <- data.frame('lambda' = log(model$lambda), 'cvm' = model$cvm,
                  'cvup' = model$cvup, 'cvlo' = model$cvlo,
                  'nz' = model$nz)
  post_fortify(d)
}

#' Autoplot \code{glmnet::cv.glmnet}
#'
#' @param object \code{glmnet::cv.glmnet} instance
#' @param sign.lambda Either plot against log(lambda) (default) or its negative if \code{sign.lambda=-1}.
#' @param label.n Number of Df labels
#' @inheritParams plot_label
#' @inheritParams post_autoplot
#' @param ... other arguments passed to methods
#' @return ggplot
#' @examples
#' autoplot(glmnet::cv.glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
#' @export
autoplot.cv.glmnet <- function (object,
                                sign.lambda = 1,
                                label.n = 12,
                                label = TRUE,
                                label.label = 'nz',
                                label.colour = NULL,
                                label.alpha = NULL,
                                label.size = NULL,
                                label.angle = NULL,
                                label.family = NULL,
                                label.fontface = NULL,
                                label.lineheight = NULL,
                                label.hjust = NULL,
                                label.vjust = NULL,
                                label.repel = FALSE,
                                xlim = c(NA, NA), ylim = c(NA, NA), log = "",
                                main = NULL, xlab = NULL,
                                ylab = NULL, asp = NULL,
                                ...) {

  if (is.null(xlab)) {
    xlab <- 'log(Lambda)'
    if (sign.lambda < 0) {
      xlab <- paste("-", xlab, sep = "")
    }
  }
  if (is.null(ylab)) {
    type_measure <- object$name
    ylab <- type_measure[[names(type_measure)[[1]]]]
  }
  plot.data <- ggplot2::fortify(object)
  plot.data$lambda <- sign.lambda * plot.data$lambda
  plot.data$label <- rep(max(object$cvup), nrow(plot.data))

  p <- ggplot2::ggplot(plot.data) +
    geom_point(aes_string('lambda', 'cvm'), ...)

  p <- p + ggplot2::geom_errorbar(aes_string(x = 'lambda', ymin = 'cvlo', ymax = 'cvup'), ...)

  indexer <- seq(1, nrow(plot.data), length = label.n)
  p <- plot_label(p = p, data = plot.data[indexer, ], x = 'lambda', y = 'label',
                  label = label,
                  label.label = label.label,
                  label.colour = label.colour,
                  label.alpha = label.alpha,
                  label.size = label.size,
                  label.angle = label.angle,
                  label.family = label.family,
                  label.fontface = label.fontface,
                  label.lineheight = label.lineheight,
                  label.hjust = label.hjust,
                  label.vjust = label.vjust,
                  label.repel = label.repel)

  p <- p +
    ggplot2::geom_vline(xintercept = sign.lambda * log(object$lambda.min), linetype = 'dashed') +
    ggplot2::geom_vline(xintercept = sign.lambda * log(object$lambda.1se), linetype = 'dashed')

  p <- post_autoplot(p = p, xlim = xlim, ylim = ylim, log = log,
                     main = main, xlab = xlab, ylab = ylab, asp = asp)
  p
}

#' Connect observations by stairs.
#'
#' @param mapping the aesthetic mapping
#' @param data a layer specific dataset
#' @param stat the statistical transformation to use on the data for this layer
#' @param position the position adjustment to use for overlapping points on this layer
#' @param na.rm logical frag whether silently remove missing values
#' @param ... other arguments passed to methods
geom_confint <- function (mapping = NULL, data = NULL, stat = "identity",
                          position = "identity", na.rm = FALSE, ...) {
  ggplot2::layer(mapping = mapping,
                 data = data,
                 stat = stat,
                 geom = GeomConfint,
                 position = position,
                 params = list(na.rm = na.rm, ...))
}

GeomConfint <- ggplot2::ggproto('GeomConfint', ggplot2::GeomRibbon,
  required_aes = c("x", "ymin", "ymax"),
  draw_group = function(self, data, panel_scales, coord, na.rm = FALSE) {
    if (na.rm) data <- data[stats::complete.cases(self$required_aes), ]
    data <- data[order(data$group, data$x), ]
    data <- self$stairstep_confint(data)
    ggplot2:::GeomRibbon$draw_group(data, panel_scales, coord, na.rm = FALSE)
  },
  stairstep_confint = function (data) {
    data <- as.data.frame(data)[order(data$x), ]
    n <- nrow(data)
    ys <- rep(1:n, each = 2)[-2 * n]
    xs <- c(1, rep(2:n, each = 2))
    data.frame(x = data$x[xs], ymin = data$ymin[ys], ymax = data$ymax[ys],
               data[xs, setdiff(names(data), c("x", "ymin", "ymax"))])
  }
)

context('test stats lm')

# Note the sort() here is to make sure the results match in CRAN MKL build
expect_sorted_equal <- function(a, b) expect_equal(sort(a), sort(b))

test_that('fortify.lm works for USArrests', {
  skip_on_cran()
  skip_on_travis()
  fortified <- ggplot2::fortify(lm(Murder ~ Assault + UrbanPop, data = USArrests))
  expect_equal(is.data.frame(fortified), TRUE)

  expected_names <- c('Murder', 'Assault', 'UrbanPop', '.hat', '.sigma',
                      '.cooksd', '.fitted', '.resid', '.stdresid')
  expect_equal(colnames(fortified), expected_names)
  expect_equal(rownames(fortified), rownames(USArrests))

  fortified <- ggplot2::fortify(glm(Murder ~ Assault + UrbanPop, data = USArrests))
  expect_equal(is.data.frame(fortified), TRUE)

  expected_names <- c('Murder', 'Assault', 'UrbanPop', '.hat', '.sigma',
                      '.cooksd', '.fitted', '.resid', '.stdresid')
  expect_equal(colnames(fortified), expected_names)
  expect_equal(rownames(fortified), rownames(USArrests))
})

test_that('autoplot.lm can accept + operator', {
  skip_on_cran()
  skip_on_travis()
  p <- autoplot(lm(Petal.Width~Petal.Length, data = iris), size = 5) + theme_bw()
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p@plots), 4)

  p <- autoplot(lm(Petal.Width~Petal.Length, data = iris), which = 1:6) + scale_colour_brewer()
  expect_true(is(p, 'ggmultiplot'))
  expect_equal(length(p@plots), 6)
})


test_that('autoplot.lm works for USArrests', {
  skip_on_cran()
  skip_on_travis()
  lm.out <- lm(Murder ~ Assault + UrbanPop, data = USArrests)
  p <- autoplot(lm.out)
  expect_true(is(p, 'ggmultiplot'))

  # Residuals vs Fitted
  assert_lm1 <- function(p) {
    expect_equal(length(p$layers), 4)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(10.98829412396, 12.61896752124, 12.55584079695, 9.32452021199, 11.27584647663, 8.69296623258))
    expect_sorted_equal(ld1$y, c(2.211705876045, -2.618967521238, -4.455840796945, -0.524520211986, -2.275846476629, -0.792966232577))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(9.80152442166, 10.45296738353, 12.55584079695))
    expect_sorted_equal(ld4$y, c(7.59847557834, -4.55296738353, -4.45584079695))
    expect_equal(ld4$label, c("Georgia", "Delaware", "Arizona"))
  }
  p <- autoplot(lm.out, which = 1)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm1(p[[1]])

  # Q-Q plot
  assert_lm2 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(1.0364333894938, -1.2265281200366, -1.8807936081513, -0.0250689082587, -0.9541652531462, -0.1763741647809))
    expect_sorted_equal(ld1$y, c(0.876705970824, -1.066488546121, -1.793656590602, -0.208487054020, -0.928454258706, -0.312994001894))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(2.32634787404, -2.32634787404, -1.88079360815))
    expect_sorted_equal(ld3$y, c(2.99084842914, -1.79592192731, -1.79365659060))
    expect_equal(ld3$label, c("Georgia", "Delaware", "Arizona"))
  }
  p <- autoplot(lm.out, which = 2)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm2(p[[1]])

  # Scale Location
  assert_lm3 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(10.98829412396, 12.61896752124, 12.55584079695, 9.32452021199, 11.27584647663, 8.69296623258))
    expect_sorted_equal(ld1$y, c(0.936325782420, 1.032709323150, 1.339274650922, 0.456603826112, 0.963563313284, 0.559458668620))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(9.80152442166, 10.45296738353, 12.55584079695))
    expect_sorted_equal(ld3$y, c(1.72940695880, 1.34012011675, 1.33927465092))
    expect_equal(ld3$label, c("Georgia", "Delaware", "Arizona"))
  }
  p <- autoplot(lm.out, which = 3)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm3(p[[1]])

  # Cook's distance
  assert_lm4 <- function(p) {
    expect_equal(length(p$layers), 2)
    expect_true(is(p$layers[[1]]$geom, 'GeomLinerange'))
    expect_true(is(p$layers[[2]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(1, 2, 3, 4, 5, 6))
    expect_sorted_equal(ld1$y, c(0.011780302261585, 0.039384807123982, 0.084368193847733, 0.000749502795764, 0.031007744531649, 0.001212457891192))

    ld2 <- head(ggplot2:::layer_data(p, 2))
    expect_sorted_equal(ld2$x, c(33, 11, 10))
    expect_sorted_equal(ld2$y, c(0.1238456342775, 0.1116513094557, 0.0934772277625))
    expect_equal(ld2$label, c("North Carolina", "Hawaii", "Georgia"))
  }
  p <- autoplot(lm.out, which = 4)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm4(p[[1]])

  # Residuals vs Leverage
  assert_lm5 <- function(p) {
    expect_equal(length(p$layers), 5)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[5]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0439588522340, 0.0941055398350, 0.0729343164009, 0.0491849650378, 0.0974013722825, 0.0357999991845))
    expect_sorted_equal(ld1$y, c(0.876705970824, -1.066488546121, -1.793656590602, -0.208487054020, -0.928454258706, -0.312994001894))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))
    ld5 <- head(ggplot2:::layer_data(p, 5))

    expect_sorted_equal(ld5$x, c(0.1831430655646, 0.1213004188902, 0.0303971016107))
    expect_sorted_equal(ld5$y, c(-1.28729694510, 1.55769202765, 2.99084842914))
    expect_equal(ld5$label, c("North Carolina", "Hawaii", "Georgia"))
  }
  p <- autoplot(lm.out, which = 5)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm5(p[[1]])

  # Cook's dist vs Leverage
  assert_lm6 <- function(p) {
    expect_equal(length(p$layers), 11)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))
    expect_true(is(p$layers[[5]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[6]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[7]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[8]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[9]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[10]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[11]]$geom, 'GeomAbline'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0439588522340, 0.0941055398350, 0.0729343164009, 0.0491849650378, 0.0974013722825, 0.0357999991845))
    expect_sorted_equal(ld1$y, c(0.011780302261585, 0.039384807123982, 0.084368193847733, 0.000749502795764, 0.031007744531649, 0.001212457891192))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(0.1831430655646, 0.1213004188902, 0.0303971016107))
    expect_sorted_equal(ld4$y, c(0.1238456342775, 0.1116513094557, 0.0934772277625))
    expect_equal(ld4$label, c("North Carolina", "Hawaii", "Georgia"))

    ld5 <- head(ggplot2:::layer_data(p, 5))
    ld6 <- head(ggplot2:::layer_data(p, 6))
    ld7 <- head(ggplot2:::layer_data(p, 7))
    ld8 <- head(ggplot2:::layer_data(p, 8))
  }
  p <- autoplot(lm.out, which = 6)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm6(p[[1]])

  # All
  p <- autoplot(lm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  assert_lm1(p[[1]])
  assert_lm2(p[[2]])
  assert_lm3(p[[3]])
  assert_lm4(p[[4]])
  assert_lm5(p[[5]])
  assert_lm6(p[[6]])
})

test_that('autoplot.lm works for binomial', {
  skip_on_cran()
  skip_on_travis()
  library(MASS)
  data(menarche)
  glm.out <- glm(cbind(Menarche, Total-Menarche) ~ Age, family=binomial(logit), data=menarche)
  p <- autoplot(glm.out)
  expect_true(is(p, 'ggmultiplot'))

  # Residuals vs Fitted
  assert_glm1 <- function(p) {
    expect_equal(length(p$layers), 4)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(-6.19596641798, -4.56399806976, -3.96016978091, -3.55217769386, -3.14418560680, -2.73619351974))
    expect_sorted_equal(ld1$y, c(-1.237231196221, -2.036310110290, -1.873973216148, -0.804382661466, -0.995331982416, -0.160716281922))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(-4.56399806976, -3.96016978091, -1.10422517152))
    expect_sorted_equal(ld4$y, c(-2.03631011029, -1.87397321615, 1.36753877427))
    expect_equal(ld4$label, c("2", "3", "10"))
  }
  p <- autoplot(glm.out, which = 1)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm1(p[[1]])

  # Q-Q plot
  assert_glm2 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(-1.554773594597, -2.053748910632, -1.281551565545, -0.305480788099, -0.524400512708, 0.201893479142))
    expect_sorted_equal(ld1$y, c(-24.50740077119, -29.77401740678, -18.46742851899, -9.12522512251, -9.74576038627, -1.56377387491))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(2.05374891063, -2.05374891063, -1.55477359460))
    expect_sorted_equal(ld3$y, c(36.3641401028, -29.7740174068, -24.5074007712))
    expect_equal(ld3$label, c("25", "2", "1"))
  }
  p <- autoplot(glm.out, which = 2)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm2(p[[1]])

  # Scale Location
  assert_glm3 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(-6.19596641798, -4.56399806976, -3.96016978091, -3.55217769386, -3.14418560680, -2.73619351974))
    expect_sorted_equal(ld1$y, c(4.95049500264, 5.45655728521, 4.29737460771, 3.02079875571, 3.12182004386, 1.25050944615))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(7.46360865666, -4.56399806976, -6.19596641798))
    expect_sorted_equal(ld3$y, c(6.03026865926, 5.45655728521, 4.95049500264))
    expect_equal(ld3$label, c("25", "2", "1"))
  }
  p <- autoplot(glm.out, which = 3)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm3(p[[1]])

  # Cook's distance
  assert_glm4 <- function(p) {
    expect_equal(length(p$layers), 2)
    expect_true(is(p$layers[[1]]$geom, 'GeomLinerange'))
    expect_true(is(p$layers[[2]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(1, 2, 3, 4, 5, 6))
    expect_sorted_equal(ld1$y, c(0.01740112703384, 0.07680098087888, 0.04095037805283, 0.02157996281830, 0.02880296835881, 0.00103150878141))

    ld2 <- head(ggplot2:::layer_data(p, 2))
    expect_sorted_equal(ld2$x, c(10, 15, 17))
    expect_sorted_equal(ld2$y, c(0.1195191318650, 0.0984796718074, 0.0975367745871))
    expect_equal(ld2$label, c("10", "15", "17"))
  }
  p <- autoplot(glm.out, which = 4)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm4(p[[1]])

  # Residuals vs Leverage
  assert_glm5 <- function(p) {
    expect_equal(length(p$layers), 5)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[5]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0417141770381, 0.0645018028661, 0.0423719600257, 0.0675630552621, 0.0612564434679, 0.0704890316373))
    expect_sorted_equal(ld1$y, c(-1.263872692299, -2.105340961150, -1.914983130008, -0.833015273652, -1.027293345029, -0.166698855175))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))
    ld5 <- head(ggplot2:::layer_data(p, 5))

    expect_sorted_equal(ld5$x, c(0.0987328446734, 0.1069957513737, 0.0965342062351))
    expect_sorted_equal(ld5$y, c(1.44049871978, 1.30959756960, -1.30015928340))
    expect_equal(ld5$label, c("10", "15", "17"))
  }
  p <- autoplot(glm.out, which = 5)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm5(p[[1]])

  # Cook's dist vs Leverage
  assert_glm6 <- function(p) {
    expect_equal(length(p$layers), 8)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))
    expect_true(is(p$layers[[5]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[6]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[7]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[8]]$geom, 'GeomAbline'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0417141770381, 0.0645018028661, 0.0423719600257, 0.0675630552621, 0.0612564434679, 0.0704890316373))
    expect_sorted_equal(ld1$y, c(0.01740112703384, 0.07680098087888, 0.04095037805283, 0.02157996281830, 0.02880296835881, 0.00103150878141))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(0.0987328446734, 0.1069957513737, 0.0965342062351))
    expect_sorted_equal(ld4$y, c(0.1195191318650, 0.0984796718074, 0.0975367745871))
    expect_equal(ld4$label, c("10", "15", "17"))

    ld5 <- head(ggplot2:::layer_data(p, 5))
    ld6 <- head(ggplot2:::layer_data(p, 6))
    ld7 <- head(ggplot2:::layer_data(p, 7))
    ld8 <- head(ggplot2:::layer_data(p, 8))
  }
  p <- autoplot(glm.out, which = 6)
  expect_true(is(p, 'ggmultiplot'))
  assert_glm6(p[[1]])

  # All
  p <- autoplot(glm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  assert_glm1(p[[1]])
  assert_glm2(p[[2]])
  assert_glm3(p[[3]])
  assert_glm4(p[[4]])
  assert_glm5(p[[5]])
  assert_glm6(p[[6]])
})

test_that('autoplot.lm works for polynomial fit', {
  skip_on_cran()
  skip_on_travis()
  lm.out <- lm(mpg ~ poly(hp, 2, raw = TRUE), data = mtcars)
  p <- autoplot(lm.out)
  expect_true(is(p, 'ggmultiplot'))

  # Residuals vs Fitted
  assert_lm1 <- function(p) {
    expect_equal(length(p$layers), 4)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(22.0370777250, 22.0370777250, 24.2110834069, 22.0370777250, 15.9676503638, 22.6512422231))
    expect_sorted_equal(ld1$y, c(-1.037077725009, -1.037077725009, -1.411083406907, -0.637077725009, 2.732349636196, -4.551242223095))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(21.6786786013, 28.3220263408, 22.6512422231))
    expect_sorted_equal(ld4$y, c(8.72132139874, 5.57797365923, -4.55124222310))
    expect_equal(ld4$label, c("Lotus Europa", "Toyota Corolla", "Valiant"))
  }
  p <- autoplot(lm.out, which = 1)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm1(p[[1]])

  # Q-Q plot
  assert_lm2 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))

    expect_sorted_equal(ld1$x, c(-0.2776904398216, -0.1970990842943, -0.5334097062413, 0.0391760855031, 0.9467817563010, -1.6759397227734))
    expect_sorted_equal(ld1$y, c(-0.344361871272, -0.344361871272, -0.471006426443, -0.211541789241, 0.917989725233, -1.512317646071))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(2.15387469406, 1.67593972277, -1.67593972277))
    expect_sorted_equal(ld3$y, c(2.89542798400, 1.92989370759, -1.51231764607))
    expect_equal(ld3$label, c("Lotus Europa", "Toyota Corolla", "Valiant"))
  }
  p <- autoplot(lm.out, which = 2)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm2(p[[1]])

  # Scale Location
  assert_lm3 <- function(p) {
    expect_equal(length(p$layers), 3)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(22.0370777250, 22.0370777250, 24.2110834069, 22.0370777250, 15.9676503638, 22.6512422231))
    expect_sorted_equal(ld1$y, c(0.586823543557, 0.586823543557, 0.686299079442, 0.459936723083, 0.958117803422, 1.229763247975))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))

    expect_sorted_equal(ld3$x, c(21.6786786013, 28.3220263408, 22.6512422231))
    expect_sorted_equal(ld3$y, c(1.70159571697, 1.38920614294, 1.22976324798))
    expect_equal(ld3$label, c("Lotus Europa", "Toyota Corolla", "Valiant"))
  }
  p <- autoplot(lm.out, which = 3)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm3(p[[1]])

  # Cook's distance
  assert_lm4 <- function(p) {
    expect_equal(length(p$layers), 2)
    expect_true(is(p$layers[[1]]$geom, 'GeomLinerange'))
    expect_true(is(p$layers[[2]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(1, 2, 3, 4, 5, 6))
    expect_sorted_equal(ld1$y, c(0.001744913090237, 0.001744913090237, 0.004075898589070, 0.000658470054463, 0.019367006055189, 0.034788166928193))

    ld2 <- head(ggplot2:::layer_data(p, 2))
    expect_sorted_equal(ld2$x, c(31, 20, 28))
    expect_sorted_equal(ld2$y, c(0.514354642953, 0.165885431468, 0.122374068009))
    expect_equal(ld2$label, c("Maserati Bora", "Toyota Corolla", "Lotus Europa"))
  }
  p <- autoplot(lm.out, which = 4)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm4(p[[1]])

  # Residuals vs Leverage
  assert_lm5 <- function(p) {
    expect_equal(length(p$layers), 5)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomHline'))
    expect_true(is(p$layers[[4]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[5]]$geom, 'GeomText'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0422770645623, 0.0422770645623, 0.0522384229500, 0.0422770645623, 0.0644989134682, 0.0436403288747))
    expect_sorted_equal(ld1$y, c(-0.344361871272, -0.344361871272, -0.471006426443, -0.211541789241, 0.917989725233, -1.512317646071))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))
    ld5 <- head(ggplot2:::layer_data(p, 5))

    expect_sorted_equal(ld5$x, c(0.735896676687, 0.117868091819, 0.041953824628))
    expect_sorted_equal(ld5$y, c(-0.744167110444, 1.929893707586, 2.895427983998))
    expect_equal(ld5$label, c("Maserati Bora", "Toyota Corolla", "Lotus Europa"))
  }
  p <- autoplot(lm.out, which = 5)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm5(p[[1]])

  # Cook's dist vs Leverage
  assert_lm6 <- function(p) {
    expect_equal(length(p$layers), 11)
    expect_true(is(p$layers[[1]]$geom, 'GeomPoint'))
    expect_true(is(p$layers[[2]]$geom, 'GeomLine'))
    expect_true(is(p$layers[[3]]$geom, 'GeomBlank'))
    expect_true(is(p$layers[[4]]$geom, 'GeomText'))
    expect_true(is(p$layers[[5]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[6]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[7]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[8]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[9]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[10]]$geom, 'GeomAbline'))
    expect_true(is(p$layers[[11]]$geom, 'GeomAbline'))

    ld1 <- head(ggplot2:::layer_data(p, 1))
    expect_sorted_equal(ld1$x, c(0.0422770645623, 0.0422770645623, 0.0522384229500, 0.0422770645623, 0.0644989134682, 0.0436403288747))
    expect_sorted_equal(ld1$y, c(0.001744913090237, 0.001744913090237, 0.004075898589070, 0.000658470054463, 0.019367006055189, 0.034788166928193))
    ld2 <- head(ggplot2:::layer_data(p, 2))
    ld3 <- head(ggplot2:::layer_data(p, 3))
    ld4 <- head(ggplot2:::layer_data(p, 4))

    expect_sorted_equal(ld4$x, c(0.735896676687, 0.117868091819, 0.041953824628))
    expect_sorted_equal(ld4$y, c(0.514354642953, 0.165885431468, 0.122374068009))
    expect_equal(ld4$label, c("Maserati Bora", "Toyota Corolla", "Lotus Europa"))

    ld5 <- head(ggplot2:::layer_data(p, 5))
    ld6 <- head(ggplot2:::layer_data(p, 6))
    ld7 <- head(ggplot2:::layer_data(p, 7))
    ld8 <- head(ggplot2:::layer_data(p, 8))
  }
  p <- autoplot(lm.out, which = 6)
  expect_true(is(p, 'ggmultiplot'))
  assert_lm6(p[[1]])

  # All
  p <- autoplot(lm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  assert_lm1(p[[1]])
  assert_lm2(p[[2]])
  assert_lm3(p[[3]])
  assert_lm4(p[[4]])
  assert_lm5(p[[5]])
  assert_lm6(p[[6]])
})

test_that('autoplot.lm works with factors', {
  skip_on_cran()
  skip_on_travis()
  lm.out <- aov(Petal.Length ~ Species, data = iris)
  p <- autoplot(lm.out, which = 5)
  expect_true(is(p, 'ggmultiplot'))
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(p[[1]]$mapping$x, quo(.nf))

  lm.out <- aov(Petal.Length ~ Species, data = iris)
  p <- autoplot(lm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(p[[5]]$mapping$x, quo(.nf))

  lm.out <- lm(Volume ~ Girth, data = trees)
  p <- autoplot(lm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(p[[5]]$mapping$x, quo(.hat))
})


test_that('autoplot.lm works with characters', {
  skip_on_cran()
  skip_on_travis()
  iris <- transform(iris, Species_chr = as.character(Species), stringsAsFactors = FALSE)
  lm.out <- aov(Petal.Length ~ Species_chr, data = iris)
  p <- autoplot(lm.out, which = 5)
  expect_true(is(p, 'ggmultiplot'))
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(p[[1]]$mapping$x, quo(.nf))

  lm.out <- aov(Petal.Length ~ Species_chr, data = iris)
  p <- autoplot(lm.out, which = c(1, 2, 3, 4, 5, 6))
  expect_true(is(p, 'ggmultiplot'))
  if (utils::packageVersion("ggplot2") >= "2.3.0") expect_equal(p[[5]]$mapping$x, quo(.nf))
})

test_that('autoplot.lm can be used in ggsave()', {
  skip_on_cran()
  skip_on_travis()
  p <- autoplot(lm(Petal.Width~Petal.Length, data = iris), size = 5)
  ggsave(p, file='temp.png', h = 6, w = 6, units = "in", dpi = 300)
  expect_true(file.exists('temp.png'))
  unlink('temp.png')
})

test_that('fortify.lm works with AsIs response', {
  skip_on_cran()
  skip_on_travis()
  mdl_cars <- lm(I(dist ^ 2) ~ speed, data = cars)
  p <- autoplot(mdl_cars, which = 1:6)
  expect_true(is(p, 'ggmultiplot'))
  # Previous problems occurred when printing, so write to file
  ggsave(p, file='temp-test-AsIs.png', h = 6, w = 6, units = "in", dpi = 50)
  expect_true(file.exists('temp-test-AsIs.png'))
  unlink('temp-test-AsIs.png')
})

context('test stats')

test_that('fortify.stl works for AirPassengers', {
  fortified <- ggplot2::fortify(stats::stl(AirPassengers, s.window = 'periodic'))
  expect_true(is.data.frame(fortified))

  expected_names <- c('Index', 'Data', 'seasonal', 'trend', 'remainder')
  expect_equal(names(fortified), expected_names)
  expect_equal(as.vector(AirPassengers), as.vector(fortified[['Data']]))
  expect_equal(fortified$Index[1], as.Date('1949-01-01'))
  expect_equal(fortified$Index[nrow(fortified)], as.Date('1960-12-01'))

  fortified <- ggplot2::fortify(stats::decompose(AirPassengers))
  expect_true(is.data.frame(fortified))

  expected_names <- c('Index', 'Data', 'seasonal', 'trend', 'remainder')
  expect_equal(names(fortified), expected_names)
  expect_equal(as.vector(AirPassengers), as.vector(fortified[['Data']]))
  expect_equal(fortified$Index[1], as.Date('1949-01-01'))
  expect_equal(fortified$Index[nrow(fortified)], as.Date('1960-12-01'))
})

test_that('fortify.Arima works for AirPassengers', {
  skip_if_not_installed("forecast")
  skip_if_not_installed("fGarch")
  fortified <- ggplot2::fortify(ar(AirPassengers))
  expect_true(is.data.frame(fortified))
  expected_names <- c('Index', 'Data', 'Fitted', 'Residuals')
  expect_equal(names(fortified), expected_names)
  expect_equal(as.vector(AirPassengers), as.vector(fortified[['Data']]))
  expect_equal(fortified$Index[1], as.Date('1949-01-01'))
  expect_equal(fortified$Index[nrow(fortified)], as.Date('1960-12-01'))

  x <- AirPassengers
  m <- stats::ar(x) # create model with temporary variable
  x <- NULL
  fortified2 <- ggplot2::fortify(m, data = AirPassengers)
  expect_equal(fortified, fortified2)
  ggplot2::autoplot(m, data = AirPassengers)

  fortified <- ggplot2::fortify(stats::arima(AirPassengers))
  expect_true(is.data.frame(fortified))
  expected_names <- c('Index', 'Data', 'Fitted', 'Residuals')
  expect_equal(names(fortified), expected_names)
  expect_equal(as.vector(AirPassengers), as.vector(fortified[['Data']]))
  expect_equal(fortified$Index[1], as.Date('1949-01-01'))
  expect_equal(fortified$Index[nrow(fortified)], as.Date('1960-12-01'))
  ggplot2::autoplot(stats::arima(AirPassengers))

  x <- AirPassengers
  m <- stats::arima(x) # create model with temporary variable
  x <- NULL
  fortified2 <- ggplot2::fortify(m, data = AirPassengers)
  expect_equal(fortified, fortified2)
  ggplot2::autoplot(m, data = AirPassengers)

  fortified <- ggplot2::fortify(stats::HoltWinters(AirPassengers))
  expect_true(is.data.frame(fortified))

  expected_names <- c('Index', 'Data', 'xhat', 'level', 'trend', 'season', 'Residuals')
  expect_equal(names(fortified), expected_names)
  expect_equal(as.vector(AirPassengers), as.vector(fortified[['Data']]))
  expect_equal(fortified$Index[1], as.Date('1949-01-01'))
  expect_equal(fortified$Index[nrow(fortified)], as.Date('1960-12-01'))

  library(fGarch)
  d.fGarch <- fGarch::garchFit(formula = ~arma(1, 1) + garch(1, 1), data = UKgas, trace = FALSE)
  fortified <- ggplot2::fortify(d.fGarch)
  expected_names <- c('Index', 'Data', 'Fitted', 'Residuals')
  expect_equal(names(fortified), expected_names)
})

test_that('fortify.prcomp works for iris', {
  df <- iris[c(1, 2, 3, 4)]
  pcs <- c('PC1', 'PC2', 'PC3', 'PC4')
  expected_names <- c(names(df), pcs)

  fortified <- ggplot2::fortify(stats::prcomp(df, center = TRUE, scale = TRUE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), df)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(stats::prcomp(df, center = FALSE, scale = TRUE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), df)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(stats::prcomp(df, center = TRUE, scale = FALSE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), df)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(stats::prcomp(df, center = FALSE, scale = FALSE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), df)
  expect_equal(rownames(fortified), rownames(df))

  # attach original
  expected_names <- c(names(df), 'Species', pcs)
  fortified <- ggplot2::fortify(stats::prcomp(df), data = iris)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4, 5)]), iris)
  expect_equal(rownames(fortified), rownames(df))

  tmp <- stats::prcomp(df)
  class(tmp) <- 'unsupportedClass'
  expect_error(ggplot2::fortify(tmp, data = iris))
})


test_that('fortify.princomp works for iris', {
  df <- iris[c(1, 2, 3, 4)]
  pcs <- c('Comp.1', 'Comp.2', 'Comp.3', 'Comp.4')
  expected_names <- c(names(df), pcs)

  fortified <- ggplot2::fortify(stats::princomp(df))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), df)
  expect_equal(rownames(fortified), rownames(df))

  # attach original
  expected_names <- c(names(df), 'Species', pcs)
  fortified <- ggplot2::fortify(stats::princomp(df), data = iris)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4, 5)]), iris)
  expect_equal(rownames(fortified), rownames(df))

  p <- ggplot2::autoplot(stats::princomp(df), data = iris, colour = 'Species')
  expect_true(is(p, 'ggplot'))

  p <- ggplot2::autoplot(stats::princomp(df), data = iris, loadings.label = TRUE)
  expect_true(is(p, 'ggplot'))

  p <- ggplot2::autoplot(stats::princomp(df), data = iris, frame.type = 'convex')
  expect_true(is(p, 'ggplot'))

  expect_error(ggplot2::autoplot(stats::princomp(df), frame.type = 'invalid'))
})

test_that('fortify.factanal works for state.x77', {
  d.factanal <- stats::factanal(state.x77, factors = 3, scores = 'regression')
  pcs <- c('Factor1', 'Factor2', 'Factor3')

  fortified <- ggplot2::fortify(d.factanal)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), pcs)
  expect_equal(rownames(fortified), rownames(state.x77))

  # attach original
  fortified <- ggplot2::fortify(d.factanal, data = state.x77)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), c(colnames(state.x77), pcs))
  expect_equal(rownames(fortified), rownames(state.x77))
})

test_that('fortify.prcomp works for USArrests', {
  pcs <- c('PC1', 'PC2', 'PC3', 'PC4')
  expected_names <- c(names(USArrests), pcs)

  fortified <- ggplot2::fortify(stats::prcomp(USArrests, center = TRUE, scale = TRUE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))

  fortified <- ggplot2::fortify(stats::prcomp(USArrests, center = FALSE, scale = TRUE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))

  fortified <- ggplot2::fortify(stats::prcomp(USArrests, center = TRUE, scale = FALSE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))

  fortified <- ggplot2::fortify(stats::prcomp(USArrests, center = FALSE, scale = FALSE))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))

  # attach original
  fortified <- ggplot2::fortify(stats::prcomp(USArrests), data = USArrests)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))
})

test_that('fortify.princomp works for USArrests', {
  pcs <- c('Comp.1', 'Comp.2', 'Comp.3', 'Comp.4')
  expected_names <- c(names(USArrests), pcs)

  fortified <- ggplot2::fortify(stats::princomp(USArrests))
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))

  # attach original
  fortified <- ggplot2::fortify(stats::princomp(USArrests), data = USArrests)
  expect_true(is.data.frame(fortified))
  expect_equal(names(fortified), expected_names)
  expect_equal(data.frame(fortified[c(1, 2, 3, 4)]), USArrests)
  expect_equal(rownames(fortified), rownames(USArrests))
})

test_that('autoplot.prcomp works for iris with scale (default)', {

  # fails on CRAN i386 because components are inversed.
  skip_on_cran()

  obj <- stats::prcomp(iris[-5])

  exp_x <- c(-0.10658039, -0.10777226, -0.11471510, -0.10901118, -0.10835099, -0.09056763)
  exp_y <- c(-0.05293913, 0.02933742, 0.02402493, 0.05275710, -0.05415858, -0.12287329)

  p <- ggplot2::autoplot(obj)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep('black', 6))

  p <- ggplot2::autoplot(obj, data = iris, colour = 'Species')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("#F8766D", 6))

  p <- ggplot2::autoplot(obj, data = iris, loadings = TRUE, loadings.label = FALSE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- ggplot2:::layer_data(p, 2)
  expect_equal(ld$x, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$xend, c(0.05086374, -0.01189621, 0.12057301, 0.05042779), tolerance = 1e-4)
  expect_equal(ld$y, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$colour, rep("#FF0000", 4))

  p <- ggplot2::autoplot(obj, data = iris, loadings.label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 3)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))
  expect_true('GeomText' %in% class(p$layers[[3]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- ggplot2:::layer_data(p, 2)
  expect_equal(ld$x, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$xend, c(0.05086374, -0.01189621, 0.12057301, 0.05042779), tolerance = 1e-4)
  expect_equal(ld$y, rep(0, 4))
  expect_equal(ld$colour, rep("#FF0000", 4))
  ld <- ggplot2:::layer_data(p, 3)
  expect_equal(ld$x, c(0.05086374, -0.01189621, 0.12057301, 0.05042779), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.09241228, -0.10276734, 0.02440152, 0.01062366), tolerance = 1e-4)
  expect_equal(ld$colour, rep("#FF0000", 4))
  expect_equal(ld$label, c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"))

  p <- ggplot2::autoplot(obj, data = iris, frame.type = 'convex')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, c(0.15071626, 0.13846286, 0.12828254, -0.09474406, -0.10501689, -0.12769748), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.04265051, -0.19487526, -0.22776373, -0.22177981, -0.19537669, -0.02212193), tolerance = 1e-4)
  expect_equal(ld$fill, rep("grey20", 6))
  expect_equal(ld$alpha, rep(0.2, 6))

  p <- ggplot2::autoplot(obj, data = iris, frame.type = 'convex', shape = FALSE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  # label will be turned on
  expect_true('GeomText' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("#000000", 6))
  expect_equal(ld$label, c('1', '2', '3', '4', '5', '6'))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, c(0.15071626, 0.13846286, 0.12828254, -0.09474406, -0.10501689, -0.12769748), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.04265051, -0.19487526, -0.22776373, -0.22177981, -0.19537669, -0.02212193), tolerance = 1e-4)
  expect_equal(ld$fill, rep("grey20", 6))
  expect_equal(ld$alpha, rep(0.2, 6))
})

test_that('autoplot.prcomp works for iris without scale', {

  # fails on CRAN i386 because components are inversed.
  skip_on_cran()

  obj <- stats::prcomp(iris[-5])

  exp_x <- c(-2.684126, -2.714142, -2.888991, -2.745343, -2.728717, -2.280860)
  exp_y <- c(-0.3193972, 0.1770012, 0.1449494, 0.3182990, -0.3267545, -0.7413304)

  p <- ggplot2::autoplot(obj, scale = 0.)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep('black', 6))

  p <- ggplot2::autoplot(obj, scale = 0., data = iris, colour = 'Species')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("#F8766D", 6))

  p <- ggplot2::autoplot(obj, scale = 0, data = iris,
                         loadings = TRUE, loadings.label = FALSE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- ggplot2:::layer_data(p, 2)
  expect_equal(ld$x, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$xend, c(0.5441042, -0.1272572, 1.2898045, 0.5394407), tolerance = 1e-4)
  expect_equal(ld$y, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$colour, rep("#FF0000", 4))

  p <- ggplot2::autoplot(obj, scale = 0., data = iris,
                         loadings.label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 3)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))
  expect_true('GeomText' %in% class(p$layers[[3]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- ggplot2:::layer_data(p, 2)
  expect_equal(ld$x, rep(0, 4), tolerance = 1e-4)
  expect_equal(ld$xend, c(0.5441042, -0.1272572, 1.2898045, 0.5394407), tolerance = 1e-4)
  expect_equal(ld$y, rep(0, 4))
  expect_equal(ld$colour, rep("#FF0000", 4))
  ld <- ggplot2:::layer_data(p, 3)
  expect_equal(ld$x, c(0.5441042, -0.1272572, 1.2898045, 0.5394407), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.9885610, -1.0993321, 0.2610301, 0.1136443), tolerance = 1e-4)
  expect_equal(ld$colour, rep("#FF0000", 4))
  expect_equal(ld$label, c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"))

  p <- ggplot2::autoplot(obj, scale = 0., data = iris,
                         frame.type = 'convex')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, c(3.795645, 3.487055, 3.230674, -2.386039, -2.644750, -3.215939), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.2573230, -1.1757393, -1.3741651, -1.3380623, -1.1787646, -0.1334681), tolerance = 1e-4)
  expect_equal(ld$fill, rep("grey20", 6))
  expect_equal(ld$alpha, rep(0.2, 6))

  p <- ggplot2::autoplot(obj, scale = 0., data = iris,
                         frame.type = 'convex', shape = FALSE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  # label will be turned on
  expect_true('GeomText' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("#000000", 6))
  expect_equal(ld$label, c('1', '2', '3', '4', '5', '6'))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, c(3.795645, 3.487055, 3.230674, -2.386039, -2.644750, -3.215939), tolerance = 1e-4)
  expect_equal(ld$y, c(-0.2573230, -1.1757393, -1.3741651, -1.3380623, -1.1787646, -0.1334681), tolerance = 1e-4)
  expect_equal(ld$fill, rep("grey20", 6))
  expect_equal(ld$alpha, rep(0.2, 6))
})

test_that('autoplot.prcomp works for USArrests', {

  # fails on CRAN SPARC because components are inversed.
  skip_on_cran()

  obj <- stats::prcomp(USArrests)

  # scale
  exp_x <- c(0.10944879, 0.15678261, 0.20954726, 0.03097574, 0.18143395, 0.05907333)
  exp_y <- c(-0.11391408, -0.17894035, 0.08786745, -0.16621327, 0.22408730, 0.13651754)
  p <- ggplot2::autoplot(obj, label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomText' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$label, c("Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado"))
  expect_equal(ld$colour, rep("#000000", 6))

  # not scale
  exp_x <- c(64.80216, 92.82745, 124.06822, 18.34004, 107.42295, 34.97599)
  exp_y <- c(-11.448007, -17.982943, 8.830403, -16.703911, 22.520070, 13.719584)
  p <- ggplot2::autoplot(obj, scale = 0., label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomText' %in% class(p$layers[[2]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep("black", 6))
  ld <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$label, c("Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado"))
  expect_equal(ld$colour, rep("#000000", 6))
})

test_that('autoplot.princomp works for iris', {

  obj <- stats::princomp(iris[-5])

  p <- ggplot2::autoplot(obj, data = iris, colour = 'Species')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))

  p <- ggplot2::autoplot(obj, data = iris, loadings = TRUE, loadings.label = FALSE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))

  p <- ggplot2::autoplot(obj, data = iris, loadings.label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 3)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomSegment' %in% class(p$layers[[2]]$geom))
  expect_true('GeomText' %in% class(p$layers[[3]]$geom))

  p <- ggplot2::autoplot(obj, data = iris, frame.type = 'convex')
  expect_true(is(p, 'ggplot'))
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))

})

test_that('autoplot.prcomp plots the desired components', {

  # fails on CRAN SPARC because components are inversed.
  skip_on_cran()

  obj <- stats::prcomp(iris[-5])

  exp_x <- c(-0.0529391329513015, 0.0293374206773287, 0.0240249314006331,
             0.0527570984441502, -0.0541585777198945, -0.1228732921883)
  exp_y <- c(0.00815003672083961, 0.0614473273321588, -0.00522617400592586,
             -0.00921410146450414, -0.0262996114135432, -0.0492472720451544
  )

  p <- ggplot2::autoplot(obj, x = 2, y = 3)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld$x, exp_x, tolerance = 1e-4)
  expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep('black', 6))
  expect_equal(p$labels$x, "PC2 (5.31%)")
  expect_equal(p$labels$y, "PC3 (1.71%)")

})

test_that('autoplot.princomp plots the desired components', {

  # fails on CRAN and Travis because components are inversed.
  skip_on_travis()
  skip_on_cran()

  obj <- stats::princomp(iris[-5])

  exp_x <- c(-0.0531164839772263, 0.0294357037689672, 0.0241054171584106,
             0.052933839637522, -0.0543400139993682, -0.123284929161055)
  exp_y <- c(-0.00817734010291634, -0.0616531816016784, 0.00524368217559065,
             0.00924496956257505, 0.0263877175612184, 0.0494122549931978)


  p <- ggplot2::autoplot(obj, x = 2, y = 3)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  ld <- head(ggplot2:::layer_data(p, 1))
  # expect_equal(ld$x, exp_x, tolerance = 1e-4)
  # expect_equal(ld$y, exp_y, tolerance = 1e-4)
  expect_equal(ld$colour, rep('black', 6))
  expect_equal(p$labels$x, "Comp.2 (5.31%)")
  expect_equal(p$labels$y, "Comp.3 (1.71%)")

})

test_that('autoplot.factanal works for state.x77', {

  obj <- stats::factanal(state.x77, factors = 3, scores = 'regression')

  p <- ggplot2::autoplot(obj)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))

  p <- ggplot2::autoplot(obj, label = TRUE)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomText' %in% class(p$layers[[2]]$geom))

})


test_that('autoplot.factanal plots the desired components', {

  obj <- stats::factanal(state.x77, factors = 3, scores = 'regression')

  p <- ggplot2::autoplot(obj, x = 2, y = 3)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_equal(p$labels$x, "Factor2 (20.15%)")
  expect_equal(p$labels$y, "Factor3 (18.24%)")
})

test_that('fortify.dist works for eurodist', {
  fortified <- ggplot2::fortify(eurodist)
  expect_true(is.data.frame(fortified))
  expect_equal(dim(fortified), c(21, 21))
})

test_that('fortify.lfda works for iris', {
  skip_on_cran()
  library(lfda)
  k <- iris[, -5]
  y <- iris[, 5]
  r <- 3
  model <- lfda(k, y, r, metric = "plain")
  fortified <- ggplot2::fortify(model)
  expect_true(is.data.frame(fortified))
  model <- klfda(kmatrixGauss(k), y, r, metric = "plain")
  fortified <- ggplot2::fortify(model)
  expect_true(is.data.frame(fortified))
  model <- self(k, y, beta=0.1, r, metric = "plain")
  fortified <- ggplot2::fortify(model)
  expect_true(is.data.frame(fortified))
})

test_that('autoplot.lfda works for iris', {
  skip_on_cran()

  k <- iris[, -5]
  y <- iris[, 5]
  r <- 4
  model <- lfda::lfda(k, y, r, metric = "plain")
  p <- autoplot(model, data=iris, frame = TRUE, frame.colour='Species')
  expect_true(is(p, 'ggplot'))
})

test_that('autoplot.acf works', {

  p <- autoplot(stats::acf(AirPassengers, plot = FALSE))
  expect_true(is(p, 'ggplot'))

  p <- autoplot(stats::acf(AirPassengers, plot = FALSE), conf.int.type = 'ma')
  expect_true(is(p, 'ggplot'))

  p <- autoplot(stats::pacf(AirPassengers, plot = FALSE))
  expect_true(is(p, 'ggplot'))

  p <- autoplot(stats::ccf(AirPassengers, AirPassengers, plot = FALSE))
  expect_true(is(p, 'ggplot'))
})

test_that('autoplot.stepfun works', {

  p <- autoplot(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
  expect_true(is(p, 'ggplot'))

  fortified <- fortify(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
  expected <- data.frame(x = c(0, 1, 1, 2, 2, 3, 3, 4),
                         y = c(4, 4, 5, 5, 6, 6, 7, 7))
  expect_equal(fortified, expected)

  fortified <- fortify(stepfun(c(1), c(4, 5)))
  expected <- data.frame(x = c(0.9375, 1.0000, 1.0000, 1.0625),
                         y = c(4, 4, 5, 5))
  expect_equal(fortified, expected)

  fortified <- fortify(stepfun(c(1, 3, 4, 8), c(4, 5, 2, 3, 5)))
  expected <- data.frame(x = c(-1, 1, 1, 3, 3, 4, 4, 8, 8, 10),
                         y = c(4, 4, 5, 5, 2, 2, 3, 3, 5, 5))
  expect_equal(fortified, expected)

  fortified <- fortify(stepfun(c(1, 2, 3, 4, 5, 6, 7, 8, 10),
                               c(4, 5, 6, 7, 8, 9, 10, 11, 12, 9)))
  expected <- data.frame(x = c(0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 10, 10, 11),
                         y = c(4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 12, 9, 9))
  expect_equal(fortified, expected)
})


test_that('autoplot.spec works', {
  result <- stats::spec.ar(AirPassengers)
  p <- autoplot(result)
  expect_true(is(p, 'ggplot'))

  expect_equal(sum(fortify(result)[1]), 1500, tolerance = 0.01)
  expect_equal(sum(fortify(result)[2]), 684799.7, tolerance = 0.01)
})

context('test survival')

test_that('fortify.survfit works for lung', {
  skip_if_not_installed("survival")
  library(survival)
  d.survfit <- survival::survfit(Surv(time, status) ~ sex, data = lung)
  fortified <- ggplot2::fortify(d.survfit)
  expect_equal(is.data.frame(fortified), TRUE)
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower', 'strata')
  expect_equal(names(fortified), expected_names)
  expect_equal(dim(fortified), c(206, 9))

  p <- ggplot2::autoplot(d.survfit)
  expect_true(is(p, 'ggplot'))

  p <- ggplot2::autoplot(d.survfit, surv.geom = 'line')
  expect_true(is(p, 'ggplot'))

  fortified2 <- ggplot2::fortify(d.survfit, fun = 'event')
  expect_true(is.data.frame(fortified))
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower', 'strata')
  expect_equal(names(fortified), expected_names)
  expect_equal(dim(fortified), c(206, 9))
  expect_equal(fortified$surv, 1 - fortified2$surv)
  expect_equal(fortified$upper, 1 - fortified2$upper)
  expect_equal(fortified$lower, 1 - fortified2$lower)

  p <- ggplot2::autoplot(d.survfit, fun = 'event')
  expect_true(is(p, 'ggplot'))

  p <- ggplot2::autoplot(d.survfit, fun = function(x) { 1-x })
  expect_true(is(p, 'ggplot'))

  fortified <- ggplot2::fortify(d.survfit, surv.connect = TRUE)
  expect_true(is.data.frame(fortified))
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower', 'strata')
  expect_equal(names(fortified), expected_names)
  expect_equal(dim(fortified), c(208, 9))
  expected <- data.frame(time = c(0, 0), n.risk = c(138, 138), n.event = c(0, 0), n.censor = c(0, 0),
                         surv = c(1, 1), std.err = c(0, 0), upper = c(1, 1), lower = c(1, 1),
                         strata = factor(c('1', '2')))
  expect_equal(fortified[1:2, ], expected)

  p <- ggplot2::autoplot(d.survfit, surv.connect = TRUE)
  expect_true(is(p, 'ggplot'))

  p <- ggplot2::autoplot(d.survfit, surv.connect = FALSE)
  expect_true(is(p, 'ggplot'))

  d.survfit <- survfit(Surv(time, status) ~ 1, data = lung)
  fortified <- ggplot2::fortify(d.survfit)

  expect_true(is.data.frame(fortified))
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower')
  expect_equal(names(fortified), expected_names)
  expect_equal(dim(fortified), c(186, 8))

  p <- ggplot2::autoplot(d.survfit)
  expect_true(is(p, 'ggplot'))

  fortified <- ggplot2::fortify(d.survfit, surv.connect = TRUE)
  expect_equal(is.data.frame(fortified), TRUE)
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower')
  expect_equal(names(fortified), expected_names)
  expect_equal(dim(fortified), c(187, 8))
  expected <- data.frame(time = 0, n.risk = 228, n.event = 0, n.censor = 0,
                         surv = 1, std.err = 0, upper = 1, lower = 1)
  expect_equal(fortified[1, ], expected)

  p <- ggplot2::autoplot(d.survfit)
  expect_true(is(p, 'ggplot'))
})

test_that('autoplot retains order of alphabetically unordered factor levels', {
  skip_if_not_installed("survival")
  library(survival)
  livingStatus <- sample(0:1, 20, replace = TRUE)
  followupTime <- rpois(20, 300)
  samplesGroups <- factor(sample(c("Low", "High"), 20, replace = TRUE), levels = c("Low", "High"))
  survivalData <- Surv(followupTime, livingStatus)
  survivalFit <- survfit(survivalData ~ samplesGroups)
  plotElements <- ggplot2::autoplot(survivalFit, conf.int = FALSE, ylim = c(0, 1))
  expect_equal(levels(plotElements[["data"]][, "strata"]), c("Low", "High"))
})

test_that('fortify.survfit works for simple data', {
  skip_if_not_installed("survival")
  library(survival)
  tdata <- data.frame(time = c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4),
                      status = rep(c(1, 0, 2), 4),
                      n = c(12, 3, 2, 6, 2, 4, 2, 0, 2, 3, 3, 5))
  fit  <- survfit(Surv(time, time, status, type='interval') ~1, data = tdata, weight = n)
  fortified <- fortify(fit)
  expected <- data.frame(time = c(1, 2, 3, 4),
                        n.risk = c(44.000000000, 20.652979445, 9.318098786, 6.634779353),
                        n.event = c(20.347020555, 9.334880659, 2.683319433, 3.634779353),
                        n.censor = c(3, 2, 0, 3),
                        surv = c(0.53756771467, 0.29459403255, 0.20976021500, 0.09484575319),
                        std.err = c(0.1398238147, 0.2438966932, 0.3207623423, 0.5343228432),
                        upper = c(0.7070521134, 0.4751483078, 0.3933295002, 0.2702927012),
                        lower = c(0.40870968683, 0.18264959088, 0.11186383878, 0.03328139035))
  expect_equal(fortified, expected, tolerance = 0.5, scale = 1)

  fortified <- fortify(fit, surv.connect = TRUE)
  expected <- data.frame(time = c(0, 1, 2, 3, 4),
                        n.risk = c(44.000000000, 44.000000000, 20.652979445, 9.318098786, 6.634779353),
                        n.event = c(0, 20.347020555, 9.334880659, 2.683319433, 3.634779353),
                        n.censor = c(0, 3, 2, 0, 3),
                        surv = c(1.0, 0.53756771467, 0.29459403255, 0.20976021500, 0.09484575319),
                        std.err = c(0, 0.1398238147, 0.2438966932, 0.3207623423, 0.5343228432),
                        upper = c(1.0, 0.7070521134, 0.4751483078, 0.3933295002, 0.2702927012),
                        lower = c(1.0, 0.40870968683, 0.18264959088, 0.11186383878, 0.03328139035))
  expect_equal(fortified, expected, tolerance = 0.5, scale = 1)

  p <- ggplot2::autoplot(fit)
  expect_true(is(p, 'ggplot'))

  tdata <- data.frame(time = c(1, 1, 2, 2, 3, 3, 4, 4),
                      status = rep(c(1, 2), 4),
                      n = c(24, 3, 20, 4, 18, 2, 15, 3))

  fit <- survfit(coxph(Surv(time, status) ~ 1, data = tdata))
  fortified <- fortify(fit)
  expected <- data.frame(time = c(1, 2, 3, 4),
                         n.risk = c(8, 6, 4, 2),
                         n.event = c(1, 1, 1, 1),
                         n.censor = c(1, 1, 1, 1),
                         surv = c(0.8824969026, 0.7470175003, 0.5817778142, 0.3528660815),
                         std.err = c(0.1250000000, 0.2083333333, 0.3254270698, 0.5965758776),
                         upper = c(1, 1, 1, 1),
                         lower = c(0.6907374403, 0.4965890298, 0.3074348749, 0.1095982468))
  expect_equal(fortified, expected)

  fortified <- fortify(fit, surv.connect = TRUE)
  expected <- data.frame(time = c(0, 1, 2, 3, 4),
                         n.risk = c(8, 8, 6, 4, 2),
                         n.event = c(0, 1, 1, 1, 1),
                         n.censor = c(0, 1, 1, 1, 1),
                         surv = c(1, 0.8824969026, 0.7470175003, 0.5817778142, 0.3528660815),
                         std.err = c(0, 0.1250000000, 0.2083333333, 0.3254270698, 0.5965758776),
                         upper = c(1, 1, 1, 1, 1),
                         lower = c(1, 0.6907374403, 0.4965890298, 0.3074348749, 0.1095982468))
  expect_equal(fortified, expected)

  p <- ggplot2::autoplot(fit)
  expect_true(is(p, 'ggplot'))
})

test_that('fortify.survfit works for simple multistate data', {
  skip_if_not_installed("survival")
  library(survival)
  tdata <- data.frame(time = c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4),
                      status = c(1, 0, 2, 1, 1, 2, 0, 0, 2, 1, 2, 2))
  fit <- survfit(Surv(time, status, type='mstate') ~1, data = tdata)
  fortified <- fortify(fit)
  expect_equal(names(fortified),
               c("time", "n.risk", "n.event", "n.censor", "pstate",
                 "std.err", "upper", "lower", "event"))
  expect_equal(dim(fortified), c(12, 9))

  fortified <- fortify(fit, surv.connect = T)
  expect_equal(names(fortified),
               c("time", "n.risk", "n.event", "n.censor", "pstate",
                 "std.err", "upper", "lower", "event"))
  expect_equal(dim(fortified), c(15, 9))

  p <- ggplot2::autoplot(fit)
  expect_true(is(p, 'ggplot'))
})

test_that('fortify.survfit.cox works for lung', {
  skip_if_not_installed("survival")
  library(survival)
  d.coxph <- coxph(Surv(time, status) ~ sex, data = lung)
  fortified <- ggplot2::fortify(survfit(d.coxph))

  expect_true(is.data.frame(fortified))
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower')
  expect_equal(names(fortified), expected_names)

  p <- ggplot2::autoplot(survfit(d.coxph))
  expect_true(is(p, 'ggplot'))
})

test_that('fortify.aareg works for lung', {
  skip_if_not_installed("survival")
  library(survival)
  fit <- aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1)
  fortified <- fortify(fit)
  expected <- apply(fit$coefficient, 2, cumsum)
  rownames(expected) <- NULL
  expected <- as.data.frame(expected)

  # compare cumulated last row
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(dim(fortified), c(139, 5))
  expect_equal(colnames(fortified), c("time", "Intercept", "age", "sex", "ph.ecog"))
  expect_equal(as.numeric(fortified[-1][nrow(fortified), ]),
               as.numeric(as.data.frame(expected)[nrow(expected), ]))
  expect_equal(as.numeric(fortified$time), c(0, unique(fit$time)))

  p <- ggplot2::autoplot(fit)
  expect_true(is(p, 'ggplot'))

  fortified <- fortify(fit, surv.connect = FALSE)
  # compare cumulated last row
  expect_true(is.data.frame(fortified))
  expect_equal(dim(fortified), c(138, 5))
  expect_equal(colnames(fortified), c("time", "Intercept", "age", "sex", "ph.ecog"))
  expect_equal(as.numeric(fortified[nrow(fortified), ][-1]),
               as.numeric(as.data.frame(expected)[nrow(expected), ]))
  expect_equal(as.numeric(fortified$time), unique(fit$time))

  fortified <- fortify(fit, melt = TRUE)
  # compare cumulated last row
  expect_true(is.data.frame(fortified))
  expect_equal(dim(fortified), c(660, 7))
  expect_equal(colnames(fortified), c("time", "variable", "coef", "se", "value", "upper", "lower"))
  expect_equal(fortified$upper - fortified$value, fortified$se * 1.96)
  expect_equal(fortified$value - fortified$lower, fortified$se * 1.96)
  expect_equal(as.numeric(fortified[nrow(fortified), ][c(-1, -2)]),
               c(-4.50000000000, 4.5453070233, -3.2842219995, 5.624579766, -12.193023765192))
})

test_that('autoplot.aareg works for lung', {
  skip_if_not_installed("survival")
  library(survival)
  fit <- aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1)
  p <- autoplot(fit)

  # fail on travis
  skip_on_cran()
  skip_on_travis()

  expect_equal(length(p$layers), 2)
  expect_true(is(p$layers[[1]]$geom, 'GeomStep'))
  expect_true(is(p$layers[[2]]$geom, 'GeomConfint'))
  ld1 <- head(ggplot2:::layer_data(p, 1))
  expect_equal(ld1$x, c(0, 5, 11, 11, 11, 12))
  expect_equal(length(ld1$colour), 6)
  ld2 <- head(ggplot2:::layer_data(p, 2))
  expect_equal(ld2$x, c(0, 5, 11, 11, 11, 12))
  expect_equal(length(ld2$fill), 6)
  expect_equal(ld2$alpha, rep(0.3, 6))
})

test_that('fortify.survfit regular expression for renaming strata works with multiple stratification variables', {
  skip_if_not_installed("survival")
  library(survival)
  d.survfit <- survival::survfit(Surv(time, status) ~ sex + ph.ecog, data = lung)
  fortified <- ggplot2::fortify(d.survfit)
  expect_equal(is.data.frame(fortified), TRUE)
  expected_names <- c('time', 'n.risk', 'n.event', 'n.censor', 'surv',
                      'std.err', 'upper', 'lower', 'strata')
  expect_equal(names(fortified), expected_names)
})

context('test cluster')

test_that('fortify.kmeans works for iris', {
  skip_on_cran()
  skip_on_travis()
  df <- iris[-5]

  fortified <- ggplot2::fortify(stats::kmeans(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(stats::kmeans(df, 3), data = iris)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(iris), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  p <- ggplot2::autoplot(stats::kmeans(df, 3), data = df)
  expect_true(is(p, 'ggplot'))

  expect_that(ggplot2::autoplot(stats::kmeans(df, 3)), throws_error())
})

test_that('fortify.partition works for iris', {
  skip_on_cran()
  skip_on_travis()
  skip_if_not_installed("cluster")
  df <- iris[-5]
  # clara
  fortified <- ggplot2::fortify(cluster::clara(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::clara(df, 3), data = iris)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(iris), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  p <- autoplot(cluster::clara(df, 3), data = iris)
  expect_true(inherits(p, 'ggplot'))

  # fanny
  fortified <- ggplot2::fortify(cluster::fanny(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::fanny(df, 3), data = iris)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(iris), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  p <- autoplot(cluster::fanny(df, 3), data = iris)
  expect_true(inherits(p, 'ggplot'))

  # pam
  fortified <- ggplot2::fortify(cluster::pam(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::pam(df, 3), data = iris)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(iris), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  p <- autoplot(cluster::pam(df, 3), data = iris)
  expect_true(inherits(p, 'ggplot'))
})

test_that('fortify.partition works for USArrests', {
  skip_on_cran()
  skip_on_travis()
  skip_if_not_installed("cluster")
  df <- USArrests

  # clara
  fortified <- ggplot2::fortify(cluster::clara(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::clara(df, 3), data = USArrests)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  # fanny
  fortified <- ggplot2::fortify(cluster::fanny(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::fanny(df, 3), data = USArrests)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  # pam
  fortified <- ggplot2::fortify(cluster::pam(df, 3))
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))

  fortified <- ggplot2::fortify(cluster::pam(df, 3), data = USArrests)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(names(fortified), c(colnames(df), 'cluster'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(df))
})

test_that('fortify.silhouette works', {
  skip_on_cran()
  skip_on_travis()
  df <- iris[-5]
  mod <- stats::kmeans(df, 3)
  sil <- cluster::silhouette(mod$cluster, stats::dist(df))
  sil.df <- as.data.frame(unclass(sil))

  fortified <- ggplot2::fortify(sil)
  expect_equal(is.data.frame(fortified), TRUE)
  expect_equal(colnames(fortified), c('cluster', 'sil_width', 'id'))
  expect_equal(is.factor(fortified$cluster), TRUE)
  expect_equal(rownames(fortified), rownames(sil.df[order(sil.df$cluster), ]))

  p <- ggplot2::autoplot(sil)
  expect_true(is(p, 'ggplot'))

  class(sil) <- 'sil'
  expect_that(ggplot2::autoplot(sil), throws_error())
})

test_that('autoplot.kmeans works for iris', {
  skip_on_cran()
  skip_on_travis()

  obj <- stats::kmeans(iris[-5], 3)

  p <- ggplot2::autoplot(obj, data = iris)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 1)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))

  p <- ggplot2::autoplot(obj, data = iris, frame.type = 'norm')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomPoint' %in% class(p$layers[[1]]$geom))
  expect_true('GeomPolygon' %in% class(p$layers[[2]]$geom))

})

test_that('autoplot.silhouette works', {
  skip_on_cran()
  skip_on_travis()

  df <- iris[-5]
  model <- stats::kmeans(df, 3)
  obj <- cluster::silhouette(model$cluster, stats::dist(df))

  p <- ggplot2::autoplot(obj)
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomBar' %in% class(p$layers[[1]]$geom))

  p <- ggplot2::autoplot(obj, colour = 'black')
  expect_true(is(p, 'ggplot'))
  expect_equal(length(p$layers), 2)
  expect_true('GeomBar' %in% class(p$layers[[1]]$geom))
  expect_true('black' == p$layers[[2]]$aes_params$colour)

})

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Plotting Diagnostics for LM and GLM}
-->

```{r global_options, include=FALSE, eval=FALSE}
library(knitr)
opts_chunk$set(fig.width=8, fig.height=6, fig.path='figures/lm-', warning=FALSE)
```

This document explains plotting diagnostics for LM and GLM using `{ggplot2}` and `{ggfortify}`. 

# Plotting Diagnostics for Linear Models 

`{ggfortify}` let `{ggplot2}` know how to interpret `lm` objects. After loading `{ggfortify}`, you can use `ggplot2::autoplot` function for `lm` objects. 

```{r, message = FALSE, fig.width = 5, fig.height = 5, eval=FALSE}
library(ggfortify)
autoplot(lm(Petal.Width ~ Petal.Length, data = iris), label.size = 3)
```

You can select desired plot by `which` option as the same manner as standard `plot`. Also, 
`ncol` and `nrow` allows you to specify the number of subplot columns and rows.

```{r, message = FALSE, fig.width = 7, fig.height = 5, eval=FALSE}
par(mfrow = c(1, 2))
m <- lm(Petal.Width ~ Petal.Length, data = iris)

autoplot(m, which = 1:6, ncol = 3, label.size = 3)
```

# Plotting Diagnostics for Generalized Linear Models

It also suppotgs `glm` instance.

```{r, message = FALSE, fig.width = 5, fig.height = 6, eval=FALSE}
m <- glm(Murder ~ Assault + UrbanPop + Rape,
         family = gaussian, data = USArrests)

autoplot(m, which = 1:6, label.size = 3)
```

# Decorating Plots

Because `{ggplot2}` itself cannot handle different kinds of plots in a single instance, `{ggfortify}` handle them using its original class named `ggmultiplot`. You can use `+` operator to decorate `ggmultiplot`.

```{r, message = FALSE, fig.width = 5, fig.height = 5, eval=FALSE}
class(autoplot(m))
autoplot(m, label.size = 3) + theme_bw()
```

# Specifing Plotting Options

Some properties can be changed by passing corresponding keywords. For example, `colour` keyword is for data points, `smooth.colour` is for smoothing lines and `ad.colour` is for additional auxiliary lies. Also, `ncol` and `nrow` control facet layout. Use `help(autoplot.lm)` (or `help(autoplot.*)` for any other objects) to check available options.

```{r, message = FALSE, fig.width = 8, fig.height = 5, eval=FALSE}
autoplot(m, which = 1:6, colour = 'dodgerblue3',
         smooth.colour = 'black', smooth.linetype = 'dashed',
         ad.colour = 'blue',
         label.size = 3, label.n = 5, label.colour = 'blue',
         ncol = 3)
```

Also, you can use column names for these properties. Note that `lm` and `glm` instances doesn't retain original data, you should pass original data via `data` keyword to use column names not included in the model.

```{r, message = FALSE, fig.width = 8, fig.height = 5, eval=FALSE}
autoplot(lm(Petal.Width ~ Petal.Length, data = iris), data = iris,
         colour = 'Species', label.size = 3)
```


# Plotting ``{glmnet}``

``{ggfortify}`` supports ``{glmnet}`` package which supports Regularized Generalized Linear Models (Ridge, Lasso and Elastic-net).

```{r, message = FALSE, fig.width = 5, fig.height = 3, eval=FALSE}
library(glmnet)
data(QuickStartExample)
fit <- glmnet::glmnet(x, y)
autoplot(fit)

fit <- glmnet::cv.glmnet(x, y)
autoplot(fit, colour = 'blue')
```

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Plotting Time Series}
-->

```{r global_options, include=FALSE}
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/ts-', warning=FALSE)
```

This document explains time series related plotting using `ggplot2` and `{ggfortify}`. 

# Plotting ts objects

`{ggfortify}` let `{ggplot2}` know how to interpret `ts` objects. After loading `{ggfortify}`, you can use `ggplot2::autoplot` function for `ts` objects. 

```{r, message = FALSE}
library(ggfortify)
autoplot(AirPassengers)
```

To change line colour and line type, use `ts.colour` and `ts.linetype` options. Use `help(autoplot.ts)` (or `help(autoplot.*)` for any other objects) to check available options.

```{r}
autoplot(AirPassengers, ts.colour = 'red', ts.linetype = 'dashed')
```

Multivariate time series will be drawn with facets. 

```{r message = FALSE}
library(vars)
data(Canada)
autoplot(Canada)
```

Specify `facets = FALSE` to draw on single axes.

```{r}
autoplot(Canada, facets = FALSE)
```

Also, `autoplot` can handle other time-series-likes. Supported packages are:

* `zoo::zooreg`
* `xts::xts`
* `timeSeries::timSeries`
* `tseries::irts`

```{r, message = FALSE, eval = FALSE}
library(xts)
autoplot(as.xts(AirPassengers), ts.colour = 'green')

library(timeSeries)
autoplot(as.timeSeries(AirPassengers), ts.colour = ('dodgerblue3'))
```

# Specifying geometrics

You can change `{ggplot2}` geometrics specifying by its name. Geometrics currently supported are `line`, `bar`, `ribbon` and `point`. 

```{r, message = FALSE}
autoplot(AirPassengers, ts.geom = 'bar', fill = 'blue')
```

```{r, message = FALSE}
autoplot(AirPassengers, ts.geom = 'ribbon', fill = 'green')
```

```{r, message = FALSE}
autoplot(AirPassengers, ts.geom = 'point', shape = 3)
```

As described above, multivariate timeseries can be drawn in a single grid specifying ``facets = FALSE``. Time series are not stacked by default. Specifying ``stacked = TRUE`` allows stacking.

```{r, message = FALSE}
mts <- ts(data.frame(a = c(1, 2, 3, 4, 4, 3), b = c(3, 2, 3, 2, 2, 1)), start = 2010)
autoplot(mts, ts.geom = 'bar', facets = FALSE)
```

```{r, message = FALSE}
autoplot(mts, ts.geom = 'bar', facets = FALSE, stacked = TRUE)
```

```{r, message = FALSE}
autoplot(mts, ts.geom = 'ribbon', facets = FALSE)
```

```{r, message = FALSE}
autoplot(mts, ts.geom = 'ribbon', facets = FALSE, stacked = TRUE)
```

# Plotting with forecast package

`{ggfortify}` supports `forecast` object in the `{forecast}` package. 

```{r, message = FALSE}
library(forecast)
d.arima <- auto.arima(AirPassengers)
d.forecast <- forecast(d.arima, level = c(95), h = 50)
autoplot(d.forecast)
```

There are some options to change basic settings.

```{r}
autoplot(d.forecast, ts.colour = 'firebrick1', predict.colour = 'red',
         predict.linetype = 'dashed', conf.int = FALSE)
```

# Plotting with vars package

`ggfortify` supports `varpred` object in `vars` package. 

```{r}
library(vars)
d.vselect <- VARselect(Canada, lag.max = 5, type = 'const')$selection[1]
d.var <- VAR(Canada, p = d.vselect, type = 'const')
```

Available options are the same as `forecast`.

```{r}
autoplot(predict(d.var, n.ahead = 50), ts.colour = 'dodgerblue4',
         predict.colour = 'blue', predict.linetype = 'dashed')
```

# Plotting with changepoint package

`{ggfortify}` supports `cpt` object in `{changepoint}` package. 

```{r message = FALSE}
library(changepoint)
autoplot(cpt.meanvar(AirPassengers))
```

You can change some options for `cpt`.

```{r}
autoplot(cpt.meanvar(AirPassengers), cpt.colour = 'blue', cpt.linetype = 'solid')
```

# Plotting with strucchange package

`ggfortify` supports `breakpoints` object in `strucchange` package. Same plotting options as `changepoint` are available.


```{r message = FALSE}
library(strucchange)
autoplot(breakpoints(Nile ~ 1), ts.colour = 'blue', ts.linetype = 'dashed',
         cpt.colour = 'dodgerblue3', cpt.linetype = 'solid')
```

# Plotting with `{dlm}` package

`autoplot` can draw both original and fitted time series, because `dlm::dlmFilter` contains them. 

```{r, message=FALSE, warning=FALSE}
library(dlm)
form <- function(theta){
  dlmModPoly(order = 1, dV = exp(theta[1]), dW = exp(theta[2]))
}

model <- form(dlmMLE(Nile, parm = c(1, 1), form)$par)
filtered <- dlmFilter(Nile, model)

autoplot(filtered)
```

You can specify some options to change how plot looks, such as `ts.linetype` and `fitted.colour`. Use `help(autoplot.tsmodel)` (or `help(autoplot.*)` for any other objects) to check available options.

```{r}
autoplot(filtered, ts.linetype = 'dashed', fitted.colour = 'blue')
```

`{ggfortify}` can plot `dlm::dlmSmooth` instance which returns a `list` instance using using class inference. Note that only smoothed result is plotted because `dlm::dlmSmooth` doesn't contain original `ts`.

```{r}
smoothed <- dlmSmooth(filtered)
class(smoothed)
autoplot(smoothed)
```

To plot original `ts`, filtered result and smoothed result, you can use`autoplot` as below. When `autoplot` accepts `ggplot` instance via `p` option, continuous plot is drawn on the passed `ggplot`. 

```{r}
p <- autoplot(filtered)
autoplot(smoothed, ts.colour = 'blue', p = p)
```

# Plotting with `{KFAS}` package

You can use `autoplot` in almost the same manner as `{dlm}`. Note that `autoplot` draws smoothed result if it exists in `KFAS::KFS` instance, and `KFAS::KFS` contains smoothed result by default. 

```{r, message=FALSE, warning=FALSE}
library(KFAS)
model <- SSModel(
  Nile ~ SSMtrend(degree=1, Q=matrix(NA)), H=matrix(NA)
)
 
fit <- fitSSM(model=model, inits=c(log(var(Nile)),log(var(Nile))), method="BFGS")
smoothed <- KFS(fit$model)
autoplot(smoothed)
```

If you want filtered result, specify `smoothing='none'` when calling `KFS`. For details, see `help(KFS)`.

```{r}
filtered <- KFS(fit$model, filtering="mean", smoothing='none')
autoplot(filtered)
```

Also, `KFAS::signal` will retrieve specific state from `KFAS::KFS` instance. The result will be a `list` which contains the retrieved state as `ts` in `signal` attribute. `ggfortify` can autoplot it using class inference. 

```{r}
trend <- signal(smoothed, states="trend")
class(trend)
```

Because `signal` is a `ts` instance, you can use `autoplot` and `p` option as the same as `dlm::dlmSmooth` example.

```{r}
p <- autoplot(filtered)
autoplot(trend, ts.colour = 'blue', p = p)
```


# Plotting time series statistics

`{ggfortify}` supports following time series related statistics in `stats` package:

* `stl`, `decomposed.ts`
* `acf`, `pacf`, `ccf`
* `spec.ar`, `spec.pgram`
* `cpgram` (covered by `ggcpgram`) 

```{r, message = FALSE}
autoplot(stl(AirPassengers, s.window = 'periodic'), ts.colour = 'blue')
```

**NOTE** With `acf` and `spec.*`, specify `plot = FALSE` to suppress default plotting outputs.

```{r, fig.width = 4, fig.height = 2}
autoplot(acf(AirPassengers, plot = FALSE))
```

You can pass some options when plotting `acf` via `autoplot`. Built-in `acf` calcurates the confidence interval at plotting time and doesn't hold the result, equivalent options can be passed to `autoplot`. Following example shows to change the value of confidence interval and method (use `ma` assuming the input follows MA model).

```{r, fig.width = 4, fig.height = 2}
autoplot(acf(AirPassengers, plot = FALSE), conf.int.fill = '#0000FF', conf.int.value = 0.8, conf.int.type = 'ma')
```


```{r, fig.width = 4, fig.height = 2}
autoplot(spec.ar(AirPassengers, plot = FALSE))
```

`ggcpgram` should output the cumulative periodogram as the same as `cpgram`. Because `cpgram` doesn't have return value, we cannot use `autoplot(cpgram(...))`.

```{r, message = FALSE, fig.width=3, fig.height=3}
ggcpgram(arima.sim(list(ar = c(0.7, -0.5)), n = 50))
```

`ggtsdiag` should output the similar diagram as `tsdiag`.

```{r, message = FALSE, fig.width = 5, fig.height = 5}
library(forecast)
ggtsdiag(auto.arima(AirPassengers))
```

`ggfreqplot` is a genelarized `month.plot`. You can pass `freq` if you want, otherwise time-series's frequency will be used.

```{r, message = FALSE, fig.width = 4, fig.height = 4}
ggfreqplot(AirPassengers)
ggfreqplot(AirPassengers, freq = 4)
```

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Introduction to ggfortify package}
-->

```{r global_options, include=FALSE}
library(knitr)
opts_chunk$set(fig.width=6, fig.height=4, fig.path='figures/basics-', warning=FALSE)
```

This document explains concepts and basics of `{ggfortify}`. `{ggfortify}` helps plotting some popular R packages with `{ggplot2}` in a unified way. See [github](https://github.com/sinhrks/ggfortify) to check the list of supported packages / classes.

# Loading Package

```{r, message = FALSE}
library(ggfortify)
```

# Using `ggplot2::autoplot`

This is the easiest way provided by `{ggfortify}`. Calling `autoplot` with supported instance should output appropriate plot based on its type, as standard `plot` function does.

```{r}
autoplot(AirPassengers)
```

You can specify some options to control how plot looks. As is often the case with statistic classes, there are some objects to be drawn, such as actual value, predicted value and confidence interval, etc. Thus, each plotting option has a dot-separated format like `<target name>.<ggplot option name>` such as `ts.colour` and `conf.int.linetype`.

These options can be used for the similar type of instances commonly. For example, `ts.colour` works for all time-series-likes (there is no separate options like `xts.colour` and `timeSeries.colour`, etc). To check available options, use `help(autoplot.ts)` or `help(autoplot.*)` for any other objects.

```{r}
autoplot(AirPassengers, ts.colour = 'blue')
```

# Decorating Plots

`autoplot` returns `ggplot` instance. Thus, you can decorate as the same as in ``{ggplot2}``. To make `autoplot` API simple, `{ggfortify}` only offers some limited options which must be done during `ggplot` instance creation. Additional decoration should be done against the returned ``ggplot`` instance.

```{r}
p <- autoplot(AirPassengers)
class(p)
# plot as it is
p

# add title and labels
p + ggtitle('AirPassengers') + xlab('Year') + ylab('Passengers')
```

```{r}
# these common options are supported as keywords 
autoplot(AirPassengers, title = 'AirPassengers', xlab = 'Year', ylab = 'Passengers')
```

```{r}
set.seed(1)
p <- autoplot(kmeans(iris[-5], 3), data = iris)
# plot as it is
p

# change colour mapping
p + scale_colour_brewer()
```

# Want Different Plots?

Internally, `autoplot` calls a generic function named `ggplot2::fortify` to convert the passed instance to `data.frame`. `{ggfortify}` defines `fortify` function for all supported classes. If you want a different type of plot, you can use `fortify` to get `data.frame`, then call `ggplot` in a normal way.

Following example shows a bar plot counting records per k-means clusters.

```{r}
df <- fortify(kmeans(iris[-5], 3), data = iris)
head(df)

ggplot(df, aes(x= cluster, fill = cluster)) + geom_bar()
```

# Handling Subplots

``autoplot`` may draw subplots in some classes. For exmple, ``autoplot.lm`` draws diagnostics plots as below.

```{r, message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE}
res <- lm(Volume ~ Girth, data = trees) 
mp <- autoplot(res, ncol = 4)
mp
```

Because `{ggplot2}` itself cannot handle different kinds of plots in a single instance, `{ggfortify}` handle them using its original class named `ggmultiplot`. You can use `+` operator to decorate `ggmultiplot`.

```{r, message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE}
class(mp)
mp + theme_bw()
```

Also, ``+`` operator appends plots if ``ggplot`` or ``ggmultiplot`` instance is given as right-hand-side. Following example attaches 2 scatter plots after diagnostics.

```{r, message = FALSE, fig.width = 9, fig.height = 5, eval=FALSE}
mp +
  (ggplot(trees, aes(Girth, Volume)) + geom_point()) +
  (ggplot(trees, aes(Girth, Height)) + geom_point())
```

You can extract subset using ``[`` and ``[[`` operators.

```{r, message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE}
mp[2:3]
```

```{r, message = FALSE, fig.width = 2.25, fig.height = 2.5, eval=FALSE}
mp[[1]]
```

You can extract arbitrary subset, modify, and draw.

```{r, message = FALSE, fig.width = 9, fig.height = 2.5, eval=FALSE}
mp[2:3] <- mp[2:3] + theme_bw()
mp
```

## Draw Multiple Instances as Subplots

Assuming you want to plot multiple instances at once. ``{ggfortify}`` can accept ``list`` of supported instances and can output subplots.

Following example shows 3 ``kmeans`` plots changing number of clusters.

```{r, message = FALSE, fig.width = 9, fig.height = 2.5}
res <- lapply(c(3, 4, 5), function(x) kmeans(iris[-5], x))
autoplot(res, data = iris[-5], ncol = 3)
```

Next example shows multiple survival curves as subplots.

```{r, message = FALSE, fig.width = 8, fig.height = 2.5}
library(survival)
res <- list(a = survfit(Surv(time, status) ~ 1, data = lung),
            b = survfit(Surv(time, status) ~ sex, data = lung))
autoplot(res)
```

You can pass a list contains different classes to ``autoplot``.

```{r, message = FALSE, fig.width = 8, fig.height = 2.5}
library(vars)
data(Canada)
res <- list(a = Canada, b = AirPassengers)
autoplot(res)
```

## Create your own subplots

You can instanciate ``ggmultiplot`` from a ``list`` of ``ggplot`` instances.

```{r, message = FALSE, fig.width = 4.5, fig.height = 2.5}
p1 <- ggplot(iris, aes(Petal.Width, Petal.Length)) + geom_point()
p2 <- ggplot(iris, aes(Petal.Width, Petal.Length)) + geom_point()
new('ggmultiplot', plots = list(p1, p2))
```

Layout can be specified via ``nrow`` and ``ncol`` keywords.

```{r, message = FALSE, fig.width = 2.25, fig.height = 5}
new('ggmultiplot', plots = list(p1, p2), ncol = 1)
```

<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Plotting PCA, clustering, LFDA and MDS results}
-->

```{r global_options, include=FALSE}
library(knitr)
opts_chunk$set(fig.width=6, fig.height=3, fig.path='figures/pca-', warning=FALSE)
```

This document explains PCA, clustering, LFDA and MDS related plotting using `{ggplot2}` and `{ggfortify}`. 

# Plotting PCA (Principal Component Analysis)

`{ggfortify}` let `{ggplot2}` know how to interpret PCA objects. After loading `{ggfortify}`, you can use `ggplot2::autoplot` function for `stats::prcomp` and `stats::princomp` objects. 

```{r, message = FALSE}
library(ggfortify)
df <- iris[1:4]
pca_res <- prcomp(df, scale. = TRUE)

autoplot(pca_res)
```

PCA result should only contains numeric values. If you want to colorize by non-numeric values which original data has, pass original data using `data` keyword and then specify column name by `colour` keyword. Use `help(autoplot.prcomp)` (or `help(autoplot.*)` for any other objects) to check available options.

```{r, message = FALSE}
autoplot(pca_res, data = iris, colour = 'Species')
```

Passing `label = TRUE` draws each data label using `rownames`

```{r, message = FALSE}
autoplot(pca_res, data = iris, colour = 'Species', label = TRUE, label.size = 3)
```

Passing `shape = FALSE` makes plot without points. In this case, `label` is turned on unless otherwise specified.

```{r, message = FALSE}
autoplot(pca_res, data = iris, colour = 'Species', shape = FALSE, label.size = 3)
```

Passing `loadings = TRUE` draws eigenvectors.

```{r, message = FALSE}
autoplot(pca_res, data = iris, colour = 'Species', loadings = TRUE)
```

You can attach eigenvector labels and change some options.

```{r, message = FALSE}
autoplot(pca_res, data = iris, colour = 'Species',
         loadings = TRUE, loadings.colour = 'blue',
         loadings.label = TRUE, loadings.label.size = 3)
```

By default, each component are scaled as the same as standard ``biplot``. You can disable the scaling by specifying ``scale = 0``

```{r, message = FALSE}
autoplot(pca_res, scale = 0)
```

# Plotting Factor Analysis

`{ggfortify}` supports `stats::factanal` object as the same manner as PCAs. Available opitons are the same as PCAs.

**Important** You must specify `scores` option when calling `factanal` to calcurate sores (default `scores = NULL`). Otherwise, plotting will fail.

```{r, message = FALSE}
d.factanal <- factanal(state.x77, factors = 3, scores = 'regression')
autoplot(d.factanal, data = state.x77, colour = 'Income')
autoplot(d.factanal, label = TRUE, label.size = 3,
         loadings = TRUE, loadings.label = TRUE, loadings.label.size  = 3)
```

# Plotting K-means

`{ggfortify}` supports `stats::kmeans` class. You must explicitly pass original data to `autoplot` function via `data` keyword. Because `kmeans` object doesn't store original data. The result will be automatically colorized by categorized cluster.

```{r, message = FALSE}
set.seed(1)
autoplot(kmeans(USArrests, 3), data = USArrests)

autoplot(kmeans(USArrests, 3), data = USArrests, label = TRUE, label.size = 3)
```

# Plotting cluster package

`{ggfortify}` supports `cluster::clara`, `cluster::fanny`, `cluster::pam` as well as `cluster::silhouette` classes. 
Because these instances should contains original data in its property, there is no need to pass original data explicitly.

```{r, message = FALSE}
library(cluster)
autoplot(clara(iris[-5], 3))
```

Specifying `frame = TRUE` in `autoplot` for `stats::kmeans` and `cluster::*` draws convex for each cluster.

```{r}
autoplot(fanny(iris[-5], 3), frame = TRUE)
```

If you want probability ellipse, `{ggplot2}` 1.0.0 or later is required. Specify whatever supported in [`ggplot2::stat_ellipse`](https://ggplot2.tidyverse.org/reference/stat_ellipse.html)'s `type` keyword via `frame.type` option.

```{r}
autoplot(pam(iris[-5], 3), frame = TRUE, frame.type = 'norm')
```

If you want a Silhouette plot, pass a Silhouette object to `autoplot` function. 

```{r}
autoplot(silhouette(pam(iris[-5], 3L)))
```

For more information on Silhouette plots and how they can be used, see 
[base R example](https://www.stat.berkeley.edu/~spector/s133/Clus.html), 
[scikit-learn example](https://scikit-learn.org/stable/auto_examples/cluster/plot_kmeans_silhouette_analysis.html) and 
[original paper](https://www.sciencedirect.com/science/article/pii/0377042787901257). 

# Plotting Local Fisher Discriminant Analysis with ``{lfda}`` package

``{lfda}`` package supports a set of Local Fisher Discriminant Analysis methods. You can use ``autoplot`` to plot the analysis result as the same manner as PCA.

```{r, message = FALSE}
library(lfda)

# Local Fisher Discriminant Analysis (LFDA)
model <- lfda(iris[-5], iris[, 5], r = 3, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')
```

```{r, message = FALSE}
# Semi-supervised Local Fisher Discriminant Analysis (SELF)
model <- self(iris[-5], iris[, 5], beta = 0.1, r = 3, metric="plain")
autoplot(model, data = iris, frame = TRUE, frame.colour = 'Species')
```


# Plotting Multidimensional Scaling

## Before Plotting

Even though MDS functions returns `matrix` or `list` (not specific class), `{ggfortify}` can infer background class from `list` attribute and perform `autoplot`.

**NOTE** Inference from `matrix` is not supported.

**NOTE** `{ggfortify}` can plot `stats::dist` instance as heatmap.

```{r, message = FALSE}
autoplot(eurodist)
```

## Plotting Classical (Metric) Multidimensional Scaling

`stats::cmdscale` performs Classical MDS and returns point coodinates as `matrix`, thus you can not use `autoplot` in this case. However, either `eig = TRUE`, `add = True` or `x.ret = True` is specified, `stats::cmdscale` return `list` instead of `matrix`. In these cases, `{ggfortify}` can infer how to plot it via `autoplot`. Refer to `help(cmdscale)` to check what these options are.

```{r}
autoplot(cmdscale(eurodist, eig = TRUE))
```

Specify `label = TRUE` to plot labels.

```{r}
autoplot(cmdscale(eurodist, eig = TRUE), label = TRUE, label.size = 3)
```

## Plotting Non-metric Multidimensional Scaling

`MASS::isoMDS` and `MASS::sammon` perform Non-metric MDS and return `list` which contains point coordinates. Thus, `autoplot` can be used.

**NOTE** On background, `autoplot.matrix` is called to plot MDS. See `help(autoplot.matrix)` to check available options.

```{r, message = FALSE}
library(MASS)
autoplot(isoMDS(eurodist), colour = 'orange', size = 4, shape = 3)
```

Passing `shape = FALSE` makes plot without points. In this case, `label` is turned on unless otherwise specified.

```{r, message = FALSE}
autoplot(sammon(eurodist), shape = FALSE, label.colour = 'blue', label.size = 3)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{get.layout}
\alias{get.layout}
\title{Calcurate layout matrix for \code{ggmultiplot}}
\usage{
get.layout(nplots, ncol, nrow)
}
\arguments{
\item{nplots}{Number of plots}

\item{ncol}{Number of grid columns}

\item{nrow}{Number of grid rows}
}
\value{
matrix
}
\description{
Calcurate layout matrix for \code{ggmultiplot}
}
\examples{
ggfortify:::get.layout(3, 2, 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_maps.R
\name{autoplot.map}
\alias{autoplot.map}
\title{Autoplot \code{maps::map}}
\usage{
\method{autoplot}{map}(
  object,
  p = NULL,
  geom = "path",
  group = "group",
  colour = "black",
  size = NULL,
  linetype = NULL,
  alpha = NULL,
  fill = NULL,
  shape = NULL,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = "",
  ylab = "",
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{maps::map} instance}

\item{p}{\code{ggplot2::ggplot} instance}

\item{geom}{geometric string for map. 'path', 'point' or 'polygon'}

\item{group}{key for grouping geoms}

\item{colour}{line colour}

\item{size}{point size}

\item{linetype}{line type}

\item{alpha}{alpha}

\item{fill}{fill colour}

\item{shape}{point shape}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{maps::map}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_cluster.R
\name{fortify.silhouette}
\alias{fortify.silhouette}
\title{Convert \code{cluster::silhouette} to \code{data.frame}}
\usage{
\method{fortify}{silhouette}(model, data = NULL, ...)
}
\arguments{
\item{model}{Silhouette instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{cluster::silhouette} to \code{data.frame}
}
\examples{
fortify(cluster::silhouette(cluster::pam(iris[-5], 3)))
fortify(cluster::silhouette(cluster::clara(iris[-5], 3)))
fortify(cluster::silhouette(cluster::fanny(iris[-5], 3)))

mod = stats::kmeans(iris[-5], 3)
fortify(cluster::silhouette(mod$cluster, stats::dist(iris[-5])))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_basis.R
\name{as_tibble.basis}
\alias{as_tibble.basis}
\title{Convert a spline basis to a tibble}
\usage{
\method{as_tibble}{basis}(x, ...)
}
\arguments{
\item{x}{object of class "basis"}

\item{...}{Ignored.}
}
\value{
A tibble constructed from the underlying matrix of the
    basis object. Each column will possess all the attributes from
    the source object, except that the "class" attribute will be
    renamed to "basis.class" to avoid interfering with dplyr
    operations.
}
\description{
Convert a spline basis to a tibble
}
\details{
This function is needed because the default method for
    converting a matrix object with an additional class attribute
    to a tibble causes issues because each column of the resulting
    tibble has the attributes, including the matrix class, copied
    from the source. Having matrices as columns in a tibble causes
    dplyr to throw errors, so a special method is needed to avoid
    copying the class attribute.
}
\examples{
library(splines)
library(tibble)
x <- seq(0, 1, by=0.001)
spl <- bs(x, df=6)
as_tibble(spl)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_raster.R
\name{fortify.RasterCommon}
\alias{fortify.RasterCommon}
\title{Convert \code{raster} to \code{data.frame}}
\usage{
\method{fortify}{RasterCommon}(model, data = NULL, maxpixels = 1e+05, rename = TRUE, ...)
}
\arguments{
\item{model}{\code{raster} instances}

\item{data}{original dataset, if needed}

\item{maxpixels}{number of pixels for resampling}

\item{rename}{logical flag indicating whether to rename coordinates to long and lat}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{raster} to \code{data.frame}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{autoplot.pca_common}
\alias{autoplot.pca_common}
\alias{autoplot.prcomp}
\alias{autoplot.princomp}
\alias{autoplot.factanal}
\title{Autoplot PCA-likes}
\usage{
\method{autoplot}{pca_common}(
  object,
  data = NULL,
  scale = 1,
  x = 1,
  y = 2,
  variance_percentage = TRUE,
  ...
)
}
\arguments{
\item{object}{PCA-like instance}

\item{data}{Joined to fitting result if provided.}

\item{scale}{scaling parameter, disabled by 0}

\item{x}{principal component number used in x axis}

\item{y}{principal component number used in y axis}

\item{variance_percentage}{show the variance explained by the principal component?}

\item{...}{other arguments passed to [ggbiplot()]}
}
\description{
Autoplot PCA-likes
}
\examples{
autoplot(stats::prcomp(iris[-5]))
autoplot(stats::prcomp(iris[-5]), data = iris)
autoplot(stats::prcomp(iris[-5]), data = iris, colour = 'Species')
autoplot(stats::prcomp(iris[-5]), label = TRUE, loadings = TRUE, loadings.label = TRUE)
autoplot(stats::prcomp(iris[-5]), frame = TRUE)
autoplot(stats::prcomp(iris[-5]), data = iris, frame = TRUE,
         frame.colour = 'Species')
autoplot(stats::prcomp(iris[-5]), data = iris, frame = TRUE,
         frame.type = 't', frame.colour = 'Species')

autoplot(stats::princomp(iris[-5]))
autoplot(stats::princomp(iris[-5]), data = iris)
autoplot(stats::princomp(iris[-5]), data = iris, colour = 'Species')
autoplot(stats::princomp(iris[-5]), label = TRUE, loadings = TRUE, loadings.label = TRUE)

#Plot PC 2 and 3
autoplot(stats::princomp(iris[-5]), x = 2, y = 3)

#Don't show the variance explained
autoplot(stats::princomp(iris[-5]), variance_percentage = FALSE)

d.factanal <- stats::factanal(state.x77, factors = 3, scores = 'regression')
autoplot(d.factanal)
autoplot(d.factanal, data = state.x77, colour = 'Income')
autoplot(d.factanal, label = TRUE, loadings = TRUE, loadings.label = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{support_autoplot}
\alias{support_autoplot}
\title{Check if passed object is supported by \code{ggplot2::autoplot}}
\usage{
support_autoplot(obj)
}
\arguments{
\item{obj}{object}
}
\value{
logical
}
\description{
Check if passed object is supported by \code{ggplot2::autoplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_forecast.R
\name{autoplot.forecast}
\alias{autoplot.forecast}
\title{Autoplot \code{forecast::forecast}}
\usage{
\method{autoplot}{forecast}(
  object,
  is.date = NULL,
  ts.connect = TRUE,
  predict.geom = "line",
  predict.colour = "#0000FF",
  predict.size = NULL,
  predict.linetype = NULL,
  predict.alpha = NULL,
  predict.fill = NULL,
  predict.shape = NULL,
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "none",
  conf.int.fill = "#000000",
  conf.int.alpha = 0.3,
  ...
)
}
\arguments{
\item{object}{\code{forecast::forecast} instance}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{predict.geom}{geometric string for predicted time-series}

\item{predict.colour}{line colour for predicted time-series}

\item{predict.size}{point size for predicted time-series}

\item{predict.linetype}{line type for predicted time-series}

\item{predict.alpha}{alpha for predicted time-series}

\item{predict.fill}{fill colour for predicted time-series}

\item{predict.shape}{point shape for predicted time-series}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{...}{other arguments passed to \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot \code{forecast::forecast}
}
\examples{
d.arima <- forecast::auto.arima(AirPassengers)
autoplot(forecast::forecast(d.arima, h = 10))
autoplot(forecast::forecast(d.arima, level = c(85), h = 10))
autoplot(forecast::forecast(d.arima, h = 5), conf.int = FALSE, is.date = FALSE)
autoplot(forecast::forecast(stats::HoltWinters(UKgas), h = 10))
\dontrun{
autoplot(forecast::forecast(forecast::ets(UKgas), h = 5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_performance.R
\name{autoplot.performance}
\alias{autoplot.performance}
\title{Autoplot \code{ROCR::performance}}
\usage{
\method{autoplot}{performance}(object, p = NULL, bins = 5, ...)
}
\arguments{
\item{object}{\code{ROCR::performance} instance}

\item{p}{\code{ggplot2::ggplot} instances}

\item{bins}{If \code{object} represents a measure whose value is just a
scalar (e.g. \code{performance(predObj, 'auc')}), a histogram will be
plotted of this scalar's values for different runs. \code{bins}
is the number of bins for this histogram.}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{ROCR::performance}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{+,ggmultiplot,ANY-method}
\alias{+,ggmultiplot,ANY-method}
\title{Generic add operator for \code{ggmultiplot}}
\usage{
\S4method{+}{ggmultiplot,ANY}(e1, e2)
}
\arguments{
\item{e1}{first argument}

\item{e2}{second argument}
}
\value{
\code{ggmultiplot}
}
\description{
Generic add operator for \code{ggmultiplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_surv.R
\name{fortify.survfit}
\alias{fortify.survfit}
\alias{fortify.survfit.cox}
\title{Convert \code{survival::survfit} to \code{data.frame}}
\usage{
\method{fortify}{survfit}(model, data = NULL, surv.connect = FALSE, fun = NULL, ...)
}
\arguments{
\item{model}{\code{survival::survfit} instance}

\item{data}{original dataset, if needed}

\item{surv.connect}{logical frag indicates whether connects survival curve to the origin}

\item{fun}{an arbitrary function defining a transformation of the survival curve}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{survival::survfit} to \code{data.frame}
}
\examples{
library(survival)
fortify(survfit(Surv(time, status) ~ sex, data = lung))
fortify(survfit(Surv(time, status) ~ 1, data = lung))
fortify(survfit(coxph(Surv(time, status) ~ sex, data = lung)))
fortify(survfit(coxph(Surv(time, status) ~ 1, data = lung)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_spatial.R
\name{fortify.SpatialCommon}
\alias{fortify.SpatialCommon}
\title{Convert \code{sp} instances to \code{data.frame}.}
\usage{
\method{fortify}{SpatialCommon}(model, data = NULL, rename = TRUE, ...)
}
\arguments{
\item{model}{\code{sp} instances}

\item{data}{original dataset, if needed}

\item{rename}{logical flag indicating whether to rename coordinates to long and lat}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{sp} instances to \code{data.frame}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{get_geom_function}
\alias{get_geom_function}
\title{Factory function to control \code{ggplot2::geom_xxx} functions}
\usage{
get_geom_function(geom, allowed = c("line", "bar", "point"))
}
\arguments{
\item{geom}{string representation of \code{ggplot2::geom_xxx} function}

\item{allowed}{character vector contains allowed values}
}
\value{
function
}
\description{
Factory function to control \code{ggplot2::geom_xxx} functions
}
\examples{
ggfortify:::get_geom_function('point')
ggfortify:::get_geom_function('line', allowed = c('line'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_surv.R
\name{fortify.aareg}
\alias{fortify.aareg}
\title{Convert \code{survival::aareg} to \code{data.frame}}
\usage{
\method{fortify}{aareg}(
  model,
  data = NULL,
  maxtime = NULL,
  surv.connect = TRUE,
  melt = FALSE,
  ...
)
}
\arguments{
\item{model}{\code{survival::aareg} instance}

\item{data}{original dataset, if needed}

\item{maxtime}{truncate the input to the model at time "maxtime"}

\item{surv.connect}{logical frag indicates whether connects survival curve to the origin}

\item{melt}{Logical flag indicating whether to melt each timeseries as variable}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{survival::aareg} to \code{data.frame}
}
\examples{
library(survival)
fortify(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1))
fortify(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1), melt = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom.R
\name{geom_confint}
\alias{geom_confint}
\title{Connect observations by stairs.}
\usage{
geom_confint(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  na.rm = FALSE,
  ...
)
}
\arguments{
\item{mapping}{the aesthetic mapping}

\item{data}{a layer specific dataset}

\item{stat}{the statistical transformation to use on the data for this layer}

\item{position}{the position adjustment to use for overlapping points on this layer}

\item{na.rm}{logical frag whether silently remove missing values}

\item{...}{other arguments passed to methods}
}
\description{
Connect observations by stairs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_glmnet.R
\name{fortify.glmnet}
\alias{fortify.glmnet}
\title{Convert \code{glmnet::glmnet} to \code{data.frame}}
\usage{
\method{fortify}{glmnet}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{glmnet::glmnet} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{glmnet::glmnet} to \code{data.frame}
}
\examples{
fortify(glmnet::glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_list.R
\name{check_names}
\alias{check_names}
\title{Check data names are equal with expected}
\usage{
check_names(data, expected)
}
\arguments{
\item{data}{\code{list} instance to be checked}

\item{expected}{expected character vector}
}
\value{
logical
}
\description{
Check data names are equal with expected
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{autoplot.ggplot}
\alias{autoplot.ggplot}
\title{Autoplot \code{ggplot} instances.
It returns the passed instance as it is.}
\usage{
\method{autoplot}{ggplot}(object, ...)
}
\arguments{
\item{object}{ggplot instance}

\item{...}{Not used.}
}
\value{
ggplot
}
\description{
Autoplot \code{ggplot} instances.
It returns the passed instance as it is.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_ts.R
\name{autoplot.ts}
\alias{autoplot.ts}
\alias{autoplot.xts}
\alias{autoplot.timeSeries}
\alias{autoplot.irts}
\alias{autoplot.stl}
\alias{autoplot.decomposed.ts}
\title{Autoplot time-series-like}
\usage{
\method{autoplot}{ts}(
  object,
  columns = NULL,
  group = NULL,
  is.date = NULL,
  index.name = "Index",
  p = NULL,
  ts.scale = FALSE,
  stacked = FALSE,
  facets = TRUE,
  nrow = NULL,
  ncol = 1,
  scales = "free_y",
  ts.geom = "line",
  ts.colour = NULL,
  ts.size = NULL,
  ts.linetype = NULL,
  ts.alpha = NULL,
  ts.fill = NULL,
  ts.shape = NULL,
  geom = ts.geom,
  colour = ts.colour,
  size = ts.size,
  linetype = ts.linetype,
  alpha = ts.alpha,
  fill = ts.fill,
  shape = ts.shape,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = "",
  ylab = "",
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{time-series-like instance}

\item{columns}{Character vector specifies target column name(s)}

\item{group}{Character vector specifies grouping}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not
If not provided, regard the input as date when the frequency is 4 or 12}

\item{index.name}{Specify column name for time series index when passing \code{data.frame} via data.}

\item{p}{\code{ggplot2::ggplot} instance}

\item{ts.scale}{Logical flag indicating whether to perform scaling each timeseries}

\item{stacked}{Logical flag indicating whether to stack multivariate timeseries}

\item{facets}{Logical value to specify use facets}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{ts.geom}{geometric string for time-series. 'line', 'bar', 'ribbon', or 'point'}

\item{ts.colour}{line colour for time-series}

\item{ts.size}{point size for time-series}

\item{ts.linetype}{line type for time-series}

\item{ts.alpha}{alpha for time-series}

\item{ts.fill}{fill colour for time-series}

\item{ts.shape}{point shape for time-series}

\item{geom}{same as ts.geom}

\item{colour}{same as ts.colour}

\item{size}{same as ts.size}

\item{linetype}{same as ts.linetype}

\item{alpha}{same as ts.alpha}

\item{fill}{same as ts.fill}

\item{shape}{same as ts.shape}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot time-series-like
}
\examples{
\dontrun{
data(Canada, package = 'vars')
autoplot(AirPassengers)
autoplot(UKgas, ts.geom = 'bar')
autoplot(Canada)
autoplot(Canada, facets = FALSE)

library(zoo)
autoplot(xts::as.xts(AirPassengers))
autoplot(timeSeries::as.timeSeries(AirPassengers))
its <- tseries::irts(cumsum(rexp(10, rate = 0.1)), matrix(rnorm(20), ncol=2))
autoplot(its)

autoplot(stats::stl(UKgas, s.window = 'periodic'))
autoplot(stats::decompose(UKgas))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_list.R
\name{infer}
\alias{infer}
\title{Infer class name}
\usage{
infer(data)
}
\arguments{
\item{data}{list instance}
}
\value{
character
}
\description{
Infer class name
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{grid.draw.ggmultiplot}
\alias{grid.draw.ggmultiplot}
\title{The implemented grid.draw method for ggmultiplot, in order to work
with ggsave() properly}
\usage{
\method{grid.draw}{ggmultiplot}(plot)
}
\arguments{
\item{plot}{\code{ggmultiplot}}
}
\description{
The implemented grid.draw method for ggmultiplot, in order to work
with ggsave() properly
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{rbind_ts}
\alias{rbind_ts}
\title{Rbind original and predicted time-series-like instances as fortified \code{data.frame}}
\usage{
rbind_ts(
  data,
  original,
  ts.connect = TRUE,
  index.name = "Index",
  data.name = "Data"
)
}
\arguments{
\item{data}{Predicted/forecasted \code{ts} instance}

\item{original}{Original \code{ts} instance}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{index.name}{Specify column name for time series index}

\item{data.name}{Specify column name for univariate time series data. Ignored in multivariate time series.}
}
\value{
data.frame
}
\description{
Rbind original and predicted time-series-like instances as fortified \code{data.frame}
}
\examples{
predicted <- predict(stats::HoltWinters(UKgas), n.ahead = 5, prediction.interval = TRUE)
rbind_ts(predicted, UKgas, ts.connect = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_forecast.R
\name{fortify.ets}
\alias{fortify.ets}
\title{Convert \code{forecast::bats} and \code{forecast::ets} to \code{data.frame}}
\usage{
\method{fortify}{ets}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{forecast::bats} or \code{forecast::ets} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{forecast::bats} and \code{forecast::ets} to \code{data.frame}
}
\examples{
\dontrun{
fortify(forecast::bats(UKgas))
fortify(forecast::ets(UKgas))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_ts.R
\name{autoplot.tsmodel}
\alias{autoplot.tsmodel}
\alias{autoplot.ar}
\alias{autoplot.fracdiff}
\alias{autoplot.nnetar}
\alias{autoplot.HoltWinters}
\alias{autoplot.fGARCH}
\title{Autoplot time series models (like AR, ARIMA)}
\usage{
\method{autoplot}{tsmodel}(
  object,
  data = NULL,
  predict = NULL,
  is.date = NULL,
  ts.connect = TRUE,
  fitted.geom = "line",
  fitted.colour = "#FF0000",
  fitted.size = NULL,
  fitted.linetype = NULL,
  fitted.alpha = NULL,
  fitted.fill = NULL,
  fitted.shape = NULL,
  predict.geom = "line",
  predict.colour = "#0000FF",
  predict.size = NULL,
  predict.linetype = NULL,
  predict.alpha = NULL,
  predict.fill = NULL,
  predict.shape = NULL,
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "none",
  conf.int.fill = "#000000",
  conf.int.alpha = 0.3,
  ...
)
}
\arguments{
\item{object}{Time series model instance}

\item{data}{original dataset, needed for \code{stats::ar}, \code{stats::Arima}}

\item{predict}{Predicted \code{stats::ts}
If not provided, try to retrieve from current environment using variable name.}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{fitted.geom}{geometric string for fitted time-series}

\item{fitted.colour}{line colour for fitted time-series}

\item{fitted.size}{point size for fitted time-series}

\item{fitted.linetype}{line type for fitted time-series}

\item{fitted.alpha}{alpha for fitted time-series}

\item{fitted.fill}{fill colour for fitted time-series}

\item{fitted.shape}{point shape for fitted time-series}

\item{predict.geom}{geometric string for predicted time-series}

\item{predict.colour}{line colour for predicted time-series}

\item{predict.size}{point size for predicted time-series}

\item{predict.linetype}{line type for predicted time-series}

\item{predict.alpha}{alpha for predicted time-series}

\item{predict.fill}{fill colour for predicted time-series}

\item{predict.shape}{point shape for predicted time-series}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{...}{Keywords passed to \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot time series models (like AR, ARIMA)
}
\examples{
d.ar <- stats::ar(AirPassengers)
autoplot(d.ar)
autoplot(d.ar, predict = predict(d.ar, n.ahead = 5))
autoplot(stats::arima(UKgas), data = UKgas)
autoplot(forecast::arfima(AirPassengers))
autoplot(forecast::nnetar(UKgas), is.date = FALSE)

d.holt <- stats::HoltWinters(USAccDeaths)
autoplot(d.holt)
autoplot(d.holt, predict = predict(d.holt, n.ahead = 5))
autoplot(d.holt, predict = predict(d.holt, n.ahead = 5, prediction.interval = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_glmnet.R
\name{autoplot.glmnet}
\alias{autoplot.glmnet}
\title{Autoplot \code{glmnet::glmnet}}
\usage{
\method{autoplot}{glmnet}(
  object,
  xvar = c("norm", "lambda", "dev"),
  label.n = 7,
  label = TRUE,
  label.label = "Df",
  label.colour = NULL,
  label.alpha = NULL,
  label.size = NULL,
  label.angle = NULL,
  label.family = NULL,
  label.fontface = NULL,
  label.lineheight = NULL,
  label.hjust = NULL,
  label.vjust = NULL,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = "Coefficients",
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{glmnet::glmnet} instance}

\item{xvar}{values to be dranw on the X axis. Either "norm" (L1-norm), "lambda" (log-lambda sequence) or "dev" (percent deviance)}

\item{label.n}{Number of Df labels}

\item{label}{Logical value whether to display labels}

\item{label.label}{Column name used for label text}

\item{label.colour}{Colour for text labels}

\item{label.alpha}{Alpha for text labels}

\item{label.size}{Size for text labels}

\item{label.angle}{Angle for text labels}

\item{label.family}{Font family for text labels}

\item{label.fontface}{Fontface for text labels}

\item{label.lineheight}{Lineheight for text labels}

\item{label.hjust}{Horizontal adjustment for text labels}

\item{label.vjust}{Vertical adjustment for text labels}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{glmnet::glmnet}
}
\examples{
autoplot(glmnet::glmnet(data.matrix(Orange[-3]), data.matrix(Orange[3])))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats_density.R
\name{ggdistribution}
\alias{ggdistribution}
\title{Plot distribution}
\usage{
ggdistribution(
  func,
  x,
  p = NULL,
  colour = "#000000",
  linetype = NULL,
  fill = NULL,
  alpha = NULL,
  ...
)
}
\arguments{
\item{func}{PDF or CDF function}

\item{x}{Numeric vector to be passed to func}

\item{p}{\code{ggplot2::ggplot} instance to plot}

\item{colour}{Line colour}

\item{linetype}{Line type}

\item{fill}{Fill colour}

\item{alpha}{Alpha}

\item{...}{Keywords passed to PDC/CDF func}
}
\value{
ggplot
}
\description{
Plot distribution
}
\examples{
ggdistribution(dnorm, seq(-3, 3, 0.1), mean = 0, sd = 1)
ggdistribution(ppois, seq(0, 30), lambda = 20)

p <- ggdistribution(pchisq, 0:20, df = 7, fill = 'blue')
ggdistribution(pchisq, 0:20, p = p, df = 9, fill = 'red')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{apply_grid}
\alias{apply_grid}
\title{Apply grid to to \code{ggplot2::ggplot}}
\usage{
apply_grid(p, formula, scales = "free_y", ...)
}
\arguments{
\item{p}{\code{ggplot2::ggplot} instance}

\item{formula}{\code{stats::formula} instance}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{...}{other arguments passed to methods}
}
\description{
Apply grid to to \code{ggplot2::ggplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_vars.R
\name{autoplot.varprd}
\alias{autoplot.varprd}
\title{Autoplot \code{vars::varprd}}
\usage{
\method{autoplot}{varprd}(
  object,
  is.date = NULL,
  ts.connect = TRUE,
  scales = "free_y",
  predict.geom = "line",
  predict.colour = "#0000FF",
  predict.size = NULL,
  predict.linetype = NULL,
  predict.alpha = NULL,
  predict.fill = NULL,
  predict.shape = NULL,
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "none",
  conf.int.fill = "#000000",
  conf.int.alpha = 0.3,
  ...
)
}
\arguments{
\item{object}{\code{vars::varpred} instance}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{predict.geom}{geometric string for predicted time-series}

\item{predict.colour}{line colour for predicted time-series}

\item{predict.size}{point size for predicted time-series}

\item{predict.linetype}{line type for predicted time-series}

\item{predict.alpha}{alpha for predicted time-series}

\item{predict.fill}{fill colour for predicted time-series}

\item{predict.shape}{point shape for predicted time-series}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{...}{other arguments passed to \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot \code{vars::varprd}
}
\examples{
data(Canada, package = 'vars')
d.var <- vars::VAR(Canada, p = 3, type = 'const')
autoplot(stats::predict(d.var, n.ahead = 50), is.date = TRUE)
autoplot(stats::predict(d.var, n.ahead = 50), conf.int = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{plot_confint}
\alias{plot_confint}
\title{Attach confidence interval to \code{ggplot2::ggplot}}
\usage{
plot_confint(
  p,
  data = NULL,
  lower = "lower",
  upper = "upper",
  conf.int = TRUE,
  conf.int.geom = "line",
  conf.int.group = NULL,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "none",
  conf.int.fill = "#000000",
  conf.int.alpha = 0.3
)
}
\arguments{
\item{p}{\code{ggplot2::ggplot} instance}

\item{data}{data contains lower and upper confidence intervals}

\item{lower}{column name for lower confidence interval}

\item{upper}{column name for upper confidence interval}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.geom}{geometric string for confidence interval. 'line' or 'step'}

\item{conf.int.group}{name of grouping variable for confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}
}
\value{
ggplot
}
\description{
Attach confidence interval to \code{ggplot2::ggplot}
}
\examples{
d <- fortify(stats::acf(AirPassengers, plot = FALSE))
p <- ggplot(data = d, mapping = aes(x = Lag))
ggfortify:::plot_confint(p, data = d)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_cluster.R
\name{autoplot.kmeans}
\alias{autoplot.kmeans}
\title{Autoplot cluster instances}
\usage{
\method{autoplot}{kmeans}(object, data = NULL, colour = "cluster", ...)
}
\arguments{
\item{object}{Clustered instance}

\item{data}{Original data used for clustering. Mandatory for \code{stats::kmeans}.}

\item{colour}{line colour for points}

\item{...}{other arguments passed to \code{autoplot::prcomp}}
}
\value{
ggplot
}
\description{
Autoplot cluster instances
}
\examples{
autoplot(stats::kmeans(iris[-5], 3), data = iris)
autoplot(cluster::clara(iris[-5], 3), label = TRUE)
autoplot(cluster::fanny(iris[-5], 3))
autoplot(cluster::fanny(iris[-5], 3), frame = TRUE)
autoplot(cluster::pam(iris[-5], 3), data = iris, colour = 'Species')
autoplot(cluster::pam(iris[-5], 3), data = iris, frame = TRUE, frame.type = 't')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{is_derived_from}
\alias{is_derived_from}
\title{Check object is target class, or object is \code{data.frame} fortified from target.}
\usage{
is_derived_from(object, target)
}
\arguments{
\item{object}{instance to be checked. For data.frame, check whether it is fortified from target class}

\item{target}{class name}
}
\value{
logical
}
\description{
Check object is target class, or object is \code{data.frame} fortified from target.
}
\examples{
ggfortify:::is_derived_from(prcomp(iris[-5]), 'prcomp')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_base.R
\name{fortify.matrix}
\alias{fortify.matrix}
\title{Convert \code{base::matrix} to \code{data.frame}}
\usage{
\method{fortify}{matrix}(model, data = NULL, compat = FALSE, ...)
}
\arguments{
\item{model}{\code{base::matrix} instance}

\item{data}{original dataset, if needed}

\item{compat}{Logical frag to specify the behaviour when converting matrix which has no column name.
If \code{FALSE}, result has character columns like c('1', '2', ...).
If \code{TRUE}, result has character columns like c('V1', 'V2', ...).}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Different from \code{as.data.frame}
}
\examples{
fortify(matrix(1:6, nrow=2, ncol=3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_basis.R
\name{fortify.basis}
\alias{fortify.basis}
\title{Convert spline basis instances to \code{data.frame}}
\usage{
\method{fortify}{basis}(model, data, n = 256, ...)
}
\arguments{
\item{model}{spline basis object}

\item{data}{x-values at which to evaluate the splines. Optional. By
default, an evenly spaced sequence of 256 values covering the
range of the splines will be used.}

\item{n}{If data is not provided, instead use an evenly-spaced
sequence of x-values of this length (plus one, since both
endpoints are included). If data is provided, this argument is
ignored.}

\item{...}{other arguments passed to methods}
}
\value{
data.frame with 3 columns: Spline (character), x (numeric),
    and y (numeric); giving the interpolated x and y values for
    each of the splines in the basis.
}
\description{
Convert spline basis instances to \code{data.frame}
}
\examples{
library(splines)
x <- seq(0, 1, by=0.001)
spl <- bs(x, df=6)
fortify(spl)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{ggfreqplot}
\alias{ggfreqplot}
\title{Plot seasonal subseries of time series, generalization of \code{stats::monthplot}}
\usage{
ggfreqplot(
  data,
  freq = NULL,
  nrow = NULL,
  ncol = NULL,
  conf.int = FALSE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "dashed",
  conf.int.fill = NULL,
  conf.int.alpha = 0.3,
  conf.int.value = 0.95,
  facet.labeller = NULL,
  ...
)
}
\arguments{
\item{data}{\code{stats::ts} instance}

\item{freq}{Length of frequency. If not provided, use time-series frequency}

\item{nrow}{Number of plot rows}

\item{ncol}{Number of plot columns}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{conf.int.value}{Coverage probability for confidence interval}

\item{facet.labeller}{A vector used as facet labels}

\item{...}{Keywords passed to autoplot.ts}
}
\value{
ggplot
}
\description{
Plot seasonal subseries of time series, generalization of \code{stats::monthplot}
}
\examples{
ggfreqplot(AirPassengers)
ggfreqplot(AirPassengers, freq = 4)
ggfreqplot(AirPassengers, conf.int = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{fortify.acf}
\alias{fortify.acf}
\title{Convert \code{stats::acf} to \code{data.frame}}
\usage{
\method{fortify}{acf}(
  model,
  data = NULL,
  conf.int = TRUE,
  conf.int.value = 0.95,
  conf.int.type = "white",
  ...
)
}
\arguments{
\item{model}{\code{stats::acf} instance}

\item{data}{original dataset, if needed}

\item{conf.int}{Logical flag indicating whether to attach confidence intervals}

\item{conf.int.value}{Coverage probability for confidence interval}

\item{conf.int.type}{Type of confidence interval, 'white' for white noise or 'ma' MA(k-1) model}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{stats::acf} to \code{data.frame}
}
\examples{
fortify(stats::acf(AirPassengers))
fortify(stats::pacf(AirPassengers))
fortify(stats::ccf(AirPassengers, AirPassengers))

fortify(stats::acf(AirPassengers), conf.int = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_list.R
\name{fortify.list}
\alias{fortify.list}
\title{Convert list to data.frame}
\usage{
\method{fortify}{list}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{list} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert list to data.frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{plot_label}
\alias{plot_label}
\title{Attach label to \code{ggplot2::ggplot}}
\usage{
plot_label(
  p,
  data,
  x = NULL,
  y = NULL,
  label = TRUE,
  label.label = "rownames",
  label.colour = NULL,
  label.alpha = NULL,
  label.size = NULL,
  label.angle = NULL,
  label.family = NULL,
  label.fontface = NULL,
  label.lineheight = NULL,
  label.hjust = NULL,
  label.vjust = NULL,
  label.repel = FALSE,
  label.show.legend = NA,
  label.position = "identity"
)
}
\arguments{
\item{p}{\code{ggplot2::ggplot} instance}

\item{data}{Data contains text label}

\item{x}{x coordinates for label}

\item{y}{y coordinates for label}

\item{label}{Logical value whether to display labels}

\item{label.label}{Column name used for label text}

\item{label.colour}{Colour for text labels}

\item{label.alpha}{Alpha for text labels}

\item{label.size}{Size for text labels}

\item{label.angle}{Angle for text labels}

\item{label.family}{Font family for text labels}

\item{label.fontface}{Fontface for text labels}

\item{label.lineheight}{Lineheight for text labels}

\item{label.hjust}{Horizontal adjustment for text labels}

\item{label.vjust}{Vertical adjustment for text labels}

\item{label.repel}{Logical flag indicating whether to use \code{ggrepel}, enabling this may take some time for plotting}

\item{label.show.legend}{Logical value indicating whether to show the legend of the text labels}

\item{label.position}{Character or a position function}
}
\value{
ggplot
}
\description{
Attach label to \code{ggplot2::ggplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{get.dtindex}
\alias{get.dtindex}
\title{Convert \code{ts} index to \code{Date} \code{vector}}
\usage{
get.dtindex(data, is.tsp = FALSE, is.date = NULL)
}
\arguments{
\item{data}{\code{ts} instance}

\item{is.tsp}{Logical frag whether data is \code{tsp} itself or not}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}
}
\value{
vector
}
\description{
Convert \code{ts} index to \code{Date} \code{vector}
}
\examples{
ggfortify:::get.dtindex(AirPassengers)
ggfortify:::get.dtindex(UKgas)
ggfortify:::get.dtindex(Nile, is.date = FALSE)
ggfortify:::get.dtindex(Nile, is.date = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{fortify.factanal}
\alias{fortify.factanal}
\title{Convert \code{stats::factanal} to \code{data.frame}}
\usage{
\method{fortify}{factanal}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{stats::factanal} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{stats::factanal} to \code{data.frame}
}
\examples{
d.factanal <- stats::factanal(state.x77, factors = 3, scores = 'regression')
fortify(d.factanal)
fortify(d.factanal, data = state.x77)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{is.univariate}
\alias{is.univariate}
\title{Check if Validates number of \code{ts} variates}
\usage{
is.univariate(data, raise = TRUE)
}
\arguments{
\item{data}{\code{ts} instance}

\item{raise}{Logical flag whether raise an error}
}
\value{
logical
}
\description{
Check if Validates number of \code{ts} variates
}
\examples{
ggfortify:::is.univariate(AirPassengers)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{fortify.lfda}
\alias{fortify.lfda}
\title{Convert \code{lfda::lfda} or \code{lfda::klfda} or \code{lfda::self} to \code{data.frame}}
\usage{
\method{fortify}{lfda}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{lfda::lfda} or \code{lfda::klfda} or \code{lfda::self} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{lfda::lfda} or \code{lfda::klfda} or \code{lfda::self} to \code{data.frame}
}
\examples{
\dontrun{
model <- lfda::lfda(iris[, -5], iris[, 5], 3, metric = "plain")
fortify(model)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats_density.R
\name{autoplot.density}
\alias{autoplot.density}
\title{Autoplot \code{stats::density}}
\usage{
\method{autoplot}{density}(
  object,
  p = NULL,
  colour = "#000000",
  linetype = NULL,
  fill = NULL,
  alpha = NULL,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{stats::density} instance}

\item{p}{\code{ggplot2::ggplot} instance to plot}

\item{colour}{Line colour}

\item{linetype}{Line type}

\item{fill}{Fill colour}

\item{alpha}{Alpha}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to PDC/CDF func}
}
\value{
ggplot
}
\description{
Autoplot \code{stats::density}
}
\examples{
autoplot(stats::density(stats::rnorm(1:50)))
autoplot(stats::density(stats::rnorm(1:50)), fill = 'blue')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{post_fortify}
\alias{post_fortify}
\title{Post process for fortify.}
\usage{
post_fortify(data, klass = NULL)
}
\arguments{
\item{data}{data.frame}

\item{klass}{instance to be added as base_class attr, should be original model before fortified}
}
\value{
data.frame
}
\description{
Post process for fortify.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{ggcpgram}
\alias{ggcpgram}
\title{Plots a cumulative periodogram}
\usage{
ggcpgram(
  ts,
  taper = 0.1,
  colour = "#000000",
  linetype = "solid",
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "dashed",
  conf.int.fill = NULL,
  conf.int.alpha = 0.3
)
}
\arguments{
\item{ts}{\code{stats::ts} instance}

\item{taper}{Proportion tapered in forming the periodogram}

\item{colour}{Line colour}

\item{linetype}{Line type}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}
}
\value{
ggplot
}
\description{
Plots a cumulative periodogram
}
\examples{
ggcpgram(AirPassengers)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_surv.R
\name{autoplot.survfit}
\alias{autoplot.survfit}
\alias{autoplot.survfit.cox}
\title{Autoplot \code{survival::survfit}}
\usage{
\method{autoplot}{survfit}(
  object,
  fun = NULL,
  surv.geom = "step",
  surv.colour = NULL,
  surv.size = NULL,
  surv.linetype = NULL,
  surv.alpha = NULL,
  surv.fill = NULL,
  surv.shape = NULL,
  surv.connect = TRUE,
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "none",
  conf.int.fill = "#000000",
  conf.int.alpha = 0.3,
  censor = TRUE,
  censor.colour = NULL,
  censor.size = 3,
  censor.alpha = NULL,
  censor.shape = "+",
  facets = FALSE,
  nrow = NULL,
  ncol = 1,
  grid = FALSE,
  strip_swap = FALSE,
  scales = "free_y",
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{survival::survfit} instance}

\item{fun}{an arbitrary function defining a transformation of the survival curve}

\item{surv.geom}{geometric string for survival curve. 'step', 'line' or 'point'}

\item{surv.colour}{line colour for survival curve}

\item{surv.size}{point size for survival curve}

\item{surv.linetype}{line type for survival curve}

\item{surv.alpha}{alpha for survival curve}

\item{surv.fill}{fill colour survival curve}

\item{surv.shape}{point shape survival curve}

\item{surv.connect}{logical frag indicates whether connects survival curve to the origin}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{censor}{Logical flag indicating whether to plot censors}

\item{censor.colour}{colour for censors}

\item{censor.size}{size for censors}

\item{censor.alpha}{alpha for censors}

\item{censor.shape}{shape for censors}

\item{facets}{Logical value to specify use facets}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{grid}{Logical flag indicating whether to draw grid}

\item{strip_swap}{swap facet or grid strips}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{survival::survfit}
}
\examples{
library(survival)
autoplot(survfit(Surv(time, status) ~ sex, data = lung))
autoplot(survfit(Surv(time, status) ~ sex, data = lung), facets = TRUE)
autoplot(survfit(Surv(time, status) ~ 1, data = lung))
autoplot(survfit(Surv(time, status) ~ sex, data=lung), conf.int = FALSE, censor = FALSE)
autoplot(survfit(coxph(Surv(time, status) ~ sex, data = lung)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_vars.R
\name{fortify.varprd}
\alias{fortify.varprd}
\title{Convert \code{vars::varprd} to \code{data.frame}}
\usage{
\method{fortify}{varprd}(
  model,
  data = NULL,
  is.date = NULL,
  ts.connect = FALSE,
  melt = FALSE,
  ...
)
}
\arguments{
\item{model}{\code{vars::varprd} instance}

\item{data}{original dataset, if needed}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{melt}{Logical flag indicating whether to melt each timeseries as variable}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{vars::varprd} to \code{data.frame}
}
\examples{
data(Canada, package = 'vars')
d.var <- vars::VAR(Canada, p = 3, type = 'const')
fortify(stats::predict(d.var, n.ahead = 50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{fortify.stepfun}
\alias{fortify.stepfun}
\title{Convert \code{stats::stepfun} to \code{data.frame}}
\usage{
\method{fortify}{stepfun}(model, data, ...)
}
\arguments{
\item{model}{\code{stats::stepfun} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{stats::stepfun} to \code{data.frame}
}
\examples{
fortify(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
fortify(stepfun(c(1), c(4, 5)))
fortify(stepfun(c(1, 3, 4, 8), c(4, 5, 2, 3, 5)))
fortify(stepfun(c(1, 2, 3, 4, 5, 6, 7, 8, 10), c(4, 5, 6, 7, 8, 9, 10, 11, 12, 9)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_maps.R
\name{fortify_map}
\alias{fortify_map}
\title{Convert \code{maps::map} to \code{data.frame}.}
\usage{
fortify_map(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{maps::map} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{maps::map} to \code{data.frame}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_cluster.R
\name{fortify.kmeans}
\alias{fortify.kmeans}
\title{Convert cluster instances to \code{data.frame}}
\usage{
\method{fortify}{kmeans}(model, data = NULL, ...)
}
\arguments{
\item{model}{Clustered instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert cluster instances to \code{data.frame}
}
\examples{
fortify(stats::kmeans(iris[-5], 3))
fortify(stats::kmeans(iris[-5], 3), data = iris)
fortify(cluster::clara(iris[-5], 3))
fortify(cluster::fanny(iris[-5], 3))
fortify(cluster::pam(iris[-5], 3), data = iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{ggtsdiag}
\alias{ggtsdiag}
\title{Plots time-series diagnostics}
\usage{
ggtsdiag(
  object,
  gof.lag = 10,
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "dashed",
  conf.int.fill = NULL,
  conf.int.alpha = 0.3,
  ad.colour = "#888888",
  ad.linetype = "dashed",
  ad.size = 0.2,
  nrow = NULL,
  ncol = 1,
  ...
)
}
\arguments{
\item{object}{A fitted time-series model}

\item{gof.lag}{The maximum number of lags for a Portmanteau goodness-of-fit test}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{ad.colour}{Line colour for additional lines}

\item{ad.linetype}{Line type for additional lines}

\item{ad.size}{Fill colour for additional lines}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{...}{other keywords}
}
\value{
ggplot
}
\description{
Plots time-series diagnostics
}
\examples{
ggtsdiag(arima(AirPassengers))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{ggbiplot}
\alias{ggbiplot}
\title{Draw \code{biplot} using \code{ggplot2}.}
\usage{
ggbiplot(
  plot.data,
  loadings.data = NULL,
  colour = NULL,
  size = NULL,
  linetype = NULL,
  alpha = NULL,
  fill = NULL,
  shape = NULL,
  label = FALSE,
  label.label = "rownames",
  label.colour = colour,
  label.alpha = NULL,
  label.size = NULL,
  label.angle = NULL,
  label.family = NULL,
  label.fontface = NULL,
  label.lineheight = NULL,
  label.hjust = NULL,
  label.vjust = NULL,
  label.repel = FALSE,
  label.position = "identity",
  loadings = FALSE,
  loadings.arrow = grid::arrow(length = grid::unit(8, "points")),
  loadings.colour = "#FF0000",
  loadings.label = FALSE,
  loadings.label.label = "rownames",
  loadings.label.colour = "#FF0000",
  loadings.label.alpha = NULL,
  loadings.label.size = NULL,
  loadings.label.angle = NULL,
  loadings.label.family = NULL,
  loadings.label.fontface = NULL,
  loadings.label.lineheight = NULL,
  loadings.label.hjust = NULL,
  loadings.label.vjust = NULL,
  loadings.label.repel = FALSE,
  label.show.legend = NA,
  frame = FALSE,
  frame.type = NULL,
  frame.colour = colour,
  frame.level = 0.95,
  frame.alpha = 0.2,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{plot.data}{data.frame}

\item{loadings.data}{data.frame}

\item{colour}{colour}

\item{size}{size}

\item{linetype}{line type}

\item{alpha}{alpha}

\item{fill}{fill}

\item{shape}{shape}

\item{label}{Logical value whether to display data labels}

\item{label.label}{Column name used for label text}

\item{label.colour}{Colour for text labels}

\item{label.alpha}{Alpha for text labels}

\item{label.size}{Size for text labels}

\item{label.angle}{Angle for text labels}

\item{label.family}{Font family for text labels}

\item{label.fontface}{Fontface for text labels}

\item{label.lineheight}{Lineheight for text labels}

\item{label.hjust}{Horizontal adjustment for text labels}

\item{label.vjust}{Vertical adjustment for text labels}

\item{label.repel}{Logical flag indicating whether to use \code{ggrepel}, enabling this may take some time for plotting}

\item{label.position}{Character or a position function}

\item{loadings}{Logical value whether to display loadings arrows}

\item{loadings.arrow}{An arrow definition}

\item{loadings.colour}{Point colour for data}

\item{loadings.label}{Logical value whether to display loadings labels}

\item{loadings.label.label}{Column name used for loadings text labels}

\item{loadings.label.colour}{Colour for loadings text labels}

\item{loadings.label.alpha}{Alpha for loadings text labels}

\item{loadings.label.size}{Size for loadings text labels}

\item{loadings.label.angle}{Angle for loadings text labels}

\item{loadings.label.family}{Font family for loadings text labels}

\item{loadings.label.fontface}{Fontface for loadings text labels}

\item{loadings.label.lineheight}{Lineheight for loadings text labels}

\item{loadings.label.hjust}{Horizontal adjustment for loadings text labels}

\item{loadings.label.vjust}{Vertical adjustment for loadings text labels}

\item{loadings.label.repel}{Logical flag indicating whether to use \code{ggrepel} automatically}

\item{label.show.legend}{Logical value indicating whether to show the legend of text labels}

\item{frame}{Logical value whether to draw outliner convex / ellipse}

\item{frame.type}{Character specifying frame type.
'convex' or types supporeted by \code{ggplot2::stat_ellipse} can be used.}

\item{frame.colour}{Colour for frame}

\item{frame.level}{Passed for \code{ggplot2::stat_ellipse} 's level. Ignored in 'convex'.}

\item{frame.alpha}{Alpha for frame}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Draw \code{biplot} using \code{ggplot2}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_forecast.R
\name{fortify.forecast}
\alias{fortify.forecast}
\title{Convert \code{forecast::forecast} to \code{data.frame}}
\usage{
\method{fortify}{forecast}(model, data = NULL, is.date = NULL, ts.connect = FALSE, ...)
}
\arguments{
\item{model}{\code{forecast::forecast} instance}

\item{data}{original dataset, if needed}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{forecast::forecast} to \code{data.frame}
}
\examples{
d.arima <- forecast::auto.arima(AirPassengers)
d.forecast <- forecast::forecast(d.arima, level = c(95), h = 50)
fortify(d.forecast)
fortify(d.forecast, ts.connect = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{unscale}
\alias{unscale}
\title{Backtransform \code{scale}-ed object}
\usage{
unscale(data, center = NULL, scale = NULL)
}
\arguments{
\item{data}{Scaled data}

\item{center}{Centered vector}

\item{scale}{Scale vector}
}
\value{
data.frame
}
\description{
Backtransform \code{scale}-ed object
}
\examples{
df <- iris[-5]
ggfortify::unscale(base::scale(df))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{deprecate.warning}
\alias{deprecate.warning}
\title{Show deprecate warning}
\usage{
deprecate.warning(old.kw, new.kw)
}
\arguments{
\item{old.kw}{Keyword being deprecated}

\item{new.kw}{Keyword being replaced}
}
\description{
Show deprecate warning
}
\examples{
ggfortify:::deprecate.warning('old', 'new')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_list.R
\name{autoplot.list}
\alias{autoplot.list}
\title{Autoplot list}
\usage{
\method{autoplot}{list}(object, data = NULL, nrow = NULL, ncol = NULL, scales = "free_y", ...)
}
\arguments{
\item{object}{\code{list} instance}

\item{data}{original dataset, if needed}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats_lm.R
\name{autoplot.lm}
\alias{autoplot.lm}
\title{Autoplot \code{stats::lm} and \code{stats::glm}}
\usage{
\method{autoplot}{lm}(
  object,
  which = c(1:3, 5),
  data = NULL,
  colour = "#444444",
  size = NULL,
  linetype = NULL,
  alpha = NULL,
  fill = NULL,
  shape = NULL,
  label = TRUE,
  label.label = ".label",
  label.colour = "#000000",
  label.alpha = NULL,
  label.size = NULL,
  label.angle = NULL,
  label.family = NULL,
  label.fontface = NULL,
  label.lineheight = NULL,
  label.hjust = NULL,
  label.vjust = NULL,
  label.repel = FALSE,
  label.n = 3,
  smooth.colour = "#0000FF",
  smooth.linetype = "solid",
  ad.colour = "#888888",
  ad.linetype = "dashed",
  ad.size = 0.2,
  nrow = NULL,
  ncol = NULL,
  ...
)
}
\arguments{
\item{object}{\code{stats::lm} instance}

\item{which}{If a subset of the plots is required, specify a subset of the numbers 1:6.}

\item{data}{original dataset, if needed}

\item{colour}{line colour}

\item{size}{point size}

\item{linetype}{line type}

\item{alpha}{alpha}

\item{fill}{fill colour}

\item{shape}{point shape}

\item{label}{Logical value whether to display labels}

\item{label.label}{Column name used for label text}

\item{label.colour}{Colour for text labels}

\item{label.alpha}{Alpha for text labels}

\item{label.size}{Size for text labels}

\item{label.angle}{Angle for text labels}

\item{label.family}{Font family for text labels}

\item{label.fontface}{Fontface for text labels}

\item{label.lineheight}{Lineheight for text labels}

\item{label.hjust}{Horizontal adjustment for text labels}

\item{label.vjust}{Vertical adjustment for text labels}

\item{label.repel}{Logical flag indicating whether to use \code{ggrepel}, enabling this may take some time for plotting}

\item{label.n}{Number of points to be laeled in each plot, starting with the most extreme}

\item{smooth.colour}{Line colour for smoother lines}

\item{smooth.linetype}{Line type for smoother lines}

\item{ad.colour}{Line colour for additional lines}

\item{ad.linetype}{Line type for additional lines}

\item{ad.size}{Fill colour for additional lines}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{stats::lm} and \code{stats::glm}
}
\examples{
\dontrun{
autoplot(lm(Petal.Width ~ Petal.Length, data = iris))
autoplot(glm(Petal.Width ~ Petal.Length, data = iris), which = 1:6)
autoplot(lm(Petal.Width~Petal.Length, data = iris), data = iris, colour = 'Species')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggfortify.R
\docType{package}
\name{ggfortify}
\alias{ggfortify-package}
\alias{ggfortify}
\title{ggfortify}
\description{
Define Fortify and Autoplot to Allow 'ggplot2' to Draw Some Popular Packages
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/sinhrks/ggfortify}
  \item Report bugs at \url{https://github.com/sinhrks/ggfortify/issues}
}

}
\author{
\strong{Maintainer}: Yuan Tang \email{terrytangyuan@gmail.com} (\href{https://orcid.org/0000-0001-5243-233X}{ORCID})

Authors:
\itemize{
  \item Masaaki Horikoshi \email{sinhrks@gmail.com}
}

Other contributors:
\itemize{
  \item Austin Dickey [contributor]
  \item Matthias Grenié [contributor]
  \item Ryan Thompson [contributor]
  \item Luciano Selzer [contributor]
  \item Dario Strbenac [contributor]
  \item Kirill Voronin [contributor]
  \item Damir Pulatov [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{confint.acf}
\alias{confint.acf}
\title{Calculate confidence interval for \code{stats::acf}}
\usage{
\method{confint}{acf}(x, ci = 0.95, ci.type = "white")
}
\arguments{
\item{x}{\code{stats::acf} instance}

\item{ci}{Float value for confidence interval}

\item{ci.type}{"white" or "ma"}
}
\value{
vector
}
\description{
Calculate confidence interval for \code{stats::acf}
}
\examples{
air.acf <- acf(AirPassengers, plot = FALSE)
ggfortify:::confint.acf(air.acf)
ggfortify:::confint.acf(air.acf, ci.type = 'ma')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{residuals.ar}
\alias{residuals.ar}
\title{Calculate residuals for \code{stats::ar}}
\usage{
\method{residuals}{ar}(object, ...)
}
\arguments{
\item{object}{\code{stats::ar} instance}

\item{...}{other keywords}
}
\value{
ts Residuals extracted from the object object.
}
\description{
Calculate residuals for \code{stats::ar}
}
\examples{
residuals(ar(WWWusage))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_changepoint.R
\name{autoplot.breakpoints}
\alias{autoplot.breakpoints}
\alias{autoplot.breakpointsfull}
\title{Autoplot \code{strucchange::breakpoints}}
\usage{
\method{autoplot}{breakpoints}(
  object,
  data = NULL,
  cpt.colour = "#FF0000",
  cpt.linetype = "dashed",
  ...
)
}
\arguments{
\item{object}{\code{strucchange::breakpoints} or \code{strucchange::breakpointsfull} instance.}

\item{data}{Original time series. Mandatory for plotting \code{strucchange::breakpoints} instance.}

\item{cpt.colour}{Line colour for changepoints}

\item{cpt.linetype}{Line type for changepoints}

\item{...}{other arguments passed to \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot \code{strucchange::breakpoints}
}
\examples{
library(strucchange)
bp.nile <- breakpoints(Nile ~ 1)
autoplot(bp.nile)
autoplot(bp.nile, is.date = TRUE)
autoplot(breakpoints(bp.nile, breaks = 2), data = Nile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{fortify.dist}
\alias{fortify.dist}
\title{Convert \code{stats::dist} to \code{data.frame}}
\usage{
\method{fortify}{dist}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{stats::dist} instance}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{stats::dist} to \code{data.frame}
}
\examples{
fortify(eurodist)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_ts.R
\name{fortify.tsmodel}
\alias{fortify.tsmodel}
\alias{fortify.ar}
\alias{fortify.Arima}
\alias{fortify.fracdiff}
\alias{fortify.nnetar}
\alias{fortify.HoltWinters}
\alias{fortify.fGARCH}
\title{Convert time series models (like AR, ARIMA) to \code{data.frame}}
\usage{
\method{fortify}{tsmodel}(
  model,
  data = NULL,
  predict = NULL,
  is.date = NULL,
  ts.connect = TRUE,
  ...
)
}
\arguments{
\item{model}{Time series model instance}

\item{data}{original dataset, needed for \code{stats::ar}, \code{stats::Arima}}

\item{predict}{Predicted \code{stats::ts}
If not provided, try to retrieve from current environment using variable name.}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{ts.connect}{Logical frag indicates whether connects original time-series and predicted values}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert time series models (like AR, ARIMA) to \code{data.frame}
}
\examples{
fortify(stats::ar(AirPassengers))
fortify(stats::arima(UKgas))
fortify(stats::arima(UKgas), data = UKgas, is.date = TRUE)
fortify(forecast::auto.arima(austres))
fortify(forecast::arfima(AirPassengers))
fortify(forecast::nnetar(UKgas))
fortify(stats::HoltWinters(USAccDeaths))

data(LPP2005REC, package = 'timeSeries')
x = timeSeries::as.timeSeries(LPP2005REC)
d.Garch = fGarch::garchFit(LPP40 ~ garch(1, 1), data = 100 * x, trace = FALSE)
fortify(d.Garch)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{autoplot.stepfun}
\alias{autoplot.stepfun}
\title{Plot \code{stats::stepfun}}
\usage{
\method{autoplot}{stepfun}(
  object,
  colour = NULL,
  size = NULL,
  linetype = NULL,
  alpha = NULL,
  shape = 1,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{stats::stepfun} instance}

\item{colour}{colour}

\item{size}{point size}

\item{linetype}{line type}

\item{alpha}{alpha}

\item{shape}{point shape}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Plot \code{stats::stepfun}
}
\examples{
autoplot(stepfun(c(1, 2, 3), c(4, 5, 6, 7)))
autoplot(stepfun(c(1), c(4, 5)), shape = NULL)
autoplot(stepfun(c(1, 3, 4, 8), c(4, 5, 2, 3, 5)), linetype = 'dashed')
autoplot(stepfun(c(1, 2, 3, 4, 5, 6, 7, 8, 10), c(4, 5, 6, 7, 8, 9, 10, 11, 12, 9)), colour = 'red')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_base.R
\name{autoplot.matrix}
\alias{autoplot.matrix}
\title{Plot \code{base::matrix}}
\usage{
\method{autoplot}{matrix}(
  object,
  original = NULL,
  geom = "tile",
  colour = NULL,
  size = NULL,
  alpha = NULL,
  fill = "#0000FF",
  shape = NULL,
  label = FALSE,
  label.label = "rownames",
  label.colour = colour,
  label.alpha = NULL,
  label.size = NULL,
  label.angle = NULL,
  label.family = NULL,
  label.fontface = NULL,
  label.lineheight = NULL,
  label.hjust = NULL,
  label.vjust = NULL,
  label.repel = FALSE,
  scale = NULL,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{base::matrix} instance}

\item{original}{Combined to data by column if provided. Intended to be used for stat functions which
returns not containing original data.}

\item{geom}{Geometric string for plotting. 'tile' or 'point'.}

\item{colour}{colour for points ('point' only)}

\item{size}{point size}

\item{alpha}{alpha}

\item{fill}{fill colour. Ignored if scale keyword is passed. ('tile' Only)}

\item{shape}{point shape}

\item{label}{Logical value whether to display labels}

\item{label.label}{Column name used for label text}

\item{label.colour}{Colour for text labels}

\item{label.alpha}{Alpha for text labels}

\item{label.size}{Size for text labels}

\item{label.angle}{Angle for text labels}

\item{label.family}{Font family for text labels}

\item{label.fontface}{Fontface for text labels}

\item{label.lineheight}{Lineheight for text labels}

\item{label.hjust}{Horizontal adjustment for text labels}

\item{label.vjust}{Vertical adjustment for text labels}

\item{label.repel}{Logical flag indicating whether to use \code{ggrepel}, enabling this may take some time for plotting}

\item{scale}{(Deprecated) \code{ggplot2::scale} instance to plot. ('tile' Only)}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Plot \code{base::matrix}
}
\examples{
autoplot(matrix(rnorm(20), nc = 5))
autoplot(matrix(rnorm(20), nc = 5), fill = 'red')
autoplot(matrix(rnorm(20), nc = 2), geom = 'point')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_basis.R
\name{autoplot.basis}
\alias{autoplot.basis}
\title{Autoplot spline basis instances}
\usage{
\method{autoplot}{basis}(object, data, n = 256, ...)
}
\arguments{
\item{object}{spline basis object}

\item{data}{x-values at which to evaluate the splines. Optional. By
default, an evenly spaced sequence of 256 values covering the
range of the splines will be used.}

\item{n}{If data is not provided, instead use an evenly-spaced
sequence of x-values of this length (plus one, since both
endpoints are included). If data is provided, this argument is
ignored.}

\item{...}{Ignored.}
}
\value{
ggplot
}
\description{
Autoplot spline basis instances
}
\examples{
library(splines)
x <- seq(0, 1, by=0.001)
spl <- bs(x, df=6)
autoplot(spl)
autoplot(spl, n=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_surv.R
\name{autoplot.aareg}
\alias{autoplot.aareg}
\title{Autoplot \code{survival::aareg}}
\usage{
\method{autoplot}{aareg}(
  object,
  maxtime = NULL,
  surv.connect = TRUE,
  facets = TRUE,
  ncol = NULL,
  xlab = "",
  ylab = "",
  ...
)
}
\arguments{
\item{object}{\code{survival::aareg} instance}

\item{maxtime}{truncate the input to the model at time "maxtime"}

\item{surv.connect}{logical frag indicates whether connects survival curve to the origin}

\item{facets}{Logical value to specify use facets}

\item{ncol}{Number of facet/subplot columns}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{...}{other arguments passed to \code{autoplot.survfit}}
}
\value{
ggplot
}
\description{
Autoplot \code{survival::aareg}
}
\examples{
library(survival)
autoplot(aareg(Surv(time, status) ~ age + sex + ph.ecog, data = lung, nmin = 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{autoplot.spec}
\alias{autoplot.spec}
\title{Autoplot \code{stats::spec}}
\usage{
\method{autoplot}{spec}(
  object,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "y",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{stats::spec} instance}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{stats::spec}
}
\examples{
autoplot(stats::spec.ar(AirPassengers))
autoplot(stats::spec.pgram(AirPassengers))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_cluster.R
\name{autoplot.silhouette}
\alias{autoplot.silhouette}
\title{Autoplot silhouette instances}
\usage{
\method{autoplot}{silhouette}(
  object,
  colour = "red",
  linetype = "dashed",
  size = 0.5,
  bar.width = 1,
  ...
)
}
\arguments{
\item{object}{Silhouette instance}

\item{colour}{reference line color}

\item{linetype}{reference line type}

\item{size}{reference line size}

\item{bar.width}{bar width}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot silhouette instances
}
\examples{
model = cluster::pam(iris[-5], 3L)
sil = cluster::silhouette(model)
autoplot(sil)

autoplot(cluster::silhouette(cluster::clara(iris[-5], 3)))
autoplot(cluster::silhouette(cluster::fanny(iris[-5], 3)))

model = stats::kmeans(iris[-5], 3)
sil = cluster::silhouette(model$cluster, stats::dist(iris[-5]))
autoplot(sil)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\docType{class}
\name{ggmultiplot-class}
\alias{ggmultiplot-class}
\alias{length,ggmultiplot-method}
\alias{[,ggmultiplot,ANY,ANY,ANY-method}
\alias{[,ggmultiplot-method}
\alias{[[,ggmultiplot-method}
\alias{[<-,ggmultiplot,ANY,ANY,ANY-method}
\alias{[<-,ggmultiplot-method}
\alias{[[<-,ggmultiplot-method}
\title{An S4 class to hold multiple \code{ggplot2::ggplot} instances}
\usage{
\S4method{length}{ggmultiplot}(x)

\S4method{[}{ggmultiplot,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)

\S4method{[[}{ggmultiplot}(x, i, j, ..., drop)

\S4method{[}{ggmultiplot,ANY,ANY,ANY}(x, i, j, ...) <- value

\S4method{[[}{ggmultiplot}(x, i, j, ...) <- value
}
\arguments{
\item{x}{\code{ggmultiplot}}

\item{i}{elements to extract or replace}

\item{j}{not used}

\item{...}{not used}

\item{drop}{not used}

\item{value}{value to be set}
}
\description{
An S4 class to hold multiple \code{ggplot2::ggplot} instances
}
\section{Slots}{

\describe{
\item{\code{plots}}{List of \code{ggplot2::ggplot} instances}

\item{\code{ncol}}{Number of grid columns}

\item{\code{nrow}}{Number of grid rows}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_MSwM.R
\name{autoplot.MSM.lm}
\alias{autoplot.MSM.lm}
\title{Autoplot \code{MSwM::MSM.lm}}
\usage{
\method{autoplot}{MSM.lm}(object, prob.colour = "#FF0000", prob.linetype = "dashed", ...)
}
\arguments{
\item{object}{\code{MSwM::MSM.lm} instance}

\item{prob.colour}{Line colour for probabilities}

\item{prob.linetype}{Line type for probabilities}

\item{...}{other arguments passed to \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot \code{MSwM::MSM.lm}
}
\examples{
\dontrun{
library(MSwM)
d <- data.frame(Data = c(rnorm(50, mean = -10), rnorm(50, mean = 10)),
                exog = cos(seq(-pi/2, pi/2, length.out = 100)))
d.mswm <- MSwM::msmFit(lm(Data ~.-1, data = d), k=2, sw=rep(TRUE, 2),
                       control = list(parallelization = FALSE))
autoplot(d.mswm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_stats.R
\name{autoplot.acf}
\alias{autoplot.acf}
\title{Autoplot \code{stats::acf}. Note to pass `plot = FALSE` to original function to suppress
standard plot output}
\usage{
\method{autoplot}{acf}(
  object,
  colour = "#000000",
  linetype = "solid",
  conf.int = TRUE,
  conf.int.colour = "#0000FF",
  conf.int.linetype = "dashed",
  conf.int.fill = NULL,
  conf.int.alpha = 0.3,
  conf.int.value = 0.95,
  conf.int.type = "white",
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = "ACF",
  asp = NULL,
  ...
)
}
\arguments{
\item{object}{\code{stats::acf} instance}

\item{colour}{Line colour}

\item{linetype}{Line type}

\item{conf.int}{Logical flag indicating whether to plot confidence intervals}

\item{conf.int.colour}{line colour for confidence intervals}

\item{conf.int.linetype}{line type for confidence intervals}

\item{conf.int.fill}{fill colour for confidence intervals}

\item{conf.int.alpha}{alpha for confidence intervals}

\item{conf.int.value}{Coverage probability for confidence interval}

\item{conf.int.type}{Type of confidence interval, 'white' for white noise or 'ma' MA(k-1) model}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Autoplot \code{stats::acf}. Note to pass `plot = FALSE` to original function to suppress
standard plot output
}
\examples{
autoplot(stats::acf(AirPassengers, plot = FALSE))
autoplot(stats::pacf(AirPassengers, plot = FALSE))
autoplot(stats::ccf(AirPassengers, AirPassengers, plot = FALSE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_performance.R
\name{fortify.performance}
\alias{fortify.performance}
\title{Convert \code{ROCR::performance} objects to \code{data.frame}}
\usage{
\method{fortify}{performance}(model, data = NULL, ...)
}
\arguments{
\item{model}{\code{performance} instances}

\item{data}{original dataset, if needed}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{ROCR::performance} objects to \code{data.frame}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{flatten}
\alias{flatten}
\title{Flatten dataframe contains matrix}
\usage{
flatten(df)
}
\arguments{
\item{df}{\code{data.frame} to be flatten}
}
\description{
tains list or matrix as column
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{cbind_wraps}
\alias{cbind_wraps}
\title{Wrapper for cbind}
\usage{
cbind_wraps(df1, df2)
}
\arguments{
\item{df1}{1st data}

\item{df2}{2nd data}
}
\value{
list
}
\description{
Wrapper for cbind
}
\examples{
ggfortify:::cbind_wraps(iris[1:2], iris[3:5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{print,ggmultiplot-method}
\alias{print,ggmultiplot-method}
\title{Generic print function for \code{ggmultiplot}}
\usage{
\S4method{print}{ggmultiplot}(x)
}
\arguments{
\item{x}{\code{ggmultiplot}}
}
\description{
Generic print function for \code{ggmultiplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_changepoint.R
\name{fortify.cpt}
\alias{fortify.cpt}
\alias{fortify.breakpointsfull}
\alias{fortify.breakpoints}
\title{Convert \code{changepoint::cpt} and \code{strucchange::breakpoints} to \code{data.frame}}
\usage{
\method{fortify}{cpt}(model, data = NULL, is.date = NULL, ...)
}
\arguments{
\item{model}{\code{chantepoint::cpt} or \code{strucchange::breakpoints} instance}

\item{data}{original dataset, if needed}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{changepoint::cpt} and \code{strucchange::breakpoints} to \code{data.frame}
}
\examples{
library(changepoint)
fortify(cpt.mean(AirPassengers))
fortify(cpt.var(AirPassengers))
fortify(cpt.meanvar(AirPassengers))

library(strucchange)
bp.nile <- breakpoints(Nile ~ 1)
fortify(bp.nile)
fortify(breakpoints(bp.nile, breaks = 2))
fortify(breakpoints(bp.nile, breaks = 2), data = Nile)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tslib.R
\name{fitted.ar}
\alias{fitted.ar}
\title{Calculate fitted values for \code{stats::ar}}
\usage{
\method{fitted}{ar}(object, ...)
}
\arguments{
\item{object}{\code{stats::ar} instance}

\item{...}{other keywords}
}
\value{
ts An time series of the one-step forecasts
}
\description{
Calculate fitted values for \code{stats::ar}
}
\examples{
fitted(ar(WWWusage))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{apply_facets}
\alias{apply_facets}
\title{Apply facets to to \code{ggplot2::ggplot}}
\usage{
apply_facets(
  p,
  formula,
  facets = TRUE,
  nrow = NULL,
  ncol = 1,
  scales = "free_y",
  ...
)
}
\arguments{
\item{p}{\code{ggplot2::ggplot} instance}

\item{formula}{\code{stats::formula} instance}

\item{facets}{Logical value to specify use facets}

\item{nrow}{Number of facet/subplot rows}

\item{ncol}{Number of facet/subplot columns}

\item{scales}{Scale value passed to \code{ggplot2}}

\item{...}{other arguments passed to methods}
}
\value{
ggplot
}
\description{
Apply facets to to \code{ggplot2::ggplot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{post_autoplot}
\alias{post_autoplot}
\title{Post process for fortify. Based on \code{ggplot2::qplot}}
\usage{
post_autoplot(
  p,
  xlim = c(NA, NA),
  ylim = c(NA, NA),
  log = "",
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  asp = NULL
)
}
\arguments{
\item{p}{\code{ggplot2::ggplot} instances}

\item{xlim}{limits for x axis}

\item{ylim}{limits for y axis}

\item{log}{which variables to log transform ("x", "y", or "xy")}

\item{main}{character vector or expression for plot title}

\item{xlab}{character vector or expression for x axis label}

\item{ylab}{character vector or expression for y axis label}

\item{asp}{the y/x aspect ratio}
}
\value{
data.frame
}
\description{
Post process for fortify. Based on \code{ggplot2::qplot}
}
\examples{
p <- qplot(Petal.Length, Petal.Width, data = iris)
ggfortify:::post_autoplot(p, xlim = c(1, 5), ylim = c(1, 5), log = 'xy', main = 'title',
                          xlab = 'x', ylab = 'y', asp = 1.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_fortify_ts.R
\name{fortify.ts}
\alias{fortify.ts}
\title{Convert time-series-like to data.frame}
\usage{
\method{fortify}{ts}(
  model,
  data = NULL,
  columns = NULL,
  is.date = NULL,
  index.name = "Index",
  data.name = "Data",
  scale = FALSE,
  melt = FALSE,
  ...
)
}
\arguments{
\item{model}{time-series-like instance}

\item{data}{original dataset, if needed}

\item{columns}{character vector specifies target column name(s)}

\item{is.date}{logical frag indicates whether the \code{stats::ts} is date or not
If not provided, regard the input as date when the frequency is 4 or 12}

\item{index.name}{specify column name for time series index}

\item{data.name}{specify column name for univariate time series data. Ignored in multivariate time series.}

\item{scale}{logical flag indicating whether to perform scaling each timeseries}

\item{melt}{logical flag indicating whether to melt each timeseries as variable}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert time-series-like to data.frame
}
\examples{
fortify(AirPassengers)
fortify(timeSeries::as.timeSeries(AirPassengers))
fortify(tseries::irts(cumsum(rexp(10, rate = 0.1)), matrix(rnorm(20), ncol=2)))
fortify(stats::stl(UKgas, s.window = 'periodic'))
fortify(stats::decompose(UKgas))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_MSwM.R
\name{fortify.MSM.lm}
\alias{fortify.MSM.lm}
\title{Convert \code{MSwM::MSM.lm} to \code{data.frame}}
\usage{
\method{fortify}{MSM.lm}(model, data = NULL, melt = FALSE, ...)
}
\arguments{
\item{model}{\code{MSwM::MSM.lm} instance}

\item{data}{original dataset, if needed}

\item{melt}{Logical flag indicating whether to melt each models}

\item{...}{other arguments passed to methods}
}
\value{
data.frame
}
\description{
Convert \code{MSwM::MSM.lm} to \code{data.frame}
}
\examples{
library(MSwM)
d <- data.frame(Data = c(rnorm(50, mean = -10), rnorm(50, mean = 10)),
                exog = cos(seq(-pi/2, pi/2, length.out = 100)))
d.mswm <- MSwM::msmFit(lm(Data ~.-1, data = d), k=2, sw=rep(TRUE, 2),
                       control = list(parallelization = FALSE))
fortify(d.mswm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{autoplot.ggmultiplot}
\alias{autoplot.ggmultiplot}
\title{Autoplot \code{ggmultiplot} instances.
It returns the passed instance as it is.}
\usage{
\method{autoplot}{ggmultiplot}(object, ...)
}
\arguments{
\item{object}{ggmultiplot instance}

\item{...}{Not used.}
}
\value{
ggmultiplot
}
\description{
Autoplot \code{ggmultiplot} instances.
It returns the passed instance as it is.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fortify_changepoint.R
\name{autoplot.cpt}
\alias{autoplot.cpt}
\title{Autoplot \code{changepoint::cpt}}
\usage{
\method{autoplot}{cpt}(
  object,
  is.date = NULL,
  cpt.colour = "#FF0000",
  cpt.linetype = "dashed",
  ...
)
}
\arguments{
\item{object}{\code{changepoint::cpt} instance}

\item{is.date}{Logical frag indicates whether the \code{stats::ts} is date or not.
If not provided, regard the input as date when the frequency is 4 or 12.}

\item{cpt.colour}{Line colour for changepoints}

\item{cpt.linetype}{Line type for changepoints}

\item{...}{other arguments passed \code{autoplot.ts}}
}
\value{
ggplot
}
\description{
Autoplot \code{changepoint::cpt}
}
\examples{
library(changepoint)
autoplot(cpt.mean(AirPassengers))
autoplot(cpt.meanvar(AirPassengers))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{geom_factory}
\alias{geom_factory}
\title{Factory function to control \code{ggplot2::geom_xxx} functions}
\usage{
geom_factory(geomfunc, data = NULL, position = NULL, ...)
}
\arguments{
\item{geomfunc}{\code{ggplot2::geom_xxx} function}

\item{data}{plotting data}

\item{position}{A position function or character}

\item{...}{other arguments passed to methods}
}
\value{
proto
}
\description{
Factory function to control \code{ggplot2::geom_xxx} functions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotlib.R
\name{show,ggmultiplot-method}
\alias{show,ggmultiplot-method}
\title{Generic show function for \code{ggmultiplot}}
\usage{
\S4method{show}{ggmultiplot}(object)
}
\arguments{
\item{object}{\code{ggmultiplot}}
}
\description{
Generic show function for \code{ggmultiplot}
}
